/***************************************************************************
 * Model changes from DDS 1.4 to DDS 5.1
 ***************************************************************************/
%MACRO migr8dds;

 options notes source source2;

/***************************************************************************
 * Edit the following macro variables to change the migration path
 ***************************************************************************
 *
 *   - _dttmfmt: Any valid SAS datetime format. (ex. DATETIME21.)
 *
 *   - _dtfmt: Any valid SAS date format. (ex. DATE9.)
 *
 *
 ***************************************************************************/

%let _dttmfmt=DATETIME21.;
%let _dtfmt=DATE9.;

%let _languageCode=en;
%let _languageDesc=English;
%let _locale=en_US;

%let stgprefix=;
%let _processTime=%sysfunc(datetime());

%bldLibNm(sesetl_sasLibRef=_CIDDS
        , sesetl_OMRSASLibraryObjName=&CrossIndustryDDStier_folder./&CrossIndustryDDSLibObj./&CrossIndustryDDSLibObj.
        , sesetl_useType=BASE,
          sesetl_validateLib=Y
        ) ;

/******************************************************************************/
%macro _migrate(_libref=);

  %let _datarep = WINDOWS_64;
  proc sql noprint;
    select datarepname into :_datarep
    from dictionary.tables
    where upcase(libname) eq upcase("&_libref")
      and datarepname = 'WINDOWS_32'
    ;
  quit;
  
  %if "&_datarep" eq "WINDOWS_64" %then %goto MACRO_END;

  %let _path=;
  PROC SQL noprint;
    select path into :_path
    from dictionary.libnames
    where upcase(libname) eq upcase("&_libref")
      and level=1
    ;
  QUIT;

  /* Backup existing tables in directory */
  %let _path = %sysfunc(strip(&_path));
  %if %str(&_path) eq %then
    %let _path = %sysfunc(pathname(&_libref));

  %let _path=%sysfunc(kcompress("&_path.", '()"'));
  %put Backup &_path....;

  %let _dt = %sysfunc(putn(%sysfunc(date()), date8.));
  %let _tm = %sysfunc(putn(%sysfunc(time()), time8.));
  %let _tm = %sysfunc(kcompress(&_tm, ':'));

  /* Paths for Windows and Unix are different */
  libname &_libref "&_path";
  options noxwait;
  %if &SYSSCP = WIN %then %do;
     x "md &_path.\backup_&_dt._&_tm.";
     x "copy &_path.\*.* &_path.\backup_&_dt._&_tm.";

     libname _backup "&_path.\backup_&_dt._&_tm.";
  %end;
  %else %do;
     x "mkdir &_path./backup_&_dt._&_tm.";
     x "cp &_path./*.* &_path./backup_&_dt._&_tm.";

     libname _backup "&_path./backup_&_dt._&_tm."
   %end;

  proc datasets lib=&_libref kill; run;

  proc migrate in=_backup out=&_libref; run;

  %MACRO_END:;
%mend _migrate;
%_migrate(_libref=_CIDDS);
/******************************************************************************/


/***************************************************************************
 * Modelchanges to table columns from DDS 1.4 to DDS 5.1
 ***************************************************************************/
%if not(%sysfunc( exist(_CIDDS.DDS_SOLUTIONS_VERSION) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.DDS_SOLUTIONS_VERSION (
       VERSION_NO            VARCHAR(32)  label='Version Number',
       SOLUTION_NM           VARCHAR(50)  label='Solution Name',
       VERSION_STATUS_DESC   VARCHAR(255) label='Version Status Description',
       LOAD_METHOD_DESC      VARCHAR(255) label='Load Method Description',
       PROCESSED_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed',
       CONSTRAINT PRIM_KEY PRIMARY KEY (VERSION_NO, SOLUTION_NM)
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/

%sessdver(solName=DDS, solVer=5.1, solStatus=Failed, solMethod=Migration);

/******************************************************************************/
/*               VALIDATION FOR NEW INTEGRITY CONSTRAINTS                     */
/******************************************************************************/

/* JOB_POSITION / INTERNAL_ORG_RK */
%let _nullCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_nullCount
  FROM _CIDDS.JOB_POSITION
  WHERE INTERNAL_ORG_RK IS NULL
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_nullCount gt 0 %then %do;
   %if not(%sysfunc( exist(_CIDDS.JOB_POSITION_BKUP) )) %then %do;
      PROC SQL;
        CREATE TABLE _CIDDS.JOB_POSITION_BKUP AS SELECT * FROM _CIDDS.JOB_POSITION;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   PROC SQL;
     DELETE FROM _CIDDS.JOB_POSITION WHERE INTERNAL_ORG_RK IS NULL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %put WARNING: One or more records contained a NULL value for INTERNAL_ORG_RK in the DDS JOB_POSITION table.
The records with these NULL values have been dropped from the JOB_POSITION table.  The JOB_POSITION_BKUP table
in the DDS is a backup of the original JOB_POSITION table;
%end;

/* COMPETENCY / COMPETENCY_ID */
%let _nullCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_nullCount
  FROM _CIDDS.COMPETENCY
  WHERE COMPETENCY_ID IS NULL
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_nullCount gt 0 %then %do;
   %if not(%sysfunc( exist(_CIDDS.COMPETENCY_BKUP) )) %then %do;
      PROC SQL;
        CREATE TABLE _CIDDS.COMPETENCY_BKUP AS SELECT * FROM _CIDDS.COMPETENCY;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   PROC SQL;
     DELETE FROM _CIDDS.COMPETENCY WHERE COMPETENCY_ID IS NULL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %put WARNING: One or more records contained a NULL value for COMPETENCY_ID in the DDS COMPETENCY table.
The records with these NULL values have been dropped from the COMPETENCY table.  The COMPETENCY_BKUP table
in the DDS is a backup of the original COMPETENCY table;
%end;

/* CURRENCY_EXCH_RATE / TIME_PERIOD_RK */
%let _nullCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_nullCount
  FROM _CIDDS.CURRENCY_EXCH_RATE
  WHERE TIME_PERIOD_RK IS NULL
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_nullCount gt 0 %then %do;
   %if not(%sysfunc( exist(_CIDDS.CURRENCY_EXCH_RATE_BKUP) )) %then %do;
      PROC SQL;
        CREATE TABLE _CIDDS.CURRENCY_EXCH_RATE_BKUP AS SELECT * FROM _CIDDS.CURRENCY_EXCH_RATE;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   PROC SQL;
     DELETE FROM _CIDDS.CURRENCY_EXCH_RATE WHERE TIME_PERIOD_RK IS NULL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %put WARNING: One or more records contained a NULL value for TIME_PERIOD_RK in the DDS CURRENCY_EXCH_RATE table.
The records with these NULL values have been dropped from the CURRENCY_EXCH_RATE table.  The CURRENCY_EXCH_RATE_BKUP table
in the DDS is a backup of the original CURRENCY_EXCH_RATE table;
%end;

/* COMPETENCY_ASSOC / COMPETENCY_ASSOC_TYPE_CD*/

%let _recCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_recCount
  FROM _CIDDS.COMPETENCY_ASSOC
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_recCount gt 0 %then %do;
   %if not(%sysfunc( exist(_CIDDS.COMPETENCY_ASSOC_BKUP) )) %then %do;
      PROC SQL;
        CREATE TABLE _CIDDS.COMPETENCY_ASSOC_BKUP AS SELECT * FROM _CIDDS.COMPETENCY_ASSOC;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   PROC SQL;
     DELETE FROM _CIDDS.COMPETENCY_ASSOC;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %put WARNING: The COMPETENCY_ASSOC_TYPE_CD column is being added to the primary key of the DDS COMPETENCY_ASSOC table.
All records have been dropped from the COMPETENCY_ASSOC table.  The COMPETENCY_ASSOC_BKUP table in the DDS is a backup of
the original COMPETENCY_ASSOC table;
%end;

/* EXPLOYEE_X_JOB_X_COMPETENCY / WEIGHT_TYPE_CD*/

%let _recCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_recCount
  FROM _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_recCount gt 0 %then %do;
   %if not(%sysfunc( exist(_CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY_BKUP) )) %then %do;
      PROC SQL;
        CREATE TABLE _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY_BKUP AS SELECT * FROM _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   PROC SQL;
     DELETE FROM _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %put WARNING: The WEIGHT_TYPE_CD column is being added to the primary key of the DDS EMPLOYEE_X_JOB_X_COMPETENCY table.
All records have been dropped from the EMPLOYEE_X_JOB_X_COMPETENCY table.  The EMPLOYEE_X_JOB_X_COMPETENCY_BKUP table in
the DDS is a backup of the original EMPLOYEE_X_JOB_X_COMPETENCY table;
%end;

/* JOB_X_COMPETENCY / WEIGHT_TYPE_CD*/

%let _recCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_recCount
  FROM _CIDDS.JOB_X_COMPETENCY
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_recCount gt 0 %then %do;
   %if not(%sysfunc( exist(_CIDDS.JOB_X_COMPETENCY_BKUP) )) %then %do;
      PROC SQL;
        CREATE TABLE _CIDDS.JOB_X_COMPETENCY_BKUP AS SELECT * FROM _CIDDS.JOB_X_COMPETENCY;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   PROC SQL;
     DELETE FROM _CIDDS.JOB_X_COMPETENCY;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %put WARNING: The WEIGHT_TYPE_CD column is being added to the primary key of the DDS JOB_X_COMPETENCY table.
All records have been dropped from the JOB_X_COMPETENCY table.  The JOB_X_COMPETENCY_BKUP table in the DDS is a
backup of the original JOB_X_COMPETENCY table;
%end;

/******************************************************************************/
%if not(%sysfunc( exist(_CIDDS.MISC_RATE) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.MISC_RATE (
          RATE_SET_RK          NUMERIC(10) NOT NULL label='Rate Set Key',
          RATE_TYPE_CD         VARCHAR(32) NOT NULL label='Rate Type Code',
          RATE_SOURCE_CD       VARCHAR(3) label='Rate Source Code',
          GL_ACCOUNT_RK        NUMERIC(10) label='GL Account Key',
          INTERNAL_ORG_RK      NUMERIC(10) label='Internal Organization Key',
          CURRENCY_CD          VARCHAR(3) label='Currency Code',
          COST_CENTER_RK       NUMERIC(10) label='Cost Center Key',
          PROFIT_CENTER_RK     NUMERIC(10) label='Profit Center Key',
          ITEM_CATEGORY_CD     VARCHAR(3) label='Item Category Code',
          TIME_PERIOD_RK       NUMERIC(10) label='Time Period Key',
          RATE_VALUE           NUMERIC(14,2) NOT NULL label='Rate Value',
          EFFECTIVE_FROM_DT    DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective From Date',
          EFFECTIVE_TO_DT      DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective To Date',
          PROCESSED_DTTM       DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed by ETL'
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if not(%sysfunc( exist(_CIDDS.MISC_RATE_SET) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.MISC_RATE_SET (
          RATE_SET_RK        NUMERIC(10) label='Rate Set Key',
          LANGUAGE_CD        VARCHAR(3) label='Language Code',
          VALID_FROM_DTTM    DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          RATE_SET_NM        VARCHAR(50) label='Rate Set Name',
          RATE_SET_DESC      VARCHAR(255) label='Rate Set  Description',
          RATE_SET_ID        VARCHAR(32) label='Rate Set Id',
          VALID_TO_DTTM      DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          PROCESSED_DTTM     DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed by ETL',
          CONSTRAINT PRIM_KEY PRIMARY KEY (RATE_SET_RK, LANGUAGE_CD, VALID_FROM_DTTM)
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if not(%sysfunc( exist(_CIDDS.MISC_RATE_TYPE) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.MISC_RATE_TYPE (
          RATE_TYPE_CD       VARCHAR(32) label='Rate Type Code',
          LANGUAGE_CD        VARCHAR(3) label='Language Code',
          VALID_FROM_DTTM    DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          VALID_TO_DTTM      DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          RATE_TYPE_DESC     VARCHAR(255) label='Rate Type Description',
          PROCESSED_DTTM     DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed by ETL',
          CONSTRAINT PRIM_KEY PRIMARY KEY (RATE_TYPE_CD, LANGUAGE_CD, VALID_FROM_DTTM)
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if not(%sysfunc( exist(_CIDDS.RATE_SOURCE) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.RATE_SOURCE (
          RATE_SOURCE_CD     VARCHAR(3)label='Rate Source Code',
          LANGUAGE_CD        VARCHAR(3) label='Language Code',
          VALID_FROM_DTTM    DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          VALID_TO_DTTM      DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          RATE_SOURCE_DESC   VARCHAR(255) label='Rate Source Description',
          PROCESSED_DTTM     DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed by ETL',
          CONSTRAINT PRIM_KEY PRIMARY KEY (RATE_SOURCE_CD, LANGUAGE_CD, VALID_FROM_DTTM)
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if not(%sysfunc( exist(_CIDDS.CURRENCY_COMPLEX_EXCH_RATE) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.CURRENCY_COMPLEX_EXCH_RATE (
          EXCHANGE_RATE_SET_RK      NUMERIC(10) NOT NULL label='Exchange Rate Set Key',
          EXCHANGE_RATE_TYPE_CD     VARCHAR(32) NOT NULL label='Exchange Rate Type Code',
          EXCHANGE_RATE_SOURCE_CD   VARCHAR(3) label='Exchange Rate Source Code',
          GL_ACCOUNT_RK             NUMERIC(10) label='GL Account Key',
          INTERNAL_ORG_RK           NUMERIC(10) label='Internal Organization Key',
          FROM_CURRENCY_CD          VARCHAR(3) label='From Currency Code',
          TO_CURRENCY_CD            VARCHAR(3) label='To Currency Code',
          COST_CENTER_RK            NUMERIC(10) label='Cost Center Key',
          PROFIT_CENTER_RK          NUMERIC(10) label='Profit Center Key',
          ITEM_CATEGORY_CD          VARCHAR(3) label='Item Category Code',
          TIME_PERIOD_RK            NUMERIC(10) label='Time Period Key',
          EXCHANGE_RT               NUMERIC(14,2) NOT NULL label='Exchange Rate',
          EFFECTIVE_FROM_DT         DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective From Date',
          EFFECTIVE_TO_DT           DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective To Date',
          PROCESSED_DTTM            DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed by ETL'
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if not(%sysfunc( exist(_CIDDS.COMPETENCY_ASSOC_TYPE) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.COMPETENCY_ASSOC_TYPE (
          COMPETENCY_ASSOC_TYPE_CD   VARCHAR(32) label='Competency Association Type Code',
          LANGUAGE_CD                VARCHAR(3) label='Language Code',
          VALID_FROM_DTTM            DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          VALID_TO_DTTM              DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          COMPETENCY_ASSOC_TYPE_DESC VARCHAR(255) label='Competency Association Type Description',
          COMPTENCY_ADK              VARCHAR(32) label='Competency ADK',
          DEFAULT_COMPETENCY_RK     NUMERIC(10) label='Default Competency Key',
          PROCESSED_DTTM             DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Datetime Processed by ETL',
          CONSTRAINT PRIM_KEY PRIMARY KEY (COMPETENCY_ASSOC_TYPE_CD, LANGUAGE_CD, VALID_FROM_DTTM)
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if not(%sysfunc( exist(_CIDDS.SASOP_DETAIL) )) %then %do;
   PROC SQL;
     CREATE TABLE _CIDDS.SASOP_DETAIL (
          INTERNAL_ORG_RK              NUMERIC(10) NOT NULL label='Internal Organization Key',
          GL_ACCOUNT_RK                NUMERIC(10) NOT NULL label='GL Account Key',
          ANALYSIS_RK                  NUMERIC(10) NOT NULL label='Analysis Key',
          CURRENCY_CD                  VARCHAR(3) label='Currency Code',
          COST_CENTER_RK               NUMERIC(10) label='Cost Center Key',
          PROFIT_CENTER_RK             NUMERIC(10) label='Profit Center Key',
          ITEM_CATEGORY_CD             VARCHAR(3) label='Item Category Code',
          TIME_PERIOD_RK               NUMERIC(10) NOT NULL label='Time Period Key',
          TRANSACTION_AMT              NUMERIC(14,2) label='Transaction Amount',
          TRANSACTION_AMT_YTD_FLG      CHAR(1) label='Transaction Amount Year-to-Date Flag',
          TRANSACTION_DT               DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Transaction Date',
          PROCESSED_DTTM               DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Processed Datetime'
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_HISTORY" and
        upcase(name) = "GPA_NO"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.EDUCATION_HISTORY
       ADD GPA_NO NUMERIC(10,3) label='Grade Point Average'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;
%else %do;
  PROC SQL;
    ALTER TABLE _CIDDS.EDUCATION_HISTORY
       MODIFY GPA_NO NUMERIC(10,3) label='Grade Point Average'
      ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_HISTORY" and
        upcase(name) = "ACADEMIC_CREDIT_HRS_INCLD_NO"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;

   DATA _CIDDS.EDUCATION_HISTORY;
       set  _CIDDS.EDUCATION_HISTORY (rename=(ACADEMIC_CREDIT_HOURS_INCLD_NO=ACADEMIC_CREDIT_HRS_INCLD_NO ACADEMIC_CREDIT_HOURS_ATTPD_NO=ACADEMIC_CREDIT_HRS_ATTPD_NO ACADEMIC_CREDIT_HOURS_ERND_NO=ACADEMIC_CREDIT_HRS_ERND_NO));
   run;

   PROC SQL;
     ALTER TABLE _CIDDS.EDUCATION_HISTORY
       MODIFY      ACADEMIC_CREDIT_HRS_INCLD_NO NUMERIC(15) label='Academic Credit Hours Included Number'
       MODIFY      ACADEMIC_CREDIT_HRS_ATTPD_NO NUMERIC(15) label='Academic Credit Hours Attempted Number'
       MODIFY      ACADEMIC_CREDIT_HRS_ERND_NO NUMERIC(15) label='Academic Credit Hours Earned Number'
       ADD CONSTRAINT _NM0001_ NOT NULL (VALID_TO_DTTM)
       ADD CONSTRAINT _NM0002_ NOT NULL (PROCESSED_DTTM)
       ADD CONSTRAINT PRIM_KEY PRIMARY KEY (EDUCATION_HISTORY_RK, VALID_FROM_DTTM)
     ;
   QUIT;

   %checkErr;
   %if &_tmpRC %then
       %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_CATEGORY_CLASS"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;

   %macro _rename;
          %let _dsid=%sysfunc(open(_CIDDS.COMPETENCY_TYPE));
          %let _AnyRows=%sysfunc(attrn(&_dsid,ANY));
          %let _dsid=%sysfunc(close(&_dsid));
          %if "&_AnyRows"="1" %then %do;
            proc datasets library=_CIDDS;
                change COMPETENCY_TYPE=COMPETENCY_TYPE_V4;
            run;
              quit;
          %end;
          %else %do;
             proc SQL;
                 DROP TABLE _CIDDS.COMPETENCY_TYPE;
            QUIT;
          %end;
   %mend;
   %_rename;

   PROC SQL;

     CREATE TABLE _CIDDS.COMPETENCY_TYPE (
          COMPETENCY_TYPE_CD  VARCHAR(10) label='Competency Type Code',
          LANGUAGE_CD          VARCHAR(3) label='Language Code',
          VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          VALID_TO_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          COMPETENCY_TYPE_DESC VARCHAR(255) label='Competency Type Description',
          PROCESSED_DTTM       DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Processed Datetime',
          CONSTRAINT PRIM_KEY PRIMARY KEY (COMPETENCY_TYPE_CD, LANGUAGE_CD,
          VALID_FROM_DTTM)
      );

     CREATE TABLE _CIDDS.COMPETENCY_CATEGORY_CLASS (
          COMPETENCY_CATEGORY_CLASS_CD   VARCHAR(10) label='Competency Category Class Code',
          VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          VALID_TO_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          COMPETENCY_CATALOG_CD VARCHAR(10) label='Competency Catalog Code',
          COMPETENCY_CLASS_CD  VARCHAR(10) label='Competency Class Code',
          COMPETENCY_TYPE_CD   VARCHAR(10) label='Competency Type Code',
          COMPETENCY_CATEGORY_CD VARCHAR(10) label='Competency Category Code',
          COMPETENCY_JOB_GROUP_CD VARCHAR(10) label='Competency Job Group Code',
          SPECIAL_REF_COMP_CD  VARCHAR(10) label='Special Ref Comp Code',
          PROCESSED_DTTM       DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Processed Datetime',
          CONSTRAINT PRIM_KEY PRIMARY KEY (COMPETENCY_CATEGORY_CLASS_CD, VALID_FROM_DTTM)
     );

     QUIT;
     %checkErr;
     %if &_tmpRC %then
       %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY" and
        upcase(name) = "COMPETENCY_CATEGORY_CLASS_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY
       DROP COMPETENCY_TYPE_CD
       ADD COMPETENCY_CATEGORY_CLASS_CD VARCHAR(10) label='Competency Category Class Code'
      ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_CATEGORY"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     CREATE TABLE _CIDDS.COMPETENCY_CATEGORY (
     COMPETENCY_CATEGORY_CD  VARCHAR(10) label='Competency Category Code',
     LANGUAGE_CD          VARCHAR(3) label='Language Code',
     VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
     VALID_TO_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
     COMPETENCY_CATEGORY_DESC VARCHAR(255) label='Competency Category Description',
     PROCESSED_DTTM       DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Processed Datetime',
     CONSTRAINT PRIM_KEY PRIMARY KEY (COMPETENCY_CATEGORY_CD, LANGUAGE_CD,
      VALID_FROM_DTTM)
   );
   QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
%end ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_EVIDENCE
       MODIFY EVIDENCE_NUMERIC_MINIMUM_VAL NUMERIC(15,2) label='Evidence Numeric Minimum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_EVIDENCE
       MODIFY EVIDENCE_NUMERIC_MAXIMUM_VAL NUMERIC(15,2) label='Evidence Numeric Maximum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_EVIDENCE
       MODIFY EVIDENCE_NUMERIC_INTERVAL_VAL NUMERIC(15,2) label='Evidence Numeric Interval Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_WEIGHT_BOUNDS
       MODIFY WEIGHT_NUMERIC_MINIMUM_VAL NUMERIC(15,2) label='Weight Numeric Minimum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_WEIGHT_BOUNDS
       MODIFY WEIGHT_NUMERIC_MAXIMUM_VAL NUMERIC(15,2) label='Weight Numeric Maximum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_WEIGHT_BOUNDS
       MODIFY WEIGHT_NUMERIC_INTERVAL_VAL NUMERIC(15,2) label='Weight Numeric Interval Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
 %let _colCouCk1 = 0 ;
 %let _colCouCk2 = 0 ;

 PROC SQL;
     SELECT count(*) INTO :_colCouCK1
       FROM dictionary.columns
       WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
             (upcase(name) = "EDUCATION_NUMERIC_MAXIMUM_VAL")
       ;
 QUIT;
 PROC SQL;
      SELECT count(*) INTO :_colCouCK2
        FROM dictionary.columns
        WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
              (upcase(name) = "EDUCATION_NUMERIC_MAX_VAL")
        ;
 QUIT;

 %if &_colCouCK1 ne 0  %then %do;
   PROC SQL;
      ALTER TABLE _CIDDS.EDUCATION_VALUE
        MODIFY EDUCATION_NUMERIC_MAXIMUM_VAL NUMERIC(15,2) label='Education Numeric Maximum Value'
      ;
    QUIT;
    %checkErr;
    %if &_tmpRC %then
     %return ;

   PROC DATASETS LIB=_CIDDS;
      MODIFY EDUCATION_VALUE;
          RENAME EDUCATION_NUMERIC_MAXIMUM_VAL=EDUCATION_NUMERIC_MAX_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return;
 %end;
 %else %if &_colCouCK2 ne 0  %then %do;
   PROC SQL;
      ALTER TABLE _CIDDS.EDUCATION_VALUE
        MODIFY EDUCATION_NUMERIC_MAX_VAL NUMERIC(15,2) label='Education Numeric Maximum Value'
      ;
    QUIT;
    %checkErr;
    %if &_tmpRC %then
     %return ;
 %end;


 /******************************************************************************/
  %let _colCouCk1 = 0 ;
  %let _colCouCk2 = 0 ;

  PROC SQL;
      SELECT count(*) INTO :_colCouCK1
        FROM dictionary.columns
        WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
              (upcase(name) = "EDUCATION_NUMERIC_MINIMUM_VAL")
        ;
  QUIT;
  PROC SQL;
       SELECT count(*) INTO :_colCouCK2
         FROM dictionary.columns
         WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
               (upcase(name) = "EDUCATION_NUMERIC_MIN_VAL")
         ;
  QUIT;

  %if &_colCouCK1 ne 0  %then %do;
    PROC SQL;
       ALTER TABLE _CIDDS.EDUCATION_VALUE
         MODIFY EDUCATION_NUMERIC_MINIMUM_VAL NUMERIC(15,2) label='Education Numeric Minimum Value'
       ;
     QUIT;
     %checkErr;
     %if &_tmpRC %then
      %return ;

    PROC DATASETS LIB=_CIDDS;
       MODIFY EDUCATION_VALUE;
           RENAME EDUCATION_NUMERIC_MINIMUM_VAL=EDUCATION_NUMERIC_MIN_VAL;
    QUIT;
    %checkErr;
    %if &_tmpRC %then
      %return;
  %end;
  %else %if &_colCouCK2 ne 0  %then %do;
    PROC SQL;
       ALTER TABLE _CIDDS.EDUCATION_VALUE
         MODIFY EDUCATION_NUMERIC_MIN_VAL NUMERIC(15,2) label='Education Numeric Minimum Value'
       ;
     QUIT;
     %checkErr;
     %if &_tmpRC %then
      %return ;
  %end;
  %let _colCouCk1 = 0 ;
 %let _colCouCk2 = 0 ;

/******************************************************************************/
   %let _colCount = 0 ;

    PROC SQL;
        SELECT count(*) INTO :_colCount
          FROM dictionary.columns
          WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
                (upcase(name) = "EDUCATION_NUMERIC_MINIMUM_VAL") and
                upcase(type) = 'CHAR'
          ;
  QUIT;

  %if &_colCouCK1 EQ 0 %then %do;
  DATA _CIDDS.EDUCATION_VALUE (drop=char_value);
       set  _CIDDS.EDUCATION_VALUE (rename=(EDUCATION_NUMERIC_VAL=char_value));
       length EDUCATION_NUMERIC_VAL 8;
       EDUCATION_NUMERIC_VAL=input(char_value, BEST12.);
   run;

   PROC SQL;
     ALTER TABLE _CIDDS.EDUCATION_VALUE
       MODIFY EDUCATION_NUMERIC_VAL NUMERIC(15,3) label='Education Numeric Value'
       ADD NOT NULL (VALID_TO_DTTM)
       ADD NOT NULL (PROCESSED_DTTM)
       ADD CONSTRAINT PRIM_KEY PRIMARY KEY (EDUCATION_HISTORY_RK, EDUCATION_VALUE_TYPE_CD,
       VALID_FROM_DTTM)
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
  %end;


 /******************************************************************************/
  %let _colCouCK1 = 0 ;
  %let _colCouCK2 = 0 ;

  PROC SQL;
  SELECT count(*) INTO :_colCouCK1
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
        (upcase(name) = "EDUCATION_STRING_MINIMUM_VAL")
   ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
   %return ;

  PROC SQL;
    SELECT count(*) INTO :_colCouCK2
    FROM dictionary.columns
    WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
         (upcase(name) = "EDUCATION_STRING_MAXIMUM_VAL")
     ;
    QUIT;
    %checkErr;
    %if &_tmpRC %then
     %return ;

    %if &_colCouCK1 ne 0 OR  &_colCouCK2 ne 0  %then %do ;

        PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
          modify EDUCATION_VALUE;
         %if &_colCouCK1 ne 0 %then %do;
           rename EDUCATION_STRING_MINIMUM_VAL=EDUCATION_STRING_MIN_VAL;
         %end;
         %if &_colCouCK2 ne 0 %then %do;
           rename EDUCATION_STRING_MAXIMUM_VAL=EDUCATION_STRING_MAX_VAL;
         %end;
         quit;;

        %checkErr;
        %if &_tmpRC %then
            %return ;
    %end ;

 %let _colCouCK1 = 0 ;
 %let _colCouCK2 = 0 ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.EMPLOYEE_ABSENCE
       MODIFY DURATION_QTY NUMERIC(8,2) label='Duration'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.EMPLOYEE_X_EVIDENCE
       MODIFY EMP_EVD_NUMERIC_VAL  NUMERIC(10,2) label='EMP EVD Numeric Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY
       MODIFY COMPETENCY_EMP_NUM_VAL NUMERIC(10,2) label='Competency Emp Numeric Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "DIMENSION_TYPE" and
        upcase(name) = "MISC_RATE_COLUMN_NM"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 1 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.DIMENSION_TYPE
       ADD MISC_COLUMN_NM VARCHAR(50) label='Miscellaneous Column Name'
     ;
     UPDATE _CIDDS.DIMENSION_TYPE
       SET MISC_COLUMN_NM = MISC_RATE_COLUMN_NM
     ;
     ALTER TABLE _CIDDS.DIMENSION_TYPE
       DROP MISC_RATE_COLUMN_NM
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "DIMENSION_TYPE" and
        upcase(name) = "MISC_COLUMN_NM"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.DIMENSION_TYPE
       ADD MISC_COLUMN_NM VARCHAR(50) label='Miscellaneous Column Name'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   PROC SQL;
     UPDATE _CIDDS.DIMENSION_TYPE
       SET MISC_COLUMN_NM = 
       CASE WHEN DIMENSION_TYPE_CD = 'TIME' then 'TIME_PERIOD_RK'
            WHEN DIMENSION_TYPE_CD = 'INTORG' then 'INTERNAL_ORG_RK'
            else BASE_FACT_COLUMN_NM
       END
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
data WORK.MISC_DTYPES;
   set _CIDDS.DIMENSION_TYPE;
   if DIMENSION_TYPE_CD not in ("ACCOUNT", "ANALYSIS", "COSTCTR", "CURRENCY", "EXTORG",
                                "INTORG", "ITEMCAT", "PROFITCTR", "TIME", "TRADER") then
      output;
   keep DIMENSION_TYPE_CD MISC_COLUMN_NM;
run;
%checkErr;
%if &_tmpRC %then
  %return ;

data _null_;
   set WORK.MISC_DTYPES;
   if (_n_ = 1) then do;
      call symput("dtype_codes", DIMENSION_TYPE_CD);
      call symput("misc_columns", MISC_COLUMN_NM);
   end;
   else do;
      call symput("dtype_codes",strip(symget("dtype_codes"))||"|"||DIMENSION_TYPE_CD);
      call symput("misc_columns",strip(symget("misc_columns"))||"|"||MISC_COLUMN_NM);
   end;
run;
%checkErr;
%if &_tmpRC %then
  %return ;

proc sql noprint;
   select count(*) INTO :_recCnt
   from WORK.MISC_DTYPES
   ;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

%do i = 1 %to &_recCnt;
   %let _dtypeCd=%sysfunc(kscan(&dtype_codes,&i,'|'));
   %let _colName=%sysfunc(kscan(&misc_columns,&i,'|'));
   %let _colCount = 0;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "MISC_RATE" and
           upcase(name) = upcase("&_colName")
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do;
     PROC SQL;
       ALTER TABLE _CIDDS.MISC_RATE
         ADD &_colName VARCHAR(32) label="&_dtypeCd Key"
       ;
     QUIT;
     %checkErr;
     %if &_tmpRC %then
       %return ;
   %end;

   %let _colCount = 0;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "CURRENCY_COMPLEX_EXCH_RATE" and
           upcase(name) = upcase("&_colName")
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do;
      PROC SQL;
        ALTER TABLE _CIDDS.CURRENCY_COMPLEX_EXCH_RATE
          ADD &_colName VARCHAR(32) label="&_dtypeCd Key"
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;

   %let _colCount = 0;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "SASOP_DETAIL" and
           upcase(name) = upcase("&_colName")
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do;
      PROC SQL;
        ALTER TABLE _CIDDS.SASOP_DETAIL
          ADD &_colName VARCHAR(32) label="&_dtypeCd Key"
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
%end;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY" and
        upcase(name) = "COMPETENCY_LEVEL_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY
       ADD COMPETENCY_LEVEL_VAL VARCHAR(20) label='Competency Level Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "JOB_X_COMPETENCY" and
        upcase(name) = "COMPETENCY_LEVEL_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 1 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.JOB_X_COMPETENCY
       DROP COMPETENCY_LEVEL_VAL
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _CIDDS.JOB_X_COMPETENCY
       MODIFY NUMERIC_REQ_LEVEL_VAL NUMERIC(15,2) label='Numeric Req Level Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_ASSOC" and
        upcase(name) = "COMPETENCY_ASSOC_TYPE_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_ASSOC
       ADD COMPETENCY_ASSOC_TYPE_CD VARCHAR(32) label='Competency Association Type Code'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EMPLOYEE" and
        upcase(name) = "USER_NM"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.EMPLOYEE
       ADD USER_NM VARCHAR(60) label='User Name'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
PROC SQL;
  ALTER TABLE _CIDDS.COMPETENCY_ASSOC
    drop PRIMARY KEY
  ;
  ALTER TABLE _CIDDS.COMPETENCY_ASSOC
    add PRIM_KEY PRIMARY KEY (COMPETENCY_RK, PARENT_COMPETENCY_RK, COMPETENCY_ASSOC_TYPE_CD,
                              VALID_FROM_DTTM )
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

/******************************************************************************/
PROC SQL;
  ALTER TABLE _CIDDS.JOB_POSITION
    MODIFY INTERNAL_ORG_RK  NUMERIC(10) label='Internal Organization Key'
    ;
QUIT;
proc sql noprint;
  select count(*) into :_exist
  from dictionary.constraint_column_usage
  where table_catalog='_CIDDS' and table_name='JOB_POSITION' and column_name='INTERNAL_ORG_RK'
      and constraint_name ne '';
  ;
quit;

%if &_exist EQ 0 %then %do;
  PROC SQL;
     ALTER TABLE _CIDDS.JOB_POSITION
        ADD NOT NULL (INTERNAL_ORG_RK)
     ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
    %return ;
%end;


/******************************************************************************/
  PROC SQL;
    ALTER TABLE _CIDDS.CURRENCY_EXCH_RATE
      DROP CONSTRAINT PRIM_KEY
    ;
    ALTER TABLE _CIDDS.CURRENCY_EXCH_RATE
      ADD CONSTRAINT PRIM_KEY PRIMARY KEY (FROM_CURRENCY_CD, TO_CURRENCY_CD,
        EFFECTIVE_FROM_DT, EXCHANGE_RATE_TYPE_CD, EXCHANGE_RATE_SET_RK,
        EXCHANGE_RATE_SOURCE_CD, TIME_PERIOD_RK)
    ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

/***************************************************************************
 * Modelchanges to remove the CAL_ Tables from stageDDS 1.4 to stageDDS 5.1
 ***************************************************************************/
%if %sysfunc( exist(_CIDDS.CAL_DATE) ) %then %do;
   PROC SQL;
     DROP TABLE _CIDDS.CAL_DATE;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if %sysfunc( exist(_CIDDS.CAL_DATE_X_TIME_PERIOD) ) %then %do;
   PROC SQL;
     DROP TABLE _CIDDS.CAL_DATE_X_TIME_PERIOD;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if %sysfunc( exist(_CIDDS.CAL_EVENT) ) %then %do;
   PROC SQL;
     DROP TABLE _CIDDS.CAL_EVENT;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
/* Update labels for columns in the ANALYSIS_ASSOC_TYPE table                 */
/******************************************************************************/
PROC SQL;
  ALTER TABLE _CIDDS.ANALYSIS_ASSOC_TYPE
    MODIFY ANALYSIS_ASSOC_TYPE_DESC VARCHAR(255) label='Analysis Association Type Description'
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

/******************************************************************************/
%let _exist = 0 ;

PROC SQL;
  SELECT count(*) INTO :_exist
  from dictionary.constraint_column_usage
  WHERE upcase(table_catalog)='_CIDDS' and upcase(table_name)='COMPETENCY' and
        upcase(column_name)='COMPETENCY_ID' and constraint_name ne ''
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_exist EQ 0 %then %do;
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY
       ADD NOT NULL (COMPETENCY_ID)
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;


%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY" and
        upcase(name) = "SOURCE_SYSTEM_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY
       ADD SOURCE_SYSTEM_CD VARCHAR(3) label='Source System Code'
       ADD COMPETENCY_ADK VARCHAR(32) label='Competency ADK'
       ADD ROLL_UP_TO_PARENT_FLG CHAR(1) label='Roll Up To Parent Flag'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_ASSOC" and
        upcase(name) = "ORDER_NO"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _CIDDS.COMPETENCY_ASSOC
       ADD ORDER_NO NUMERIC(8) label='Order Number'
       ADD COMPETENCY_ADK VARCHAR(32) label='Competency ADK'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_EVIDENCE" and
        upcase(name) = "EVIDENCE_NUM_MINIMUM_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
     MODIFY COMPETENCY_EVIDENCE;
       RENAME EVIDENCE_NUMERIC_MINIMUM_VAL=EVIDENCE_NUM_MINIMUM_VAL
              EVIDENCE_NUMERIC_MAXIMUM_VAL=EVIDENCE_NUM_MAXIMUM_VAL
              EVIDENCE_NUMERIC_INTERVAL_VAL=EVIDENCE_NUM_INTERVAL_VAL
              EVIDENCE_STRING_MINIMUM_VAL=EVIDENCE_STR_MINIMUM_VAL
              EVIDENCE_STRING_MAXIMUM_VAL=EVIDENCE_STR_MAXIMUM_VAL
              EVIDENCE_STRING_INTERVAL_VAL=EVIDENCE_STR_INTERVAL_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_WEIGHT_BOUNDS" and
        upcase(name) = "WEIGHT_NUM_MINIMUM_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
     MODIFY COMPETENCY_WEIGHT_BOUNDS;
       RENAME WEIGHT_NUMERIC_MINIMUM_VAL=WEIGHT_NUM_MINIMUM_VAL
              WEIGHT_NUMERIC_MAXIMUM_VAL=WEIGHT_NUM_MAXIMUM_VAL
              WEIGHT_NUMERIC_INTERVAL_VAL=WEIGHT_NUM_INTERVAL_VAL
              WEIGHT_STRING_MINIMUM_VAL=WEIGHT_STR_MINIMUM_VAL
              WEIGHT_STRING_MAXIMUM_VAL=WEIGHT_STR_MAXIMUM_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EDUCATION_VALUE" and
        upcase(name) = "EDUCATION_NUM_MIN_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
     MODIFY EDUCATION_VALUE;
       RENAME EDUCATION_NUMERIC_MIN_VAL=EDUCATION_NUM_MIN_VAL
              EDUCATION_NUMERIC_MAX_VAL=EDUCATION_NUM_MAX_VAL
              EDUCATION_NUMERIC_VAL=EDUCATION_NUM_VAL
              EDUCATION_STRING_MIN_VAL=EDUCATION_STR_MIN_VAL
              EDUCATION_STRING_MAX_VAL=EDUCATION_STR_MAX_VAL
              EDUCATION_STRING_VAL=EDUCATION_STR_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EMPLOYEE_X_EVIDENCE" and
        upcase(name) = "EMP_EVD_NUM_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
     MODIFY EMPLOYEE_X_EVIDENCE;
       RENAME EMP_EVD_NUMERIC_VAL=EMP_EVD_NUM_VAL
              EMP_EVD_NUMERIC_VALUE_DESC=EMP_EVD_NUM_VALUE_DESC
              EMP_EVD_STRING_VAL=EMP_EVD_STR_VAL
              EMP_EVD_STRING_VALUE_DESC=EMP_EVD_STR_VALUE_DESC;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EMPLOYEE_X_JOB_X_COMPETENCY" and
        upcase(name) = "COMPETENCY_EMP_STR_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
     MODIFY EMPLOYEE_X_JOB_X_COMPETENCY;
       RENAME COMPETENCY_EMP_STRING_VAL=COMPETENCY_EMP_STR_VAL
              COMPETENCY_EMP_STRING_VAL_DESC=COMPETENCY_EMP_STR_VAL_DESC;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "COMPETENCY_ASSOC_TYPE" and
        upcase(name) = "COMPETENCY_ADK"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_CIDDS NODETAILS NOLIST;
     MODIFY COMPETENCY_ASSOC_TYPE;
       RENAME COMPTENCY_ADK=COMPETENCY_ADK;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
/* CHANGES FOR DEFECT S0548336                                                */
/******************************************************************************/
%let _colCount = 0 ;

PROC SQL NOPRINT;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "EMPLOYEE_X_JOB_X_COMPETENCY" and
        upcase(name) = "WEIGHT_TYPE_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   %let _recCnt = 0 ;
   PROC SQL NOPRINT;
      SELECT count(*) INTO :_recCnt
      FROM _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY
      ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %if &_recCnt > 0 %then %do;
      PROC SQL;
         CREATE TABLE _CIDDS.OLD_EMP_X_JOB_X_COMPETENCY AS
           SELECT * FROM _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY
         ;
         DROP TABLE _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY
         ;
         CREATE TABLE _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY (
           EMPLOYEE_RK          NUMERIC(10) label='Employee Key',
           JOB_CD               VARCHAR(10) label='Job Code',
           START_DT             DATE FORMAT=&DTFMT INFORMAT=&DTFMT label='Job Code Start Date',
           COMPETENCY_RK        NUMERIC(10) label='Competency Key',
           WEIGHT_TYPE_CD       VARCHAR(10) label='Weight Type Code',
           VALID_FROM_DTTM      DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid From Datetime',
           VALID_TO_DTTM        DATE NOT NULL FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid To Datetime',
           EMPLOYEE_ID          VARCHAR(32) label='Employee Id',
           COMPETENCY_ID        VARCHAR(32) label='Competency Id',
           COMPETENCY_EMP_NUM_VAL NUMERIC(10,2) label='Competency Emp Numeric Value',
           COMPETENCY_EMP_NUM_VAL_DESC VARCHAR(255) label='Competency Emp Numeric Value Description',
           COMPETENCY_EMP_STR_VAL VARCHAR(20) label='Competency Emp String Value',
           COMPETENCY_EMP_STR_VAL_DESC VARCHAR(255) label='Competency Emp String Value Description',
           POSITION_CD          VARCHAR(10) label='Position Code',
           END_DT               DATE FORMAT=&DTFMT INFORMAT=&DTFMT label='End Date',
           PROCESSED_DTTM       DATE NOT NULL FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime',
           CONSTRAINT PRIM_KEY PRIMARY KEY (EMPLOYEE_RK, JOB_CD, START_DT, COMPETENCY_RK,
                                            WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         );
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   %else %do;
      PROC SQL;
         ALTER TABLE _CIDDS.EMPLOYEE_X_JOB_X_COMPETENCY
           ADD WEIGHT_TYPE_CD VARCHAR(10) label='Weight Type Code'
           DROP CONSTRAINT PRIM_KEY
           ADD CONSTRAINT PRIM_KEY PRIMARY KEY (EMPLOYEE_RK, JOB_CD, START_DT, COMPETENCY_RK,
                                                WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_CIDDS" and upcase(memname) = "JOB_X_COMPETENCY" and
        upcase(name) = "WEIGHT_TYPE_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   %let _recCnt = 0 ;
   PROC SQL NOPRINT;
      SELECT count(*) INTO :_recCnt
      FROM _CIDDS.JOB_X_COMPETENCY
      ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %if &_recCnt > 0 %then %do;
      PROC SQL;
         CREATE TABLE _CIDDS.OLD_JOB_X_COMPETENCY AS
           SELECT * FROM _CIDDS.JOB_X_COMPETENCY
         ;
         DROP TABLE _CIDDS.JOB_X_COMPETENCY
         ;
         CREATE TABLE _CIDDS.JOB_X_COMPETENCY (
           JOB_CD               VARCHAR(10) label='Job Code',
           COMPETENCY_RK        NUMERIC(10) label='Competency Key',
           WEIGHT_TYPE_CD       VARCHAR(10) label='Weight Type Code',
           VALID_FROM_DTTM      DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid From Datetime',
           VALID_TO_DTTM        DATE NOT NULL FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid To Datetime',
           COMPETENCY_ID        VARCHAR(32) label='Competency Id',
           COMPETENCY_IMPORTANCE_VAL VARCHAR(20) label='Competency Importance Value',
           NUMERIC_REQ_LEVEL_VAL NUMERIC(15,2) label='Numeric Req Level Value',
           NUMERIC_REQ_LEVEL_VAL_DESC VARCHAR(255) label='Numeric Req Level Value Description',
           STRING_REQ_LEVEL_VAL VARCHAR(20) label='String Req Level Value',
           STRING_REQ_LEVEL_VAL_DESC VARCHAR(255) label='String Req Level Value Description',
           PROCESSED_DTTM       DATE NOT NULL FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Processed Datetime',
           CONSTRAINT PRIM_KEY PRIMARY KEY (JOB_CD, COMPETENCY_RK, WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         );
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   %else %do;
      PROC SQL;
         ALTER TABLE _CIDDS.JOB_X_COMPETENCY
           ADD WEIGHT_TYPE_CD VARCHAR(10) label='Weight Type Code'
           DROP CONSTRAINT PRIM_KEY
           ADD CONSTRAINT PRIM_KEY PRIMARY KEY (JOB_CD, COMPETENCY_RK, WEIGHT_TYPE_CD,
                                                VALID_FROM_DTTM)
         ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
%end ;

/******************************************************************************/

%sessdver(solName=DDS, solVer=5.1, solStatus=Successful, solMethod=Migration);

/******************************************************************************/

%MEND migr8dds;
