/***************************************************************************
 * Model changes from stageDDS 1.4 to stageDDS 5.1
 ***************************************************************************/
%MACRO migr8stg;

options notes source source2;

/***************************************************************************
 * Edit the following macro variables to change the migration path
 ***************************************************************************
 *
 *   - _dttmfmt: Any valid SAS datetime format. (ex. DATETIME21.)
 *
 *   - _dtfmt: Any valid SAS date format. (ex. DATE9.)
 *
 *
 ***************************************************************************/

%let _dttmfmt=DATETIME21.;
%let _dtfmt=DATE9.;

%bldLibNm(sesetl_sasLibRef=_StgDDS
        , sesetl_OMRSASLibraryObjName=&CrossIndustryDDStier_folder./&StgLibObj./&StgLibObj
        , sesetl_useType=BASE,
          sesetl_validateLib=Y
         ) ;

/******************************************************************************/
%macro _migrate(_libref=);

  %let _datarep = WINDOWS_64;
  proc sql noprint;
    select datarepname into :_datarep
    from dictionary.tables
    where upcase(libname) eq upcase("&_libref")
      and datarepname = 'WINDOWS_32'
    ;
  quit;
  
  %if "&_datarep" eq "WINDOWS_64" %then %goto MACRO_END;

  %let _path=;
  PROC SQL noprint;
    select path into :_path
    from dictionary.libnames
    where upcase(libname) eq upcase("&_libref")
      and level=1
    ;
  QUIT;

  /* Backup existing tables in directory */
  %let _path = %sysfunc(strip(&_path));
  %if %str(&_path) eq %then
    %let _path = %sysfunc(pathname(&_libref));

  %let _path=%sysfunc(kcompress("&_path.", '()"'));
  %put Backup &_path....;

  %let _dt = %sysfunc(putn(%sysfunc(date()), date8.));
  %let _tm = %sysfunc(putn(%sysfunc(time()), time8.));
  %let _tm = %sysfunc(kcompress(&_tm, ':'));

  /* Paths for Windows and Unix are different */
  libname &_libref "&_path";
  options noxwait;
  %if &SYSSCP = WIN %then %do;
     x "md &_path.\backup_&_dt._&_tm.";
     x "copy &_path.\*.* &_path.\backup_&_dt._&_tm.";

     libname _backup "&_path.\backup_&_dt._&_tm.";
  %end;
  %else %do;
     x "mkdir &_path./backup_&_dt._&_tm.";
     x "cp &_path./*.* &_path./backup_&_dt._&_tm.";

     libname _backup "&_path./backup_&_dt._&_tm."
   %end;

  proc datasets lib=&_libref kill; run;

  proc migrate in=_backup out=&_libref; run;

  %MACRO_END:;
%mend _migrate;
%_migrate(_libref=_StgDDS);
/******************************************************************************/

%sessdver(solName=StageDDS, solVer=5.1, solStatus=Failed, solMethod=Migration);

/******************************************************************************/
/*      Modelchanges to table columns from stageDDS 1.4 to stageDDS 5.1       */
/******************************************************************************/

/* MISC_RATE */

%if not(%sysfunc( exist(_StgDDS.MISC_RATE) )) %then %do;
   PROC SQL;
   CREATE TABLE _StgDDS.MISC_RATE (
        RATE_SET_ID          VARCHAR(32) NOT NULL label='Rate Set Id',
        RATE_TYPE_CD         VARCHAR(32) NOT NULL label='Rate Type Code',
        RATE_SOURCE_CD       VARCHAR(3) label='Rate Source Code',
        GL_ACCOUNT_ID        VARCHAR(32) label='GL Account Id',
        INTERNAL_ORG_ID      VARCHAR(32) label='Internal Organization Id',
        CURRENCY_CD          VARCHAR(3) label='Currency Code',
        COST_CENTER_ID       VARCHAR(32) label='Cost Center Id',
        PROFIT_CENTER_ID     VARCHAR(32) label='Profit Center Id',
        ITEM_CATEGORY_CD     VARCHAR(3) label='Item Category Code',
        TIME_PERIOD_ID       VARCHAR(32) label='Time Period Id',
        RATE_VALUE           NUMERIC(14,2) NOT NULL label='Rate Value',
        EFFECTIVE_FROM_DT    DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective From Date',
        EFFECTIVE_TO_DT      DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective To Date'
   );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* MISC_RATE_SET */

%if not(%sysfunc( exist(_StgDDS.MISC_RATE_SET) )) %then %do;
   PROC SQL;
   CREATE TABLE _StgDDS.MISC_RATE_SET (
        RATE_SET_ID          VARCHAR(32) label='Rate Set Key',
        LANGUAGE_CD          VARCHAR(3) label='Language Code',
        VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
        RATE_SET_NM          VARCHAR(50) label='Rate Set Name',
        RATE_SET_DESC        VARCHAR(255) label='Rate Set  Description',
        RATE_SET_ID          VARCHAR(32) label='Rate Set Id',
        VALID_TO_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime'
   );
   CREATE INDEX MISC_RATE_SET_IDX ON _StgDDS.MISC_RATE_SET (RATE_SET_ID, LANGUAGE_CD, VALID_FROM_DTTM);
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* MISC_RATE_TYPE */

%if not(%sysfunc( exist(_StgDDS.MISC_RATE_TYPE) )) %then %do;
   PROC SQL;
   CREATE TABLE _StgDDS.MISC_RATE_TYPE (
        RATE_TYPE_CD         VARCHAR(32) label='Rate Type Code',
        LANGUAGE_CD          VARCHAR(3) label='Language Code',
        VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
        VALID_TO_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
        RATE_TYPE_DESC       VARCHAR(255) label='Rate Type Description'
   );
   CREATE INDEX MISC_RATE_TYPE_IDX ON _StgDDS.MISC_RATE_TYPE (RATE_TYPE_CD, LANGUAGE_CD, VALID_FROM_DTTM);
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* RATE_SOURCE */

%if not(%sysfunc( exist(_StgDDS.RATE_SOURCE) )) %then %do;
   PROC SQL;
   CREATE TABLE _StgDDS.RATE_SOURCE (
        RATE_SOURCE_CD       VARCHAR(3) label='Rate Source Code',
        LANGUAGE_CD          VARCHAR(3) label='Language Code',
        VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
        VALID_TO_DTTM        DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
        RATE_SOURCE_DESC     VARCHAR(255) label='Rate Source Description'
   );
   CREATE INDEX RATE_SOURCE_IDX ON _StgDDS.RATE_SOURCE (RATE_SOURCE_CD, LANGUAGE_CD, VALID_FROM_DTTM);
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* COMPETENCY_ASSOC_TYPE */

%if not(%sysfunc( exist(_StgDDS.COMPETENCY_ASSOC_TYPE) )) %then %do;
   PROC SQL;
   CREATE TABLE _StgDDS.COMPETENCY_ASSOC_TYPE (
        COMPETENCY_ASSOC_TYPE_CD VARCHAR(32) label='Competency Association Type Code',
        LANGUAGE_CD              VARCHAR(3) label='Language Code',
        VALID_FROM_DTTM          DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
        VALID_TO_DTTM            DATE NOT NULL FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
        COMPETENCY_ASSOC_TYPE_DESC VARCHAR(255) label='Competency Association Type Description',
        COMPTENCY_ADK            VARCHAR(32) label='Competency ADK',
        DEFAULT_COMPETENCY_ID    VARCHAR(32) label='Default Competency ID'
   );
   CREATE INDEX COMPETENCY_ASSOC_TYPE_IDX ON _StgDDS.COMPETENCY_ASSOC_TYPE (COMPETENCY_ASSOC_TYPE_CD, LANGUAGE_CD, VALID_FROM_DTTM);
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* CURRENCY_COMPLEX_EXCH_RATE */

%if not(%sysfunc( exist(_StgDDS.CURRENCY_COMPLEX_EXCH_RATE) )) %then %do;
   PROC SQL;
   CREATE TABLE _StgDDS.CURRENCY_COMPLEX_EXCH_RATE (
        EXCHANGE_RATE_SET_ID     VARCHAR(32)   NOT NULL label='Exchange Rate Set Id',
        EXCHANGE_RATE_TYPE_CD    VARCHAR(32)   NOT NULL label='Exchange Rate Type Code',
        EXCHANGE_RATE_SOURCE_CD  VARCHAR(3)    label='Exchange Rate Source Code',
        GL_ACCOUNT_ID            VARCHAR(32)   label='GL Account Id',
        INTERNAL_ORG_ID          VARCHAR(32)   label='Internal Organization Id',
        FROM_CURRENCY_CD         VARCHAR(3)    label='From Currency Code',
        TO_CURRENCY_CD           VARCHAR(3)    label='To Currency Code',
        COST_CENTER_ID           VARCHAR(32)   label='Cost Center Id',
        PROFIT_CENTER_ID         VARCHAR(32)   label='Profit Center Id',
        ITEM_CATEGORY_CD         VARCHAR(3)    label='Item Category Code',
        TIME_PERIOD_ID           VARCHAR(32)   label='Time Period Id',
        EXCHANGE_RT              NUMERIC(14,2) NOT NULL label='Exchange Rate',
        EFFECTIVE_FROM_DT        DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective From Date',
        EFFECTIVE_TO_DT          DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Effective To Date'
   );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/* SASOP_DETAIL */

%if not(%sysfunc( exist(_StgDDS.SASOP_DETAIL) )) %then %do;
   PROC SQL;
     CREATE TABLE _StgDDS.SASOP_DETAIL (
          INTERNAL_ORG_ID              VARCHAR(32) NOT NULL label='Internal Organization Id',
          GL_ACCOUNT_ID                VARCHAR(32) NOT NULL label='GL Account Id',
          ANALYSIS_ID                  VARCHAR(32) NOT NULL label='Analysis Id',
          CURRENCY_CD                  VARCHAR(3) label='Currency Code',
          COST_CENTER_ID               VARCHAR(32) label='Cost Center Id',
          PROFIT_CENTER_ID             VARCHAR(32) label='Profit Center Id',
          ITEM_CATEGORY_CD             VARCHAR(3) label='Item Category Code',
          TIME_PERIOD_ID               VARCHAR(32) NOT NULL label='Time Period Id',
          TRANSACTION_AMT              NUMERIC(14,2) label='Transaction Amount',
          TRANSACTION_AMT_YTD_FLG      CHAR(1) label='Transaction Amount Year-to-Date Flag',
          TRANSACTION_DT               DATE FORMAT=&_dtfmt INFORMAT=&_dtfmt label='Transaction Date'
     );
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_HISTORY" and
        upcase(name) = "GPA_NO"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.EDUCATION_HISTORY
       add GPA_NO NUMERIC(10,3) label='Grade Point Average'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;
%else %do;
  PROC SQL;
    ALTER TABLE _StgDDS.EDUCATION_HISTORY
       MODIFY GPA_NO NUMERIC(10,3) label='Grade Point Average'
      ;
    QUIT;
    %checkErr;
    %if &_tmpRC %then
       %return ;
%end;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_HISTORY" and
        upcase(name) = "ACADEMIC_CREDIT_HRS_INCLD_NO"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;

   DATA _STGDDS.EDUCATION_HISTORY;
          set  _STGDDS.EDUCATION_HISTORY (rename=(ACADEMIC_CREDIT_HOURS_INCLD_NO=ACADEMIC_CREDIT_HRS_INCLD_NO ACADEMIC_CREDIT_HOURS_ATTPD_NO=ACADEMIC_CREDIT_HRS_ATTPD_NO ACADEMIC_CREDIT_HOURS_ERND_NO=ACADEMIC_CREDIT_HRS_ERND_NO));
   run;

   PROC SQL;
        ALTER TABLE _STGDDS.EDUCATION_HISTORY
          MODIFY      ACADEMIC_CREDIT_HRS_INCLD_NO NUMERIC(15) label='Academic Credit Hours Included Number'
          MODIFY      ACADEMIC_CREDIT_HRS_ATTPD_NO NUMERIC(15) label='Academic Credit Hours Attempted Number'
          MODIFY      ACADEMIC_CREDIT_HRS_ERND_NO NUMERIC(15) label='Academic Credit Hours Earned Number'
        ;

        CREATE INDEX EDUCATION_HISTORY_IDX ON _STGDDS.EDUCATION_HISTORY (EDUCATION_HISTORY_ID, VALID_FROM_DTTM);

   QUIT;

   %checkErr;
   %if &_tmpRC %then
       %return ;
%end ;

/******************************************************************************/
  %let _colCouCK1 = 0 ;
  %let _colCouCK2 = 0 ;

  PROC SQL;
    SELECT count(*) INTO :_colCouCK1
    FROM dictionary.columns
    WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
          (upcase(name) = "EDUCATION_NUMERIC_MINIMUM_VAL")
    ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
  %return ;

  PROC SQL;
    SELECT count(*) INTO :_colCouCK2
    FROM dictionary.columns
    WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
          (upcase(name) = "EDUCATION_NUMERIC_MIN_VAL")
    ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
  %return ;

  %if &_colCouCK1 ne 0 %then %do;
  PROC SQL;
     ALTER TABLE _STGDDS.EDUCATION_VALUE
       MODIFY EDUCATION_NUMERIC_MINIMUM_VAL NUMERIC(15,2) label='Education Numeric Minimum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
  %end;
  %else %if &_colCouCK2 ne 0 %then %do;
    PROC SQL;
     ALTER TABLE _STGDDS.EDUCATION_VALUE
       MODIFY EDUCATION_NUMERIC_MIN_VAL NUMERIC(15,2) label='Education Numeric Minimum Value'
       ;
    QUIT;
    %checkErr;
    %if &_tmpRC %then
      %return ;
  %end;
/******************************************************************************/
  %let _colCouCK1 = 0 ;
  %let _colCouCK2 = 0 ;

  PROC SQL;
    SELECT count(*) INTO :_colCouCK1
    FROM dictionary.columns
    WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
          (upcase(name) = "EDUCATION_NUMERIC_MAXIMUM_VAL")
    ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
  %return ;

  PROC SQL;
    SELECT count(*) INTO :_colCouCK2
    FROM dictionary.columns
    WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
          (upcase(name) = "EDUCATION_NUMERIC_MAX_VAL")
    ;
  QUIT;
  %checkErr;
  %if &_tmpRC %then
  %return ;
  %if &_colCouCK1 ne 0 %then %do;
  PROC SQL;
     ALTER TABLE _STGDDS.EDUCATION_VALUE
       MODIFY EDUCATION_NUMERIC_MAXIMUM_VAL NUMERIC(15,2) label='Education Numeric Maximum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;
%else %if &_colCouCK2 ne 0 %then %do;
PROC SQL;
     ALTER TABLE _STGDDS.EDUCATION_VALUE
       MODIFY EDUCATION_NUMERIC_MAX_VAL NUMERIC(15,2) label='Education Numeric Maximum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
 %end;

/******************************************************************************/
%let _colCouCK1 = 0 ;
%let _colCouCK2 = 0 ;
%let _colCouCK3 = 0 ;
%let _colCouCK4 = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCouCK1
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
        (upcase(name) = "EDUCATION_NUMERIC_MINIMUM_VAL")
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

 PROC SQL;
    SELECT count(*) INTO :_colCouCK2
    FROM dictionary.columns
    WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
           (upcase(name) = "EDUCATION_NUMERIC_MAXIMUM_VAL")
    ;
  QUIT;
 %checkErr;
 %if &_tmpRC %then
 %return ;

 PROC SQL;
 SELECT count(*) INTO :_colCouCK3
 FROM dictionary.columns
 WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
       (upcase(name) = "EDUCATION_STRING_MINIMUM_VAL")
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

  PROC SQL;
    SELECT count(*) INTO :_colCouCK4
    FROM dictionary.columns
    WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
         (upcase(name) = "EDUCATION_STRING_MAXIMUM_VAL")
    ;
  QUIT;
 %checkErr;
 %if &_tmpRC %then
 %return ;

%if &_colCouCK1 ne 0 OR  &_colCouCK2 ne 0 or &_colCouCK3 ne 0 or &_colCouCK4 ne 0 %then %do ;

   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     modify EDUCATION_VALUE;
    %if &_colCouCK1 ne 0 %then %do;
     rename EDUCATION_NUMERIC_MINIMUM_VAL=EDUCATION_NUMERIC_MIN_VAL;
    %end;
    %if &_colCouCK2 ne 0 %then %do;
     rename EDUCATION_NUMERIC_MAXIMUM_VAL=EDUCATION_NUMERIC_MAX_VAL;
    %end;
    %if &_colCouCK3 ne 0 %then %do;
      rename EDUCATION_STRING_MINIMUM_VAL=EDUCATION_STRING_MIN_VAL;
    %end;
    %if &_colCouCK4 ne 0 %then %do;
      rename EDUCATION_STRING_MAXIMUM_VAL=EDUCATION_STRING_MAX_VAL;
    %end;
   QUIT;

   PROC SQL;
        DROP INDEX EDUCATION_VALUE_IDX
        FROM _StgDDS.EDUCATION_VALUE;
        CREATE INDEX EDUCATION_VALUE_IDX ON _STGDDS.EDUCATION_VALUE (EDUCATION_HISTORY_ID, EDUCATION_VALUE_TYPE_CD, VALID_FROM_DTTM);
   QUIT;

   %checkErr;
   %if &_tmpRC %then
       %return ;
%end ;

%let _colCouCK1 = 0 ;
%let _colCouCK2 = 0 ;
%let _colCouCK3 = 0 ;
%let _colCouCK4 = 0 ;

/******************************************************************************/

   %let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_CATEGORY_CLASS"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;

   %macro _rename;
          %let _dsid=%sysfunc(open(_STGDDS.COMPETENCY_TYPE));
          %let _AnyRows=%sysfunc(attrn(&_dsid,ANY));
          %let _dsid=%sysfunc(close(&_dsid));
          %if "&_AnyRows"="1" %then %do;
            proc datasets library=_STGDDS;
                change COMPETENCY_TYPE=COMPETENCY_TYPE_V4;
            run;
              quit;
          %end;
          %else %do;
             proc SQL;
                 DROP INDEX COMPETENCY_TYPE_IDX
               FROM _STGDDS.COMPETENCY_TYPE
             ;
             DROP TABLE _STGDDS.COMPETENCY_TYPE;

            QUIT;
          %end;
   %mend;
   %_rename;

   PROC SQL;

     CREATE TABLE _STGDDS.COMPETENCY_TYPE (
         COMPETENCY_TYPE_CD  VARCHAR(10) label='Competency Type Code',
         LANGUAGE_CD          VARCHAR(3) label='Language Code',
         VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
         VALID_TO_DTTM        DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
         COMPETENCY_TYPE_DESC VARCHAR(255) label='Competency Type Description'
      );

     CREATE INDEX COMPETENCY_TYPE_IDX ON _STGDDS.COMPETENCY_TYPE (COMPETENCY_TYPE_CD, LANGUAGE_CD, VALID_FROM_DTTM);

     CREATE TABLE _STGDDS.COMPETENCY_CATEGORY_CLASS (
               COMPETENCY_CATEGORY_CLASS_CD   VARCHAR(10) label='Competency Category Class Code',
            VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
            VALID_TO_DTTM        DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
            COMPETENCY_CATALOG_CD VARCHAR(10) label='Competency Catalog Code',
            COMPETENCY_CLASS_CD  VARCHAR(10) label='Competency Class Code',
            COMPETENCY_TYPE_CD VARCHAR(10) label='Competency Type Code',
            COMPETENCY_CATEGORY_CD VARCHAR(10) label='Competency Category Code',
            COMPETENCY_JOB_GROUP_CD VARCHAR(10) label='Competency Job Group Code',
               SPECIAL_REF_COMP_CD  VARCHAR(10) label='Special Ref Comp Code'
     );

     CREATE INDEX COMPETENCY_CATEGORY_CLASS_IDX ON _STGDDS.COMPETENCY_CATEGORY_CLASS (COMPETENCY_CATEGORY_CLASS_CD, VALID_FROM_DTTM);

     QUIT;
     %checkErr;
     %if &_tmpRC %then
       %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY" and
        upcase(name) = "COMPETENCY_CATEGORY_CLASS_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY
       DROP COMPETENCY_TYPE_CD
       ADD COMPETENCY_CATEGORY_CLASS_CD VARCHAR(10) label='Competency Category Class Code'
      ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_CATEGORY"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     CREATE TABLE _STGDDS.COMPETENCY_CATEGORY (
          COMPETENCY_CATEGORY_CD  VARCHAR(10) label='Competency Category Code',
          LANGUAGE_CD          VARCHAR(3) label='Language Code',
          VALID_FROM_DTTM      DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid From Datetime',
          VALID_TO_DTTM        DATE FORMAT=&_dttmfmt INFORMAT=&_dttmfmt label='Valid To Datetime',
          COMPETENCY_CATEGORY_DESC VARCHAR(255) label='Competency Category Description'
     );

     CREATE INDEX COMPETENCY_CATEGORY_IDX ON _STGDDS.COMPETENCY_CATEGORY (COMPETENCY_CATEGORY_CD, LANGUAGE_CD, VALID_FROM_DTTM);

   QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
%end ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_EVIDENCE
       MODIFY EVIDENCE_NUMERIC_MINIMUM_VAL NUMERIC(15,2) label='Evidence Numeric Minimum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_EVIDENCE
       MODIFY EVIDENCE_NUMERIC_MAXIMUM_VAL NUMERIC(15,2) label='Evidence Numeric Maximum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_EVIDENCE
       MODIFY EVIDENCE_NUMERIC_INTERVAL_VAL NUMERIC(15,2) label='Evidence Numeric Interval Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_WEIGHT_BOUNDS
       MODIFY WEIGHT_NUMERIC_MINIMUM_VAL NUMERIC(15,2) label='Weight Numeric Minimum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_WEIGHT_BOUNDS
       MODIFY WEIGHT_NUMERIC_MAXIMUM_VAL NUMERIC(15,2) label='Weight Numeric Maximum Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_WEIGHT_BOUNDS
       MODIFY WEIGHT_NUMERIC_INTERVAL_VAL NUMERIC(15,2) label='Weight Numeric Interval Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;


/******************************************************************************/
   DATA _STGDDS.EDUCATION_VALUE (drop=char_value);
          set  _STGDDS.EDUCATION_VALUE (rename=(EDUCATION_NUMERIC_VAL=char_value));
          length EDUCATION_NUMERIC_VAL 8;
          EDUCATION_NUMERIC_VAL=input(char_value, BEST12.);
   run;

   PROC SQL;
     ALTER TABLE _STGDDS.EDUCATION_VALUE
       MODIFY EDUCATION_NUMERIC_VAL NUMERIC(15,3) label='Education Numeric Value'
     ;

     CREATE INDEX EDUCATION_VALUE_IDX ON _STGDDS.EDUCATION_VALUE (EDUCATION_HISTORY_ID, EDUCATION_VALUE_TYPE_CD, VALID_FROM_DTTM);

   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.EMPLOYEE_ABSENCE
       MODIFY DURATION_QTY NUMERIC(8,2) label='Duration'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.EMPLOYEE_X_EVIDENCE
       MODIFY EMP_EVD_NUMERIC_VAL  NUMERIC(10,2) label='EMP EVD Numeric Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
       MODIFY COMPETENCY_EMP_NUM_VAL NUMERIC(10,2) label='Competency Emp Numeric Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "DIMENSION_TYPE" and
        upcase(name) = "MISC_RATE_COLUMN_NM"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 1 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.DIMENSION_TYPE
       ADD MISC_COLUMN_NM VARCHAR(50) label='Miscellaneous Column Name'
     ;
     UPDATE _StgDDS.DIMENSION_TYPE
       SET MISC_COLUMN_NM = MISC_RATE_COLUMN_NM
     ;
     ALTER TABLE _StgDDS.DIMENSION_TYPE
       DROP MISC_RATE_COLUMN_NM
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "DIMENSION_TYPE" and
        upcase(name) = "MISC_COLUMN_NM"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.DIMENSION_TYPE
       ADD MISC_COLUMN_NM VARCHAR(50) label='Miscellaneous Column Name'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   PROC SQL;
     UPDATE _StgDDS.DIMENSION_TYPE
       SET MISC_COLUMN_NM = 
       CASE WHEN DIMENSION_TYPE_CD = 'TIME' then 'TIME_PERIOD_RK'
            WHEN DIMENSION_TYPE_CD = 'INTORG' then 'INTERNAL_ORG_RK'
            else BASE_FACT_COLUMN_NM
       END
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
       %return ;
%end ;


/******************************************************************************/
%if %sysfunc( exist(_StgDDS.SOURCE_DIMENSION_TYPE) ) %then %do;
   %let _colCount = 0 ;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "SOURCE_DIMENSION_TYPE" and
           upcase(name) = "MISC_RATE_COLUMN_NM"
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 1 %then %do ;
      PROC SQL;
        ALTER TABLE _StgDDS.SOURCE_DIMENSION_TYPE
          ADD MISC_COLUMN_NM VARCHAR(50) label='Miscellaneous Column Name'
        ;
        UPDATE _StgDDS.SOURCE_DIMENSION_TYPE
          SET MISC_COLUMN_NM = MISC_RATE_COLUMN_NM
        ;
        ALTER TABLE _StgDDS.SOURCE_DIMENSION_TYPE
          DROP MISC_RATE_COLUMN_NM
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end ;

   %let _colCount = 0 ;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "SOURCE_DIMENSION_TYPE" and
           upcase(name) = "MISC_COLUMN_NM"
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do ;
      PROC SQL;
        ALTER TABLE _StgDDS.SOURCE_DIMENSION_TYPE
          ADD MISC_COLUMN_NM VARCHAR(50) label='Miscellaneous Column Name'
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
      PROC SQL;
        UPDATE _StgDDS.SOURCE_DIMENSION_TYPE
          SET MISC_COLUMN_NM = BASE_FACT_COLUMN_NM
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end ;
%end ;

/******************************************************************************/

%let _recCnt=0;

%if %sysfunc( exist(_StgDDS.SOURCE_DIMENSION_TYPE) ) %then %do;
  data _null_;
     set _StgDDS.SOURCE_DIMENSION_TYPE;
     if (_n_ = 1) then do;
        call symput("dtype_codes", DIMENSION_TYPE_CD);
        call symput("misc_columns", BUSINESS_ID_COLUMN_NM);
     end;
     else do;
        call symput("dtype_codes",strip(symget("dtype_codes"))||"|"||DIMENSION_TYPE_CD);
        call symput("misc_columns",strip(symget("misc_columns"))||"|"||BUSINESS_ID_COLUMN_NM);
     end;
  run;
  %checkErr;
  %if &_tmpRC %then
    %return ;

proc sql noprint;
   select count(*) INTO :_recCnt
   from _StgDDS.SOURCE_DIMENSION_TYPE
   ;
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

%END;

%do i = 1 %to &_recCnt;
   %let _dtypeCd=%sysfunc(kscan(&dtype_codes,&i,'|'));
   %let _colName=%sysfunc(kscan(&misc_columns,&i,'|'));
   %let _colCount = 0;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "MISC_RATE" and
           upcase(name) = upcase("&_colName")
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do;
     PROC SQL;
       ALTER TABLE _StgDDS.MISC_RATE
         ADD &_colName VARCHAR(32) label="&_dtypeCd Id"
       ;
     QUIT;
     %checkErr;
     %if &_tmpRC %then
       %return ;
   %end;

   %let _colCount = 0;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "CURRENCY_COMPLEX_EXCH_RATE" and
           upcase(name) = upcase("&_colName")
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do;
      PROC SQL;
        ALTER TABLE _StgDDS.CURRENCY_COMPLEX_EXCH_RATE
          ADD &_colName VARCHAR(32) label="&_dtypeCd Id"
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;

   %let _colCount = 0;

   PROC SQL;
     SELECT count(*) INTO :_colCount
     FROM dictionary.columns
     WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "SASOP_DETAIL" and
           upcase(name) = upcase("&_colName")
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

   %if &_colCount EQ 0 %then %do;
      PROC SQL;
        ALTER TABLE _StgDDS.SASOP_DETAIL
          ADD &_colName VARCHAR(32) label="&_dtypeCd Id"
        ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
%end;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY" and
        upcase(name) = "COMPETENCY_LEVEL_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.COMPETENCY
       ADD COMPETENCY_LEVEL_VAL VARCHAR(20) label='Competency Level Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "JOB_X_COMPETENCY" and
        upcase(name) = "COMPETENCY_LEVEL_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 1 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.JOB_X_COMPETENCY
       DROP COMPETENCY_LEVEL_VAL
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
   PROC SQL;
     ALTER TABLE _STGDDS.JOB_X_COMPETENCY
       MODIFY NUMERIC_REQ_LEVEL_VAL NUMERIC(15,2) label='Numeric Req Level Value'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_ASSOC" and
        upcase(name) = "COMPETENCY_ASSOC_TYPE_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.COMPETENCY_ASSOC
       ADD COMPETENCY_ASSOC_TYPE_CD VARCHAR(32) label='Competency Association Type Code'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EMPLOYEE" and
        upcase(name) = "USER_NM"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _StgDDS.EMPLOYEE
       ADD USER_NM VARCHAR(60) label='User Name'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;
/******************************************************************************/
PROC SQL;
  DROP INDEX COMPETENCY_ASSOC_IDX
  FROM _StgDDS.COMPETENCY_ASSOC
  ;
  CREATE INDEX COMPETENCY_ASSOC_IDX
  ON _StgDDS.COMPETENCY_ASSOC (COMPETENCY_ID, PARENT_COMPETENCY_ID,
                               COMPETENCY_ASSOC_TYPE_CD, VALID_FROM_DTTM)
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

/***************************************************************************
 * Modelchanges to remove the CAL_ Tables from stageDDS 1.4 to stageDDS 5.1
 ***************************************************************************/
%if %sysfunc( exist(_StgDDS.CAL_DATE) ) %then %do;
   PROC SQL;
     DROP TABLE _StgDDS.CAL_DATE;
   QUIT;
%end;
%checkErr;
%if &_tmpRC %then
  %return ;

%if %sysfunc( exist(_StgDDS.CAL_DATE_X_TIME_PERIOD) ) %then %do;
   PROC SQL;
     DROP TABLE _StgDDS.CAL_DATE_X_TIME_PERIOD;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

%if %sysfunc( exist(_StgDDS.CAL_EVENT) ) %then %do;
   PROC SQL;
     DROP TABLE _StgDDS.CAL_EVENT;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
/* Update labels for columns in the ANALYSIS_ASSOC_TYPE and                   */
/* COMPETENCY_X_TAXONOMY_SOURCE tables                                        */
/******************************************************************************/
PROC SQL;
  ALTER TABLE _STGDDS.ANALYSIS_ASSOC_TYPE
    MODIFY ANALYSIS_ASSOC_TYPE_DESC VARCHAR(255) label='Analysis Association Type Description'
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

PROC SQL;
  ALTER TABLE _STGDDS.COMPETENCY_X_TAXONOMY_SOURCE
    MODIFY TAXONOMY_SOURCE_ID VARCHAR(32) label='Taxonomy Source Id'
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

/******************************************************************************/
/* Remove all NOT NULL properties in the StageDDS                             */
/******************************************************************************/
proc sql;
create table work.constraint_table
as select table_name, constraint_name from dictionary.constraint_column_usage
where upcase(table_catalog) = "_STGDDS" and
      upcase(table_name) in ("MISC_RATE", "MISC_RATE_SET", "MISC_RATE_TYPE", "RATE_SOURCE",
                     "COMPETENCY_ASSOC_TYPE", "CURRENCY_COMPLEX_EXCH_RATE", "SASOP_DETAIL");
quit;
%checkErr;
%if &_tmpRC %then
  %return ;

%let _recCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_recCount
  FROM work.constraint_table
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%do i = 1 %to &_recCount;
   data _null_;
      set work.constraint_table (obs=&i);
      call symput("constraint_table", strip(table_name));
      call symput("constraint_name", strip(constraint_name));
   run;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   proc sql;
     ALTER TABLE _StgDDS.&constraint_table.
       DROP CONSTRAINT &constraint_name.
     ;
   quit;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end;

/******************************************************************************/
/* CHANGES FOR DEFECT S0546280                                                */
/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY" and
        upcase(name) = "SOURCE_SYSTEM_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY
       ADD SOURCE_SYSTEM_CD VARCHAR(3) label='Source System Code'
       ADD COMPETENCY_ADK VARCHAR(32) label='Competency ADK'
       ADD ROLL_UP_TO_PARENT_FLG CHAR(1) label='Roll Up To Parent Flag'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_ASSOC" and
        upcase(name) = "ORDER_NO"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC SQL;
     ALTER TABLE _STGDDS.COMPETENCY_ASSOC
       ADD ORDER_NO NUMERIC(8) label='Order Number'
       ADD COMPETENCY_ADK VARCHAR(32) label='Competency ADK'
     ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_EVIDENCE" and
        upcase(name) = "EVIDENCE_NUM_MINIMUM_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     MODIFY COMPETENCY_EVIDENCE;
       RENAME EVIDENCE_NUMERIC_MINIMUM_VAL=EVIDENCE_NUM_MINIMUM_VAL
              EVIDENCE_NUMERIC_MAXIMUM_VAL=EVIDENCE_NUM_MAXIMUM_VAL
              EVIDENCE_NUMERIC_INTERVAL_VAL=EVIDENCE_NUM_INTERVAL_VAL
              EVIDENCE_STRING_MINIMUM_VAL=EVIDENCE_STR_MINIMUM_VAL
              EVIDENCE_STRING_MAXIMUM_VAL=EVIDENCE_STR_MAXIMUM_VAL
              EVIDENCE_STRING_INTERVAL_VAL=EVIDENCE_STR_INTERVAL_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_WEIGHT_BOUNDS" and
        upcase(name) = "WEIGHT_NUM_MINIMUM_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     MODIFY COMPETENCY_WEIGHT_BOUNDS;
       RENAME WEIGHT_NUMERIC_MINIMUM_VAL=WEIGHT_NUM_MINIMUM_VAL
              WEIGHT_NUMERIC_MAXIMUM_VAL=WEIGHT_NUM_MAXIMUM_VAL
              WEIGHT_NUMERIC_INTERVAL_VAL=WEIGHT_NUM_INTERVAL_VAL
              WEIGHT_STRING_MINIMUM_VAL=WEIGHT_STR_MINIMUM_VAL
              WEIGHT_STRING_MAXIMUM_VAL=WEIGHT_STR_MAXIMUM_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EDUCATION_VALUE" and
        upcase(name) = "EDUCATION_NUM_MIN_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     MODIFY EDUCATION_VALUE;
       RENAME EDUCATION_NUMERIC_MIN_VAL=EDUCATION_NUM_MIN_VAL
              EDUCATION_NUMERIC_MAX_VAL=EDUCATION_NUM_MAX_VAL
              EDUCATION_NUMERIC_VAL=EDUCATION_NUM_VAL
              EDUCATION_STRING_MIN_VAL=EDUCATION_STR_MIN_VAL
              EDUCATION_STRING_MAX_VAL=EDUCATION_STR_MAX_VAL
              EDUCATION_STRING_VAL=EDUCATION_STR_VAL;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EMPLOYEE_X_EVIDENCE" and
        upcase(name) = "EMP_EVD_NUM_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     MODIFY EMPLOYEE_X_EVIDENCE;
       RENAME EMP_EVD_NUMERIC_VAL=EMP_EVD_NUM_VAL
              EMP_EVD_NUMERIC_VALUE_DESC=EMP_EVD_NUM_VALUE_DESC
              EMP_EVD_STRING_VAL=EMP_EVD_STR_VAL
              EMP_EVD_STRING_VALUE_DESC=EMP_EVD_STR_VALUE_DESC;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EMPLOYEE_X_JOB_X_COMPETENCY" and
        upcase(name) = "COMPETENCY_EMP_STR_VAL"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     MODIFY EMPLOYEE_X_JOB_X_COMPETENCY;
       RENAME COMPETENCY_EMP_STRING_VAL=COMPETENCY_EMP_STR_VAL
              COMPETENCY_EMP_STRING_VAL_DESC=COMPETENCY_EMP_STR_VAL_DESC;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "COMPETENCY_ASSOC_TYPE" and
        upcase(name) = "COMPETENCY_ADK"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   PROC DATASETS lib=_STGDDS NODETAILS NOLIST;
     MODIFY COMPETENCY_ASSOC_TYPE;
       RENAME COMPTENCY_ADK=COMPETENCY_ADK;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
%end ;

/******************************************************************************/
/* CHANGES FOR DEFECT S0548336                                                */
/******************************************************************************/
%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "EMPLOYEE_X_JOB_X_COMPETENCY" and
        upcase(name) = "WEIGHT_TYPE_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   %let _recCnt = 0 ;
   PROC SQL NOPRINT;
      SELECT count(*) INTO :_recCnt
      FROM _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
      ;
   QUIT;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %if &_recCnt > 0 %then %do;
      PROC SQL;
         CREATE TABLE _STGDDS.OLD_EMP_X_JOB_X_COMPETENCY AS
           SELECT * FROM _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
         ;
         DROP TABLE _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
         ;
         CREATE TABLE _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY (
           EMPLOYEE_ID          VARCHAR(32) label='Employee Id',
           JOB_CD               VARCHAR(10) label='Job Code',
           START_DT             DATE FORMAT=&DTFMT INFORMAT=&DTFMT label='Job Code Start Date',
           COMPETENCY_ID        VARCHAR(32) label='Competency Id',
           WEIGHT_TYPE_CD       VARCHAR(10) label='Weight Type Code',
           VALID_FROM_DTTM      DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid From Datetime',
           VALID_TO_DTTM        DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid To Datetime',
           COMPETENCY_EMP_NUM_VAL NUMERIC(10,2) label='Competency Emp Numeric Value',
           COMPETENCY_EMP_NUM_VAL_DESC VARCHAR(255) label='Competency Emp Numeric Value Description',
           COMPETENCY_EMP_STR_VAL VARCHAR(20) label='Competency Emp String Value',
           COMPETENCY_EMP_STR_VAL_DESC VARCHAR(255) label='Competency Emp String Value Description',
           POSITION_CD          VARCHAR(10) label='Position Code',
           END_DT               DATE FORMAT=&DTFMT INFORMAT=&DTFMT label='End Date'
         );
         CREATE INDEX EMPLOYEE_X_JOB_X_COMPETENCY_IDX ON _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
           (EMPLOYEE_ID, JOB_CD, START_DT, COMPETENCY_ID, WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   %else %do;
      PROC SQL;
         ALTER TABLE _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
           ADD WEIGHT_TYPE_CD VARCHAR(10) label='Weight Type Code'
         ;
         DROP INDEX EMPLOYEE_X_JOB_X_COMPETENCY_IDX
           FROM _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY
         ;
         CREATE INDEX EMPLOYEE_X_JOB_X_COMPETENCY_IDX
           ON _STGDDS.EMPLOYEE_X_JOB_X_COMPETENCY (EMPLOYEE_ID, JOB_CD, START_DT, COMPETENCY_ID,
                                                   WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end ;
%end;

%let _colCount = 0 ;

PROC SQL;
  SELECT count(*) INTO :_colCount
  FROM dictionary.columns
  WHERE upcase(libname) = "_STGDDS" and upcase(memname) = "JOB_X_COMPETENCY" and
        upcase(name) = "WEIGHT_TYPE_CD"
  ;
QUIT;
%checkErr;
%if &_tmpRC %then
  %return ;

%if &_colCount EQ 0 %then %do ;
   %let _recCnt = 0 ;
   PROC SQL NOPRINT;
      SELECT count(*) INTO :_recCnt
      FROM _STGDDS.JOB_X_COMPETENCY
      ;
   quit;
   %checkErr;
   %if &_tmpRC %then
     %return ;
   %if &_recCnt > 0 %then %do;
      PROC SQL;
         CREATE TABLE _STGDDS.OLD_JOB_X_COMPETENCY AS
           SELECT * FROM _STGDDS.JOB_X_COMPETENCY
         ;
         DROP TABLE _STGDDS.JOB_X_COMPETENCY
         ;
         CREATE TABLE _STGDDS.JOB_X_COMPETENCY (
           JOB_CD               VARCHAR(10) label='Job Code',
           COMPETENCY_ID        VARCHAR(32) label='Competency Id',
           WEIGHT_TYPE_CD       VARCHAR(10) label='Weight Type Code',
           VALID_FROM_DTTM      DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid From Datetime',
           VALID_TO_DTTM        DATE FORMAT=&DTTMFMT INFORMAT=&DTTMFMT label='Valid To Datetime',
           COMPETENCY_IMPORTANCE_VAL VARCHAR(20) label='Competency Importance Value',
           NUMERIC_REQ_LEVEL_VAL NUMERIC(15,2) label='Numeric Req Level Value',
           NUMERIC_REQ_LEVEL_VAL_DESC VARCHAR(255) label='Numeric Req Level Value Description',
           STRING_REQ_LEVEL_VAL VARCHAR(20) label='String Req Level Value',
           STRING_REQ_LEVEL_VAL_DESC VARCHAR(255) label='String Req Level Value Description'
         );
         CREATE INDEX JOB_X_COMPETENCY_IDX
           ON _STGDDS.JOB_X_COMPETENCY (JOB_CD, COMPETENCY_ID, WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
   %else %do;
      PROC SQL;
         ALTER TABLE _STGDDS.JOB_X_COMPETENCY
           ADD WEIGHT_TYPE_CD VARCHAR(10) label='Weight Type Code'
         ;
         DROP INDEX JOB_X_COMPETENCY_IDX
           FROM _STGDDS.JOB_X_COMPETENCY
         ;
         CREATE INDEX JOB_X_COMPETENCY_IDX
           ON _STGDDS.JOB_X_COMPETENCY (JOB_CD, COMPETENCY_ID, WEIGHT_TYPE_CD, VALID_FROM_DTTM)
         ;
      QUIT;
      %checkErr;
      %if &_tmpRC %then
        %return ;
   %end;
%end ;

/******************************************************************************/

%sessdver(solName=StageDDS, solVer=5.1, solStatus=Successful, solMethod=Migration);

/******************************************************************************/

%MEND migr8stg;
