/*********************************************************************************
 *  Copyright (c) 2007 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *  NAME:      migr8hcm.sas
 *
 *  PURPOSE:   Migrates HCM data to the current release
 *
 *********************************************************************************/
%macro migr8hcm(hcmlibref=_hcmlib);
%let hcmlibref=%qupcase(&hcmlibref.);

%If %sysfunc(libref(&hcmlibref.)) ne 0 %then
/*Assign the libref to the HCM data.*/
 %bldLibNm(sesetl_sasLibRef=&hcmlibref.
        , sesetl_OMRSASLibraryObjName=%nrbquote(&HCMdatatier_folder./&hcmlibobj./&hcmlibobj.)
        , sesetl_useType=BASE
        , sesetl_validateLib=Y);

%If %sysfunc(libref(_hcmmap)) ne 0 %then
/*Assign the libref to the HCM Map data.*/
%bldLibNm(sesetl_sasLibRef=_hcmmap
        , sesetl_OMRSASLibraryObjName=%nrbquote(&HCMsystem_folder./&mapslibobj./&mapslibobj.)
        , sesetl_useType=BASE
        , sesetl_validateLib=Y);

%If %sysfunc(libref(conform)) ne 0 %then
/*Assign the libref to the Solutions Conform tables.*/
%bldLibNm(sesetl_sasLibRef=conform
        , sesetl_OMRSASLibraryObjName=%nrbquote(&Soldatatier_folder./&CONFLibObj./&CONFLibObj.)
        , sesetl_useType=BASE
        , sesetl_validateLib=Y);

/*Assign the libref to the Solutions SASSDM data.*/
%If %sysfunc(libref(_sdmlib)) ne 0 %then
%bldLibNm(sesetl_sasLibRef=_sdmlib
           , sesetl_OMRSASLibraryObjName=&Solsystem_folder./&SdmLibObj./&SdmLibObj.
           , sesetl_validateLib=Y);

%If not(%sysfunc(exist(&hcmlibref..sas_hcmmetatable))) %then %do;
 %put ERROR: No previous version of SAS Human Capital Mangement exists.;
 %return;
%End;

%bldLibNm(sesetl_OMRSASLibraryObjName=%nrbquote(&HCMdatatier_folder./&hcmlibobj./&hcmlibobj.),
          sesetl_useType=CON
         )

/******************************************************************************/
%macro _migrate(_libref=);

  %let _datarep = WINDOWS_64;
  proc sql noprint;
    select datarepname into :_datarep
    from dictionary.tables
    where upcase(libname) eq upcase("&_libref")
      and datarepname = 'WINDOWS_32'
    ;
  quit;
  
  %if "&_datarep" eq "WINDOWS_64" %then %goto MACRO_END;

  %let _path=;
  PROC SQL noprint;
    select path into :_path
    from dictionary.libnames
    where upcase(libname) eq upcase("&_libref")
      and level=1
    ;
  QUIT;

  /* Backup existing tables in directory */
  %let _path = %sysfunc(strip(&_path));
  %if %str(&_path) eq %then
    %let _path = %sysfunc(pathname(&_libref));

  %let _path=%sysfunc(kcompress("&_path.", '()"'));
  %put Backup &_path....;

  %let _dt = %sysfunc(putn(%sysfunc(date()), date8.));
  %let _tm = %sysfunc(putn(%sysfunc(time()), time8.));
  %let _tm = %sysfunc(kcompress(&_tm, ':'));

  /* Paths for Windows and Unix are different */
  libname &_libref "&_path";
  options noxwait;
  %if &SYSSCP = WIN %then %do;
     x "md &_path.\backup_&_dt._&_tm.";
     x "copy &_path.\*.* &_path.\backup_&_dt._&_tm.";

     libname _backup "&_path.\backup_&_dt._&_tm.";
  %end;
  %else %do;
     x "mkdir &_path./backup_&_dt._&_tm.";
     x "cp &_path./*.* &_path./backup_&_dt._&_tm.";

     libname _backup "&_path./backup_&_dt._&_tm."
   %end;

  proc datasets lib=&_libref kill; run;

  proc migrate in=_backup out=&_libref; run;

  %MACRO_END:;
%mend _migrate;
%_migrate(_libref=conform);
/******************************************************************************/


/*Create HCM_Version_Data table.*/
PROC SQL;
     connect to &sesetl_EngineType as PASSTHRU
               (&sesetl_BestLogin &sesetl_ConnectionOptions );
     execute (
          CREATE TABLE IF NOT EXISTS HCM_VERSION_DATA (
              VERSION_NO            VARCHAR(32),
              SOLUTION_NM           VARCHAR(50),
              VERSION_STATUS_DESC   VARCHAR(255),
              LOAD_METHOD_DESC      VARCHAR(255),
              PROCESSED_DTTM        DATETIME
               ) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
        ) by PASSTHRU ;
     disconnect from PASSTHRU ;
Quit ;
%checkErr
%if &_tmpRC %then %goto exit;

/*Extract the current HCM release information.*/
%seschver(NameChk=HCM)
%checkErr
%if &_tmpRC %then %goto exit;
/*********************************************************************************************/
            /**********Migration code for HCM MySql tables*************/

/**********************************/
/********Existing Tables***********/
/**********************************/
%local _colCount;

/*Drop Tables*/
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           DROP TABLE IF EXISTS
           EXCLUSIONLIST, PROFILETABLEATTR
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Create a working table containing a list of the tables in the &hcmlibref library.*/
Proc sql;
 Create table hcmtbl as
 Select memname
 From dictionary.tables
 where upcase(libname) = "&hcmlibref.";
Quit;

/*Rename tables*/
/**********************/
/* DEFAULTPROPERTIES  */
/**********************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "DEFAULTPROPERTIES";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE DEFAULTPROPERTIES TO SAS_DEFAULT_PROPERTIES
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/**********************/
/* EMPLOYEE_PROFILE   */
/**********************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "EMPLOYEE_PROFILE";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE EMPLOYEE_PROFILE TO SAS_EMPLOYEE_PROFILE
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/**********************/
/*     HCMFORMATS     */
/**********************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "HCMFORMATS";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE HCMFORMATS TO SAS_HCM_FORMATS
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/**********************/
/*  HIERARCHYMAPPING  */
/**********************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "HIERARCHYMAPPING";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE HIERARCHYMAPPING TO SAS_HIERARCHY_MAPPING
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/**********************/
/*      MEASURES      */
/**********************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(ksubstr(memname,1,8)) = "MEASURES";
quit;

%if &_colCount gt 0 %then %do;
 Data _null_;
  Set hcmtbl;
  Where upcase(ksubstr(memname,1,8)) = "MEASURES";
  Call symput('meas'||strip(put(_n_,3.)),strip(memname));
 Run;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
 %Do i=1 %to &_colCount.;
  execute (
           RENAME TABLE &&meas&i TO SAS_%upcase(&&meas&i)
           ) by PASSTHRU ;
 %End;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/**********************/
/*  PROFILECATEGORY   */
/**********************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "PROFILECATEGORY";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE PROFILECATEGORY TO SAS_PROFILE_CATEGORY
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/************************/
/* PROFILECATEGORYATTR  */
/************************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "PROFILECATEGORYATTR";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE PROFILECATEGORYATTR TO SAS_PROFILE_CATEGORYATTR
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/************************/
/*   PROFILEHEADERATTR  */
/************************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "PROFILEHEADERATTR";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE PROFILEHEADERATTR TO SAS_PROFILE_HEADERATTR
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/************************/
/*   PROFILESEARCHATTR  */
/************************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "PROFILESEARCHATTR";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE PROFILESEARCHATTR TO SAS_PROFILE_SEARCHATTR
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/************************/
/*  PROFILESUMMARYATTR  */
/************************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "PROFILESUMMARYATTR";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE PROFILESUMMARYATTR TO SAS_PROFILE_SUMMARYATTR
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/************************/
/*     USEREMPLOYEE     */
/************************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "USEREMPLOYEE";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE USEREMPLOYEE TO SAS_USER_EMPLOYEE
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/************************/
/*     WRKGRPHIER       */
/************************/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmtbl
  where  upcase(memname) = "WRKGRPHIER";
quit;

%if &_colCount gt 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           RENAME TABLE WRKGRPHIER TO SAS_WRKGRPHIER
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;


/*Create a working table containing the tables and columns in the &hcmlibref library.*/
Proc sql;
 Create table hcmcol as
 Select memname, name
 From dictionary.columns
 where upcase(libname) = "&hcmlibref.";
Quit;


/*************************/
/*   SAS_HCMMETATABLE    */
/*************************/
/*ID*/
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           change `ID` `ID` int(11) NOT NULL AUTO_INCREMENT
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*BUILD_DT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "BUILD_DT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           ADD BUILD_DT datetime default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*IS_GEO_TABLE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "IS_GEO_TABLE";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           ADD IS_GEO_TABLE tinyint(1) default '0'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HCM_TABLE_NAME*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "HCM_TABLE_NAME";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           CHANGE HCM_TABLE_NAME TABLE_NM varchar(32) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*DESCRIPTION*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "DESCRIPTION";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           CHANGE DESCRIPTION TABLE_DESC varchar(100) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ISMASTER*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "ISMASTER";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           CHANGE ISMASTER IS_MASTER tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ISHISTORY*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "ISHISTORY";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           CHANGE ISHISTORY IS_HISTORY tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ISHIDDEN*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "ISHIDDEN";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           CHANGE ISHIDDEN IS_HIDDEN tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USEFMT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETATABLE" and
        upcase(name) = "USEFMT";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETATABLE
           CHANGE USEFMT USE_FMT tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*************************/
/*   SAS_HCMMETACOLUMN   */
/*************************/
/*IS_SEARCHABLE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "IS_SEARCHABLE";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           ADD IS_SEARCHABLE tinyint(1) default '1'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*SYMBOL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "SYMBOL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           ADD SYMBOL_CD varchar(3) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*TABLEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "TABLEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE TABLEID TABLE_ID int(11) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*HCM_COLUMN_NAME*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "HCM_COLUMN_NAME";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE HCM_COLUMN_NAME HCM_COLUMN_NM varchar(32) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*DESCRIPTION*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "DESCRIPTION";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE DESCRIPTION COLUMN_DESC varchar(100) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*FORMAT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "FORMAT";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE FORMAT COLUMN_FMT varchar(32) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*DISPLAYORDER*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "DISPLAYORDER";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE DISPLAYORDER DISPLAY_ORDER int(11) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ISHIDDEN*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "ISHIDDEN";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE ISHIDDEN IS_HIDDEN tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*USEFMT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "USEFMT";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE USEFMT USE_FMT tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ISANALYSIS*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "ISANALYSIS";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE ISANALYSIS IS_ANALYSIS tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*NMISS*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "NMISS";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE NMISS STAT_NMISS tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*USS*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "USS";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE USS STAT_USS tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CSS*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "CSS";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE CSS STAT_CSS tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*VAR*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "VAR";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE COLUMN VAR STAT_VAR tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*STD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "STD";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE STD STAT_STD tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*STDERR*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "STDERR";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE STDERR STAT_STDERR tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CV*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "CV";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE CV STAT_CV tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*T*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "T";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE COLUMN T STAT_T tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*PRT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "PRT";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE PRT STAT_PRT tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*LCLM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "LCLM";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE LCLM STAT_LCLM tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*UCLM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCMMETACOLUMN" and
        upcase(name) = "UCLM";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCMMETACOLUMN
           CHANGE UCLM STAT_UCLM tinyint(1) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*************************/
/*SAS_DEFAULT_PROPERTIES */
/*************************/
%If %sysfunc(exist(&hcmlibref..SAS_DEFAULT_PROPERTIES)) %then %do;
/*MODULETYPE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_DEFAULT_PROPERTIES" and
        upcase(name) = "MODULETYPE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_DEFAULT_PROPERTIES
           CHANGE MODULETYPE MODULE_TYPE varchar(10) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*PROPERTYNAME*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_DEFAULT_PROPERTIES" and
        upcase(name) = "PROPERTYNAME";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_DEFAULT_PROPERTIES
           CHANGE PROPERTYNAME PROPERTY_NM varchar(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*DEFAULTVALUE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_DEFAULT_PROPERTIES" and
        upcase(name) = "DEFAULTVALUE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_DEFAULT_PROPERTIES
           CHANGE DEFAULTVALUE DEFAULT_VALUE varchar(255) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*DESCRIPTION*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_DEFAULT_PROPERTIES" and
        upcase(name) = "DESCRIPTION";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_DEFAULT_PROPERTIES
           CHANGE COLUMN DESCRIPTION PROPERTY_DESC varchar(255) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/* SAS_HIERARCHY_MAPPING */
/*************************/
%If %sysfunc(exist(&hcmlibref..SAS_HIERARCHY_MAPPING)) %then %do;
/*HIERARCHY_CODE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HIERARCHY_MAPPING" and
        upcase(name) = "HIERARCHY_CODE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HIERARCHY_MAPPING
           CHANGE HIERARCHY_CODE HIERARCHY_CD varchar(32) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*SRCTYPE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HIERARCHY_MAPPING" and
        upcase(name) = "SRCTYPE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HIERARCHY_MAPPING
           CHANGE SRCTYPE SRC_TYPE decimal(8,0) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*INFOTABLENAME*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HIERARCHY_MAPPING" and
        upcase(name) = "INFOTABLENAME";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HIERARCHY_MAPPING
           CHANGE INFOTABLENAME TABLE_NM varchar(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*INFOTABLELINKFIELD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HIERARCHY_MAPPING" and
        upcase(name) = "INFOTABLELINKFIELD";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HIERARCHY_MAPPING
           CHANGE INFOTABLELINKFIELD RPTGRP_COL varchar(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ISDEFAULTASSOCTABLE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HIERARCHY_MAPPING" and
        upcase(name) = "ISDEFAULTASSOCTABLE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HIERARCHY_MAPPING
           CHANGE ISDEFAULTASSOCTABLE IS_DEFAULT_ASSOC_TABLE varchar(3) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*DETAILDIRECTIVE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HIERARCHY_MAPPING" and
        upcase(name) = "DETAILDIRECTIVE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HIERARCHY_MAPPING
           DROP DETAILDIRECTIVE
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/* SAS_PROFILE_CATEGORY  */
/*************************/
%If %sysfunc(exist(&hcmlibref..SAS_PROFILE_CATEGORY)) %then %do;
/*PROFILEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORY" and
        upcase(name) = "PROFILEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORY
           CHANGE PROFILEID PROFILE_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CATEGORYID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORY" and
        upcase(name) = "CATEGORYID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORY
           CHANGE CATEGORYID CATEGORY_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CATEGORYTYPE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORY" and
        upcase(name) = "CATEGORYTYPE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORY
           CHANGE CATEGORYTYPE CATEGORY_TYPE varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CATEGORYLABEL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORY" and
        upcase(name) = "CATEGORYLABEL";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORY
           CHANGE CATEGORYLABEL CATEGORY_LABEL varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/**************************/
/*SAS_PROFILE_CATEGORYATTR*/
/**************************/
%If %sysfunc(exist(&hcmlibref..SAS_PROFILE_CATEGORYATTR)) %then %do;
/*PROFILEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORYATTR" and
        upcase(name) = "PROFILEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORYATTR
           CHANGE PROFILEID PROFILE_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CATEGORYID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORYATTR" and
        upcase(name) = "CATEGORYID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORYATTR
           CHANGE CATEGORYID CATEGORY_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*CATEGORYATTRID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORYATTR" and
        upcase(name) = "CATEGORYATTRID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORYATTR
           CHANGE CATEGORYATTRID CATEGORY_ATTR_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ATTRIBUTEVALUE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_CATEGORYATTR" and
        upcase(name) = "ATTRIBUTEVALUE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_CATEGORYATTR
           CHANGE ATTRIBUTEVALUE ATTRIBUTE_VALUE VARCHAR(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/**************************/
/* SAS_PROFILE_HEADERATTR */
/**************************/
%If %sysfunc(exist(&hcmlibref..SAS_PROFILE_HEADERATTR)) %then %do;
/*PROFILEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_HEADERATTR" and
        upcase(name) = "PROFILEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_HEADERATTR
           CHANGE PROFILEID PROFILE_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*HEADERATTRID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_HEADERATTR" and
        upcase(name) = "HEADERATTRID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_HEADERATTR
           CHANGE HEADERATTRID HEADER_ATTR_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ATTRIBUTEVALUE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_HEADERATTR" and
        upcase(name) = "ATTRIBUTEVALUE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_HEADERATTR
           CHANGE ATTRIBUTEVALUE ATTRIBUTE_VALUE varchar(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/**************************/
/* SAS_PROFILE_SEARCHATTR */
/**************************/
%If %sysfunc(exist(&hcmlibref..SAS_PROFILE_SEARCHATTR)) %then %do;
/*PROFILEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SEARCHATTR" and
        upcase(name) = "PROFILEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SEARCHATTR
           CHANGE PROFILEID PROFILE_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*SEARCHATTRID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SEARCHATTR" and
        upcase(name) = "SEARCHATTRID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SEARCHATTR
           CHANGE SEARCHATTRID SEARCH_ATTR_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ATTRIBUTEVALUE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SEARCHATTR" and
        upcase(name) = "ATTRIBUTEVALUE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SEARCHATTR
           CHANGE ATTRIBUTEVALUE ATTRIBUTE_VALUE varchar(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/**************************/
/*SAS_PROFILE_SUMMARYATTR */
/**************************/
%If %sysfunc(exist(&hcmlibref..SAS_PROFILE_SUMMARYATTR)) %then %do;
/*PROFILEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SUMMARYATTR" and
        upcase(name) = "PROFILEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SUMMARYATTR
           CHANGE PROFILEID PROFILE_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*SUMMARYATTRID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SUMMARYATTR" and
        upcase(name) = "SUMMARYATTRID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SUMMARYATTR
           CHANGE SUMMARYATTRID SUMMARY_ATTR_ID varchar(36) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ATTRIBUTEVALUE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SUMMARYATTR" and
        upcase(name) = "ATTRIBUTEVALUE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SUMMARYATTR
           CHANGE ATTRIBUTEVALUE ATTRIBUTE_VALUE varchar(60) collate utf8_bin NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
/*ATTRIBUTETABLE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_PROFILE_SUMMARYATTR" and
        upcase(name) = "ATTRIBUTETABLE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_PROFILE_SUMMARYATTR
           CHANGE ATTRIBUTETABLE ATTRIBUTE_TABLE varchar(36) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*   SAS_USER_EMPLOYEE   */
/*************************/
%If %sysfunc(exist(&hcmlibref..SAS_USER_EMPLOYEE)) %then %do;
/*So that this code is re-runnable, reload sas_user_employee to eliminate the primary key if present.*/
%If %sysfunc(exist(&hcmlibref..sas_temp_user_employee)) %then %do;
Proc sql;
 DROP TABLE &hcmlibref..sas_temp_user_employee;
Quit;
%End;

Proc sql;
 CREATE TABLE &hcmlibref..sas_temp_user_employee AS
 SELECT * FROM &hcmlibref..sas_user_employee;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

Proc sql;
 DROP TABLE &hcmlibref..sas_user_employee;

 CREATE TABLE &hcmlibref..sas_user_employee AS
 SELECT * FROM &hcmlibref..sas_temp_user_employee;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

Proc sql;
 DROP TABLE &hcmlibref..sas_temp_user_employee;
Quit;

/*USER_ID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_USER_EMPLOYEE" and
        upcase(name) = "USER_ID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
          ALTER TABLE sas_user_employee
          ADD PRIMARY KEY(USER_ID)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_ID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_USER_EMPLOYEE" and
        upcase(name) = "EMPLOYEE_ID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_USER_EMPLOYEE
           MODIFY EMPLOYEE_ID varchar(32) NOT NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;
/*************************/
/*    SAS_WRKGRPHIER     */
/*************************/
%If %sysfunc(exist(&hcmlibref..SAS_WRKGRPHIER)) %then %do;
/*IMG_EMP_NODE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "IMG_EMP_NODE";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           ADD IMG_EMP_NODE varchar(32) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*IMG_DEPT_NODE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "IMG_DEPT_NODE";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           ADD IMG_DEPT_NODE varchar(32) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*SECURITY_LINK_FIELD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "SECURITY_LINK_FIELD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           ADD SECURITY_LINK_FIELD varchar(50) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*SECURITY_HIERARCHY*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "SECURITY_HIERARCHY";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           ADD SECURITY_HIERARCHY int(1) default '0'
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*IS_SECURITY_DEFAULT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "IS_SECURITY_DEFAULT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           ADD IS_SECURITY_DEFAULT  decimal(8,0)
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*DIMENSION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "DIMENSION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           ADD DIMENSION_CD  varchar(32)
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ISDEFAULT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "ISDEFAULT";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           CHANGE ISDEFAULT IS_DEFAULT decimal(8,0) NOT NULL
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HIERARCHYCODE*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "HIERARCHYCODE";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           CHANGE HIERARCHYCODE HIERARCHY_CD varchar(32) collate utf8_bin NOT NULL
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HIERARCHYNAME*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "HIERARCHYNAME";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           CHANGE HIERARCHYNAME HIERARCHY_NM varchar(50) collate utf8_bin default NULL
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HIERARCHYDESC*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_WRKGRPHIER" and
        upcase(name) = "HIERARCHYDESC";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_WRKGRPHIER
           CHANGE HIERARCHYDESC HIERARCHY_DESC varchar(255) collate utf8_bin default NULL
          ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;

%local orghier;
Data _null_;
 Set &hcmlibref..sas_wrkgrphier;
 If is_default=1;
 Call symput('orghier',"'"||ktrim(kleft(upcase(hierarchy_cd)))||"'");
Run;
%If %superq(orghier)= %then %let orghier='INTORG_HR';
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*  SAS_EMPLOYEE_PROFILE */
/*************************/
/*IS_DEFAULT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "IS_DEFAULT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           ADD IS_DEFAULT tinyint(1) default '0'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HIDE_EMPLOYEE_NODES*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "HIDE_EMPLOYEE_NODES";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           ADD HIDE_EMPLOYEE_NODES tinyint(1) default '0'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*DETAIL_TEMPLATE_ID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "DETAIL_TEMPLATE_ID";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           ADD DETAIL_TEMPLATE_ID varchar(200) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end ;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*DEFAULT_HIERARCHY*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "DEFAULT_HIERARCHY";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           ADD DEFAULT_HIERARCHY varchar(50) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HIDE_DEPARTMENT_NODES*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "HIDE_DEPARTMENT_NODES";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           ADD HIDE_DEPARTMENT_NODES int(1) default '0'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*PROFILEID*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "PROFILEID";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           CHANGE PROFILEID PROFILE_ID varchar(36) collate utf8_bin NOT NULL default ''
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*PROFILE_NAME*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "PROFILE_NAME";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           CHANGE PROFILE_NAME PROFILE_NM varchar(60) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*KEYATTR*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "KEYATTR";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           CHANGE KEYATTR UNIQUE_ATTR varchar(60) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*DESCRIPTION*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "DESCRIPTION";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           CHANGE DESCRIPTION PROFILE_DESC varchar(100) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADVSEARCHSTATUS*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "ADVSEARCHSTATUS";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           CHANGE ADVSEARCHSTATUS ADV_SEARCH_STATUS smallint(6) default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*LINKATTR*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "LINKATTR";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           CHANGE LINKATTR URL_LINK_ATTR varchar(60) collate utf8_bin default NULL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*PROFILE_LABEL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "PROFILE_LABEL";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           DROP PROFILE_LABEL
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*IDENTIFIER*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_EMPLOYEE_PROFILE" and
        upcase(name) = "IDENTIFIER";
quit;

%if &_colCount > 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_EMPLOYEE_PROFILE
           DROP IDENTIFIER
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*************************/
/*       ABSHMAST        */
/*************************/
%If %sysfunc(exist(&hcmlibref..ABSHMAST)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD EMPLOYEE_EMAIL varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ABSHMAST" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ABSHMAST
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*        ACTHIST        */
/*************************/
%If %sysfunc(exist(&hcmlibref..ACTHIST)) %then %do;
/*EMPLOYEE_RK*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHIST" and
        upcase(name) = "EMPLOYEE_RK";
quit;

%if &_colCount EQ 1 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHIST
           DROP EMPLOYEE_RK
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_STATUS_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHIST" and
        upcase(name) = "EMPLOYEE_STATUS_CD";
quit;

%if &_colCount EQ 1 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHIST
           MODIFY EMPLOYEE_STATUS_CD VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*FLSA_STATUS_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHIST" and
        upcase(name) = "FLSA_STATUS_CD";
quit;

%if &_colCount EQ 1 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHIST
           MODIFY FLSA_STATUS_CD VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CURRENCY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHIST" and
        upcase(name) = "CURRENCY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHIST
           ADD CURRENCY_CD VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       ACTHMAST        */
/*************************/
%If %sysfunc(exist(&hcmlibref..ACTHMAST)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD EMPLOYEE_EMAIL varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "ACTHMAST" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE ACTHMAST
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       CGRADE          */
/*************************/
%If %sysfunc(exist(&hcmlibref..CGRADE)) %then %do;
/*MIDPOINT_DIFF*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CGRADE" and
        upcase(name) = "MIDPOINT_DIFF";
quit;

%if &_colCount GT 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CGRADE
           MODIFY MIDPOINT_DIFF NUMERIC(8,3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*RANGE_SPREAD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CGRADE" and
        upcase(name) = "RANGE_SPREAD";
quit;

%if &_colCount GT 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CGRADE
           MODIFY RANGE_SPREAD NUMERIC(8,3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       CHURN           */
/*************************/
%If %sysfunc(exist(&hcmlibref..CHURN)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD EMPLOYEE_EMAIL varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "CHURN" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE CHURN
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       EMPMAST         */
/*************************/
%If %sysfunc(exist(&hcmlibref..EMPMAST)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD EMPLOYEE_EMAIL varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPMAST" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPMAST
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       EMPGEN          */
/*************************/
%If %sysfunc(exist(&hcmlibref..EMPGEN)) %then %do;
/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPGEN" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPGEN
           ADD EMPLOYEE_EMAIL VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "EMPGEN" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE EMPGEN
           ADD USER_NM VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       FACTORSITE      */
/*************************/
%If %sysfunc(exist(&hcmlibref..FACTORSITE)) %then %do;
/*MEASURE_VALUE_NO*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "FACTORSITE" and
        upcase(name) = "MEASURE_VALUE_NO";
quit;

%if &_colCount GT 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE FACTORSITE
           MODIFY MEASURE_VALUE_NO NUMERIC(15,2)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       HEADSUM         */
/*************************/
%If %sysfunc(exist(&hcmlibref..HEADSUM)) %then %do;
/*EVALUATION_RESULT_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "HEADSUM" and
        upcase(name) = "EVALUATION_RESULT_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE HEADSUM
           ADD EVALUATION_RESULT_CD varchar(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*          POS          */
/*************************/
%If %sysfunc(exist(&hcmlibref..POS)) %then %do;
/*HRPAYPER*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "POS" and
        upcase(name) = "HRPAYPER";
quit;

%if &_colCount GT 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE POS
           MODIFY HRPAYPER NUMERIC(8,2)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       SALHIST         */
/*************************/
%If %sysfunc(exist(&hcmlibref..SALHIST)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD EMPLOYEE_EMAIL varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHIST" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHIST
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       SALHSUM         */
/*************************/
%If %sysfunc(exist(&hcmlibref..SALHSUM)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EEO_CLASS_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "EEO_CLASS_CD";
quit;

%if &_colCount EQ 1 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           MODIFY EEO_CLASS_CD  VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EMPLOYEE_EMAIL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "EMPLOYEE_EMAIL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD EMPLOYEE_EMAIL varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SALHSUM" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SALHSUM
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*       TERMMAST        */
/*************************/
%If %sysfunc(exist(&hcmlibref..TERMMAST)) %then %do;
/*ADDRESS_LINE_1_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "ADDRESS_LINE_1_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD ADDRESS_LINE_1_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*ADDRESS_LINE_2_TXT*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "ADDRESS_LINE_2_TXT";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD ADDRESS_LINE_2_TXT varchar(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*CITY_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "CITY_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD CITY_NM  VARCHAR(100)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*COUNTRY_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "COUNTRY_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD COUNTRY_CD  VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EVALUATION_RESULT_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "EVALUATION_RESULT_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD EVALUATION_RESULT_CD varchar(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*POSTAL_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "POSTAL_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD POSTAL_CD   VARCHAR(20)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*STATE_REGION_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "STATE_REGION_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD STATE_REGION_CD   VARCHAR(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TERMMAST" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TERMMAST
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*          TIP          */
/*************************/
%If %sysfunc(exist(&hcmlibref..TIP)) %then %do;
/*EVALUATION_RESULT_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TIP" and
        upcase(name) = "EVALUATION_RESULT_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TIP
           ADD EVALUATION_RESULT_CD varchar(10)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*USER_NM*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "TIP" and
        upcase(name) = "USER_NM";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE TIP
           ADD USER_NM   VARCHAR(60)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/***********************
 *  Primary Key Field  *
 *     HCM_UNIQ_ID     *
 ***********************/
/*Extract the name of the hcm database.  Use &database for both hcm_uniq_id and employee_id index.*/
%local database;
%let database=%upcase(%scan(%substr(&sesetl_ConnectionOptions,%index(&sesetl_ConnectionOptions,DATABASE=)+9),1,%str( )));

proc sql;
 connect to &sesetl_EngineType (&sesetl_BestLogin &sesetl_ConnectionOptions );
 create table mysqlauto as
 select upcase(table_name) as TABLE_NAME, auto_increment
 from connection to &sesetl_EngineType
  (SELECT TABLE_NAME, AUTO_INCREMENT
   FROM INFORMATION_SCHEMA.TABLES
   WHERE UPPER(TABLE_SCHEMA)="&database.")
 where kindex(upcase(table_name),'SAS_')=0
  and upcase(table_name) not in ('HCM_VERSION_DATA','HIERMAX','HRVANLY2','FACTORSITE','HCBNCHMRK','LOCALELIST');
 disconnect from &sesetl_EngineType;
quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

%Macro uniqid (table=);
%let table=%upcase(&table);
%If %sysfunc(exist(&hcmlibref..&table.)) %then %do;
%local _colCount;
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "&table." and
        upcase(name) = "HCM_UNIQ_ID";
quit;

%local autocount;
%let autocount=;
proc sql noprint;
  select count(*) into :autocount
  from mysqlauto
  where  table_name = "&table." and
        auto_increment ne .;
quit;

%if &_colCount EQ 0 %then %do;
%if &autocount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE &table.
           ADD HCM_UNIQ_ID INT NOT NULL AUTO_INCREMENT UNIQUE FIRST
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
%else %do;
%put ERROR: Unable to add AUTO_INCREMENT column HCM_UNIQ_ID to HCM table &table. because an AUTO_INCREMENT column already exists on the table.
Remove the AUTO_INCREMENT attribute from the existing column in &table., and kick-off the migration process again.;
%abort RETURN;
%end;
%end;
%End;
%Mend;

%local tid tobs rc1 tcount;
%let tid=%sysfunc(open(work.mysqlauto));
%let tobs=%sysfunc(attrn(&tid.,NLOBS));
%let rc1=%sysfunc(close(&tid.));

/*Write the table names to macro variables (&htable1, &htable2, etc.).*/
%If &tobs.>0 %then %do;
Data _null_;
 Set mysqlauto end=last;
 Call symput('htable'||compress(put(_n_,3.)),strip(table_name));
 If last then call symput('tcount',compress(put(_n_,3.)));
Run;
 %checkErr
 %if &_tmpRC %then %goto exit;

 %Do v=1 %to &tcount.;
  %uniqid(table=&&htable&v)
 %End;
%End;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*%uniqid (table=abshist)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=abshmast)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=acthist)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=acthmast)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=apphist)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=apphmast)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=cgrade)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=churn)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=cjobs)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=comphist)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=cwrkgrp)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=empgen)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=empmast)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=grade)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=headsum)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=jobs)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=openpos)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=oposmast)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=opossum)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=pos)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=salhist)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=salhsum)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=termmast)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=tip)
 %checkErr
 %if &_tmpRC %then %goto exit;
%uniqid (table=wrkgrp)
 %checkErr
 %if &_tmpRC %then %goto exit;*/

/***********************
 *    Add index on     *
 *     EMPLOYEE_ID     *
 ***********************/
/*Extract a list of the data tables to add the EMPLOYEE_ID index.*/
proc sql;
 connect to &sesetl_EngineType (&sesetl_BestLogin &sesetl_ConnectionOptions );
 create table mysqlindex as
 select upcase(table_name) as TABLE_NAME, column_name, column_key
 from connection to &sesetl_EngineType
  (SELECT TABLE_NAME, COLUMN_NAME, COLUMN_KEY
   FROM INFORMATION_SCHEMA.COLUMNS
   WHERE UPPER(TABLE_SCHEMA)="&database." AND UPPER(COLUMN_NAME)='EMPLOYEE_ID')
 where column_key=''
  and kindex(upcase(table_name),'SAS_')=0
  and upcase(table_name) not in ('HCM_VERSION_DATA','HIERMAX','HRVANLY2');
 disconnect from &sesetl_EngineType;
quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

%local tid iobs rc tablecount;
%let tid=%sysfunc(open(work.mysqlindex));
%let iobs=%sysfunc(attrn(&tid.,NLOBS));
%let rc=%sysfunc(close(&tid.));

/*Write the table names to macro variables (&indextable1, &indextable2, etc.).*/
%If &iobs.>0 %then %do;
Data _null_;
 Set mysqlindex end=last;
 Call symput('indextable'||compress(put(_n_,3.)),strip(table_name));
 If last then call symput('tablecount',compress(put(_n_,3.)));
Run;
 %checkErr
 %if &_tmpRC %then %goto exit;


 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
%Do v=1 %to &tablecount.;
  execute (
           ALTER TABLE &&indextable&v
           ADD INDEX (EMPLOYEE_ID)
           ) by PASSTHRU ;
%End;
  disconnect from PASSTHRU ;
 quit;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*    SAS_HCM_FORMATS    */
/*************************/
/*Modify lengths of existing columns*/
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           MODIFY FMTNAME VARCHAR(32) NOT NULL,
           MODIFY FMTSTART VARCHAR(128),
           MODIFY FMTEND   VARCHAR(128),
           MODIFY FMTVALUE VARCHAR(256)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Add new columns.*/
/*DEFAULT_LENGTH_NO*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "DEFAULT_LENGTH_NO";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD DEFAULT_LENGTH_NO SMALLINT(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*EEXCL_FLG*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "EEXCL_FLG";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD EEXCL_FLG VARCHAR(1)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*HLO_VAL*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "HLO_VAL";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD HLO_VAL VARCHAR(11) DEFAULT 'M'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*LENGTH_NO*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "LENGTH_NO";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD LENGTH_NO SMALLINT(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*MAX_NO*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "MAX_NO";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD MAX_NO SMALLINT(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*MIN_NO*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "MIN_NO";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD MIN_NO SMALLINT(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*SEXCL_FLG*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "SEXCL_FLG";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD SEXCL_FLG  VARCHAR(1)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*TYPE_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "TYPE_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD TYPE_CD  VARCHAR(1)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*FUZZ_NO*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "FUZZ_NO";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD FUZZ_NO  DOUBLE
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*SOURCE_SYSTEM_CD*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from hcmcol
  where  upcase(memname) = "SAS_HCM_FORMATS" and
        upcase(name) = "SOURCE_SYSTEM_CD";
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           ALTER TABLE SAS_HCM_FORMATS
           ADD SOURCE_SYSTEM_CD   VARCHAR(3)
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Replace SAS_HCM_FORMATS with the CNTLOUT from the HCM Formats catalog (if populated).*/
Proc format cntlout=tempfmt library=_cfgfhcm;
Run;
 %checkErr
 %if &_tmpRC %then %goto exit;

%let tid=%sysfunc(OPEN(work.tempfmt));
%let nobs=%sysfunc(ATTRN(&tid.,NOBS));
%let rc=%sysfunc(CLOSE(&tid.));

%If &nobs.>0 %then %do;
/*If necessary, add WEEK range for IPAYPER format.*/
%let _colCount=;
proc sql noprint;
  select count(*) into :_colCount
  from tempfmt
  where  upcase(label) = "WEEK" and
        upcase(fmtname) = "IPAYPER" and
        type='C';
quit;

%if &_colCount EQ 0 %then %do;
 proc sql ;
  Insert into tempfmt (FMTNAME, START, END, LABEL, TYPE)
  Values ('IPAYPER', 'WK', 'WK', 'WEEK', 'C');
 quit;
%end;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Delete all rows from SAS_HCM_FORMATS.*/
proc sql ;
 connect to &sesetl_EngineType as PASSTHRU
           (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (
          DELETE FROM SAS_HCM_FORMATS
          ) by PASSTHRU;
 disconnect from PASSTHRU;
quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

Data tempfmt;
 Set tempfmt;
 Length SOURCE $3;
 If kindex(hlo,'M')=0 then hlo=kcompress(hlo)||'M';
/*Set SOURCE for DDS formats.*/
 If type='C' and fmtname in
   ('MARITAL' 'GENDER' 'EMPSTAT' 'EMPTYPE' 'ETHNIC' 'LANG' 'COUNTRY' 'LVCODE' 'COMPTYP' 'STATE' 'ACTION' 'ACTRSN' 'RECSRC' 'REJRSN'
    'JOBGRP' 'EEOCL' 'FLSA' 'EXEMPT' 'MONEY' 'APPST' 'PAYPER' 'REGTEMP' 'VETERAN' 'UNION' 'PSTAT' 'POSTY' 'GRECTYP' 'IORGS')
 then source='ETL';
Run;

 /* Populate SAS_HCM_FORMATS with the formats. */
PROC SQL;
        insert into &hcmlibref..SAS_HCM_FORMATS (FMTNAME,FMTSTART,FMTEND,FMTVALUE,DEFAULT_LENGTH_NO,EEXCL_FLG,HLO_VAL,LENGTH_NO,MAX_NO,MIN_NO,SEXCL_FLG,TYPE_CD,FUZZ_NO,SOURCE_SYSTEM_CD)
        select FMTNAME, START, END, LABEL, DEFAULT, EEXCL, HLO, LENGTH, MAX, MIN, SEXCL, TYPE, FUZZ, SOURCE
        from tempfmt;
QUIT;

%deltable(tables=tempfmt msa)
%End;
%Else %if &nobs.=0 %then %do;
 %put WARNING: The HCM Formats catalog is empty.  The default formats will be loaded into the SAS_HCM_FORMATS table.;

  proc format library=work.fmttmp;
   value $eeoshrt
      '001'-'022' = 'MANAGERS'          /* Executive, Administrative, and Managerial */
      '023'-'202' = 'PROFESSIONALS'     /* Professional Specialty */
      '203'-'242' = 'TECHNICAL'         /* Technical and related support */
      '243'-'302' = 'SALES'             /* Sales */
      '303'-'402' = 'CLERICAL'          /* Administrative Support including Clerical */
      '403'-'472' = 'SERVICE'           /* Service */
      '473'-'502' = 'FARMING'           /* Farming, Forestry, and Fishing */
      '503'-'702' = 'CRAFT'             /* Precision Production, Craft, and Repair */
      '703'-'863' = 'OPERATIVES'        /* Machine Operators, Assemblers, and Inspectors */
      '864'-'902' = 'LABORERS'          /* Handlers, Equipment Cleaners, Helpers, and Laborers */
      '000', '903'-'999'
                  = 'UNCLASSIFIED'
      ;
   run;

  /* LOS format to group service years into length of service groups */
  proc format library=work.fmttmp;
   value los
      low-<1  = ' 0 to 1 yrs'
      1-2     = ' 1+ to 3 yrs'
      3-4     = ' 3+ to 5 yrs'
      5-9     = ' 5+ to 10 yrs'
      10-high = '10+ yrs'
      ;

   run;

  /* format to group ages into age groups */
  proc format library=work.fmttmp;
   value agerng
      0-24.999   = '0 to 24 yrs'
      25-34.999  = '25 to 34 yrs'
      35-44.999  = '35 to 44 yrs'
      45-54.999  = '45 to 54 yrs'
      55-64.999  = '55 to 64 yrs'
      65-999 = '65+ yrs'
      ;
   run;

 /* Create a format for the salary ranges */
  proc format library=work.fmttmp;
   value rngefmt
    low-<0=' < Min' 0-.10='0-10' .10-.20='10-20' .20-.30='20-30'
    .30-.40='30-40' .40-.50='40-50' .50-.60='50-60'
    .60-.70='60-70' .70-.80='70-80' .80-.90='80-90'
    .90-1.00='90-100' 1.00-high='Max >';
  run;

 /* For future releases there may be a DDS reference table to create this format from */
  proc format library=work.fmttmp;
     value $evalres
        'C' = 'Constantly Exceeds Expectations'
        'F' = 'Followup Evaluation Needed'
        'M' = 'Meets Expectations'
        'O' = 'Often Exceeds Expectations'
        'S' = 'Seldom Meets Expectations'
        ' ' = 'Unknown'
        ;
  run;

  proc format library=work.fmttmp;
  value $yesno
      'N'='No'
      'Y'='Yes'
      ;
  run;

/*************************************************************************/
/* Create the internal formats                                           */
/*************************************************************************/
proc format library=work.fmttmp;
value $IACTION
"ITER" = "ITERM"         /*Int. code for Inv. Terms */
"HIRE" = "NHIRES"        /*Int. code for New Hires */
"PINC" = "PAY"           /*Int. code for Pay Changes */
"VTER" = "VTERM"         /*Int. code for Vol. Terms */
OTHER  = "UNKNOWN"
;
value $ICHURN
"DPRO" = "CHURN"        /*Int. code for Internal Movement */
OTHER  = "UNKNOWN"
;
value $IEEOCL
"1" = "Mgmt"           /*Int. code for EEO Class */
"2" = "Prof"           /*Int. code for EEO Class */
"3" = "Prof"           /*Int. code for EEO Class */
"4" = "Sales"          /*Int. code for EEO Class */
"5" = "OandC"          /*Int. code for EEO Class */
"6" = "Op"             /*Int. code for EEO Class */
"7" = "Op"             /*Int. code for EEO Class */
"8" = "Op"             /*Int. code for EEO Class */
"9" = "Op"             /*Int. code for EEO Class */
"FARMING" = "Op"       /*Int. code for EEO Class */
" " = "UNCLASSIFIED"   /*Int. code for EEO Class */
"Pro" = "Prof"         /*Int. code for EEO Class */
"Tec" = "Prof"         /*Int. code for EEO Class */
"Off" = "OandC"        /*Int. code for EEO Class */
"Ser" = "Op"           /*Int. code for EEO Class */
"Ski" = "Op"           /*Int. code for EEO Class */
OTHER  = "UNKNOWN"
;
value $IEMPSTA
"A" = "A"              /*Int. code for Active Employee Status */
"T" = "IN"             /*Int. code for Inactive Employee Status */
"M" = "IN"             /*Int. code for Inactive Employee Status */
"S" = "IN"             /*Int. code for Inactive Employee Status */
OTHER  = "UNKNOWN"
;
value $IEMPTYP
"RFT" = "FT"           /*Int. code for Employee Type Fulltime */
"RPT" = "PT"           /*Int. code for Employee Type Parttime */
"TS" = "TS"            /*Int. code for Employee Type Temporary */
"TFT" = "TFT"          /*Int. code for Employee Type Temp Fulltime */
"TPT" = "TPT"          /*Int. code for Employee Type Temp Parttime */
OTHER  = "UNKNOWN"
;
value $IETHNIC
"A" = "A"              /*Int. code for Ethnicity Asian */
"B" = "B"              /*Int. code for Ethnicity African American */
"H" = "H"              /*Int. code for Ethnicity ???? */
"U" = "U"              /*Int. code for Ethnicity Undetermined */
"W" = "W"              /*Int. code for Ethnicity Caucasion */
"N" = "N"              /*Int. code for Ethnicity North American */
OTHER  = "UNKNOWN"
;
value $IEXEMPT
"Y" = "Ex"             /*Int. code for Exempt Status */
"N" = "Nonex"          /*Int. code for Non-Exempt Status */
"U" = "UNKNOWN"        /*Int. code for Unknown Exempt Status */
OTHER  = "UNKNOWN"
;
value $IGENDER
"M" = "MALE"           /*Int. code for Male Gender */
"F" = "FEMALE"         /*Int. code for Female Gender */
OTHER  = "UNKNOWN"
;
value $IMNSTAT
"Y" = "Minority"       /*Int. code for Minority Status */
"N" = "Non-Minority"   /*Int. code for Non-Minority Status */
OTHER  = "UNKNOWN"
;
value $IPAYPER
"HR" = "HR"            /*Pay Period Codes for Hourly */
"YR" = "YR"            /*Pay Period Codes for Yearly */
"MN" = "MNTH"        /*Pay Period Codes for Monthly */
"BW" = "BW"            /*Pay Period Codes for Bi-weekly */
"SM" = "SM"            /*Pay Period Codes for Semi-mnthly */
"WK" = "WEEK"        /*Pay Period Codes for Weekly */
OTHER  = "UNKNOWN"
;
value $IREGTMP
"R" = "R"               /*Int. code for Reg/Temp */
"T" = "T"               /*Int. code for Reg/Temp */
OTHER  = "UNKNOWN"
;
value $ITERM
"ITERM" = "TERMS"       /*Int. code for Terminations */
"VTERM" = "TERMS"       /*Int. code for Terminations */
OTHER  = "UNKNOWN"
;
value $IONPYRL
"Y" = "WFOnP"           /*Saratoga on/off payroll */
"N" = "WFOffP"          /*Saratoga on/off payroll */
OTHER  = "UNKNOWN"
;
value $ISTECLS
"Executive" = "Exec"         /*Saratoga Employee Class */
"Staff" = "Staff"       /*Saratoga Employee Class */
"Manager" = "Mgr"       /*Saratoga Employee Class */
OTHER  = "UNKNOWN"
;
run;


/*Create the CNTLOUT for the formats created above*/
Proc format library=work.fmttmp cntlout=hcmformats;
Run;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Delete all rows from SAS_HCM_FORMATS.*/
proc sql ;
 connect to &sesetl_EngineType as PASSTHRU
           (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (
          DELETE FROM SAS_HCM_FORMATS
          ) by PASSTHRU;
 disconnect from PASSTHRU;
quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Write out the format entries work.hcmformats to the mysql SAS_HCM_FORMATS table.*/
 Proc sql;
  Insert into &hcmlibref..SAS_HCM_FORMATS (FMTNAME, FMTSTART, FMTEND, FMTVALUE, DEFAULT_LENGTH_NO, EEXCL_FLG, HLO_VAL, LENGTH_NO, MAX_NO, MIN_NO, SEXCL_FLG, TYPE_CD, FUZZ_NO)
  Select FMTNAME, START, END, LABEL, DEFAULT, EEXCL, kcompress(HLO)||'M', LENGTH, MAX, MIN, SEXCL, TYPE, FUZZ
  From hcmformats;
 Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Write out the msa format entries to the mysql SAS_HCM_FORMATS table.  Work.MSA was created outside of the macro (at the end).*/
Proc sql;
 Insert into &hcmlibref..SAS_HCM_FORMATS (FMTNAME, FMTSTART, FMTVALUE, TYPE_CD, FMTEND)
 Select FMTNAME, START, LABEL, TYPE, FMTEND
 From msa;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/* Update the formats catalog with the formats in SAS_HCM_FORMATS. */
%fmtupdt()

/*Clean up*/
%deltable(tables=hcmformats msa tempfmt)
%End;
 %checkErr
 %if &_tmpRC %then %goto exit;

/**********************************/
/***********New Tables*************/
/**********************************/
proc sql ;
 connect to &sesetl_EngineType as PASSTHRU
           (&sesetl_BestLogin &sesetl_ConnectionOptions );

/*************************/
/*SAS_DEFAULT_SEARCH_COLS*/
/*************************/
execute(
DROP TABLE IF EXISTS SAS_DEFAULT_SEARCH_COLS
) by PASSTHRU;

execute(
CREATE TABLE SAS_DEFAULT_SEARCH_COLS (
    TABLE_NAME  varchar(32) NOT NULL,
    COLUMN_NAME varchar(32) NOT NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*        EDUHIST        */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS EDUHIST (
     HCM_UNIQ_ID                           INT NOT NULL AUTO_INCREMENT UNIQUE,
     EDUCATION_HISTORY_ID                  VARCHAR(32) ,
     EDUHIST_SEQ_NO                        INT ,
     VALID_FROM_DT                         DATE ,
     VALID_TO_DT                           DATE ,
     EMPLOYEE_ID                           VARCHAR(32) ,
     EDUCATION_ACHIEVEMENT_NM              VARCHAR(40) ,
     TERMINAL_DEGREE_FLG                   CHAR(1) ,
     ACADEMIC_HONORS_CD                    VARCHAR(10) ,
     HONORS_PROGRAM_CD                     VARCHAR(10) ,
     OTHER_HONORS_CD                       VARCHAR(10) ,
     GRADUATION_DT                         DATE ,
     DEGREE_TYPE_CD                        VARCHAR(10) ,
     DEGREE_DESCRIPTION                    VARCHAR(255) ,
     DEGREE_CONCENTRATION_CD               VARCHAR(10) ,
     DEGREE_PROGRAM_CD                     VARCHAR(10) ,
     DEGREE_OPTION_CD                      VARCHAR(10) ,
     GRADUATING_DEGREE_CD                  VARCHAR(10) ,
     EXAM_REQUIRED_FLG                     CHAR(1) ,
     EXAM_PASSED_FLG                       CHAR(1) ,
     COMMENTS_TXT                          VARCHAR(100) ,
     ATTENDANCE_STATUS_CD                  VARCHAR(10) ,
     TUITION_ASSISTANCE_FLG                CHAR(1) ,
     MAJOR1_NM                             VARCHAR(40),
     MAJOR2_NM                             VARCHAR(40),
     MINOR1_NM                             VARCHAR(40),
     MINOR2_NM                             VARCHAR(40),
     EDUCATION_VALUE_SYSTEM_CD             VARCHAR(10) ,
     EXCESSIVE_VALUE_FLG                   CHAR(1) ,
     GOOD_STUDENT_FLG                      CHAR(1) ,
     CUMULATIVE_SUMMARY_FLG                CHAR(1) ,
     ACADEMIC_CREDIT_CD                    VARCHAR(10) ,
     COURSE_LEVEL_CD                       VARCHAR(10) ,
     CLASS_RANK_NO                         NUMERIC(15) ,
     TOTAL_STUDENTS_NO                     NUMERIC(15) ,
     GPA_NO                                NUMERIC(10,3),
     ATTENDANCE_START_DT                   DATE ,
     ENROLLMENT_STATUS_CD                  VARCHAR(10) ,
     CURRENTLY_ENROLLED_FLG                CHAR(1) ,
     STUDENT_IN_GOOD_STANDING_FLG          CHAR(1) ,
     ANTICIPATED_GRADUATION_DT             DATE ,
     ATTENDANCE_END_DT                     DATE ,
     SCHOOL_DEPT_NM                        VARCHAR(40) ,
     SCHOOL_DEPT_TYPE_CD                   VARCHAR(10) ,
     SCHOOL_TYPE_CD                        VARCHAR(10) ,
     SCHOOL_NM                             VARCHAR(40) ,
     SCHOOL_NAME_TYPE_CD                   VARCHAR(10) ,
     FICE_CD                               VARCHAR(10) ,
     INTERNET_DOMAIN_NM                    VARCHAR(40) ,
     STATE_REGION_CD_SCHOOL                VARCHAR(10) ,
     COUNTRY_CD_SCHOOL                     VARCHAR(3) ,
     ADDRESS_LINE_1_SCHOOL                 VARCHAR(100) ,
     ADDRESS_LINE_2_SCHOOL                 VARCHAR(100) ,
     CITY_NM_SCHOOL                        VARCHAR(50) ,
     POSTAL_CD_SCHOOL                      VARCHAR(20),
     ACADEMIC_CREDIT_HRS_INCLD_NO          NUMERIC(15),
     ACADEMIC_CREDIT_HRS_ATTPD_NO          NUMERIC(15),
     ACADEMIC_CREDIT_HRS_ERND_NO           NUMERIC(15),
     _LASTREC                              VARCHAR(1),
     INDEX (EMPLOYEE_ID)
                ) ENGINE=InnoDB
        ) by PASSTHRU;

/*************************/
/*        EDUVAL         */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS EDUVAL (
     HCM_UNIQ_ID                           INT NOT NULL AUTO_INCREMENT UNIQUE ,
     EDUCATION_HISTORY_ID                  VARCHAR(32) ,
     EMPLOYEE_ID                           VARCHAR(32) ,
     SCHOOL_DEPT_NM                        VARCHAR(40) ,
     EDUCATION_VALUE_TYPE_CD               VARCHAR(10) ,
     VALID_FROM_DT                         DATE ,
     VALID_TO_DT                           DATE ,
     EDUVAL_SEQ_NO                         INT ,
     SCHOOL_NM                             VARCHAR(40) ,
     EDUCATION_NUM_MIN_VAL                 NUMERIC(15,2) ,
     EDUCATION_NUM_MAX_VAL                 NUMERIC(15,2) ,
     EDUCATION_NUM_VAL                     NUMERIC(16,3) ,
     EDUCATION_STR_MIN_VAL                 VARCHAR(20) ,
     EDUCATION_STR_MAX_VAL                 VARCHAR(20) ,
     EDUCATION_STR_VAL                     VARCHAR(20) ,
     _LASTREC                              VARCHAR(1)  ,
     INDEX (EMPLOYEE_ID)
        ) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*       EMPSCORES       */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS EMPSCORES (
            HCM_UNIQ_ID                  INT NOT NULL AUTO_INCREMENT UNIQUE,
            ADDRESS_LINE_1_TXT           VARCHAR(100),
            ADDRESS_LINE_2_TXT           VARCHAR(100),
            AGE                          NUMERIC(5,2),
            AGE_RANGE                    VARCHAR(15),
            AMT_OVER_MAXIMUM             NUMERIC(13,2),
            AMT_UNDER_MINIMUM            NUMERIC(13,2),
            ANNUAL_SALARY                NUMERIC(15,2),
            BIRTH_DT                     DATE,
            CITIZENSHIP_COUNTRY_CD       VARCHAR(3),
            CITY_NM                      VARCHAR(100),
            COMRATIO                     NUMERIC(8,2),
            COUNTRY_CD                   VARCHAR(3),
            COUNTY_NM                    VARCHAR(50),
            CURRENCY_CD                  VARCHAR(3),
            DISABILITY_FLG               VARCHAR(1),
            DISCIPLINARY_ACTION_FLG      VARCHAR(1),
            EEO_CLASS_CD                 VARCHAR(10),
            EMPLOYEE_EMAIL               VARCHAR(100),
            EMPLOYEE_ID                  VARCHAR(32) NOT NULL,
            EMPLOYEE_NAME                VARCHAR(60),
            EMPLOYEE_NO                  VARCHAR(20),
            EMPLOYEE_PAID_FREQUENCY_CD   VARCHAR(3),
            EMPLOYEE_STATUS_CD           VARCHAR(10),
            EMPLOYEE_TYPE_CD             VARCHAR(10),
            ETHNICITY_CD                 VARCHAR(10),
            EVALUATION_RESULT_CD         VARCHAR(10),
            EXEMPT_STATUS_CD             VARCHAR(10),
            FLSA_STATUS_CD               VARCHAR(10),
            FTEQUIV                      NUMERIC(15,2),
            FTE_RT                       NUMERIC(8,2),
            GENDER_CD                    VARCHAR(3),
            HIRE_DT                      DATE,
            HOURLY_SALARY                NUMERIC(13,2),
            HOURS_PER_WEEK_CNT           NUMERIC(8),
            INTORG_HR                    VARCHAR(32),
            INTORG_HR1                   VARCHAR(32),
            INTORG_HR2                   VARCHAR(32),
            INTORG_HR3                   VARCHAR(32),
            INTORG_HR4                   VARCHAR(32),
            INTORG_HR5                   VARCHAR(32),
            INTORG_MGR                   VARCHAR(32),
            INTORG_MGR1                  VARCHAR(32),
            INTORG_MGR2                  VARCHAR(32),
            INTORG_MGR3                  VARCHAR(32),
            INTORG_MGR4                  VARCHAR(32),
            INTORG_MGR5                  VARCHAR(32),
            JOB_CD                       VARCHAR(10),
            JOB_GROUP_CD                 VARCHAR(10),
            JOB_GROUP_DESC               VARCHAR(255),
            JOB_TITLE_TXT                VARCHAR(100),
            LENGTH_OF_SERVICE            VARCHAR(15),
            MANAGER_ID                   VARCHAR(32),
            MANAGER_NAME                 VARCHAR(101),
            MARITAL_STATUS_CD            VARCHAR(3),
            MAXIMUM_AMT                  NUMERIC(10,2),
            MIDPOINT_AMT                 NUMERIC(10,2),
            MILITARY_EXPERIENCE_DT       DATE,
            MILITARY_EXPERIENCE_TYPE_CD  VARCHAR(10),
            MINIMUM_AMT                  NUMERIC(10,2),
            MINORITY_FLG                 VARCHAR(1),
            MNMLEV1                      VARCHAR(101),
            MNMLEV2                      VARCHAR(101),
            MNMLEV3                      VARCHAR(101),
            MNMLEV4                      VARCHAR(101),
            MNMLEV5                      VARCHAR(101),
            MONTHLY_SALARY               NUMERIC(13,2),
            ONPAYRL                      VARCHAR(1),
            OVER40                       VARCHAR(1),
            PAY_FREQUENCY_CD             VARCHAR(3),
            PAY_GRADE_FREQUENCY_CD       VARCHAR(3),
            PAY_LEVEL_CD                 VARCHAR(20),
            PAY_LEVEL_STRUCTURE_CD       VARCHAR(20),
            PAY_LEVEL_TYPE               VARCHAR(25),
            PCT_OVER_MAXIMUM             NUMERIC(8,2),
            PCT_UNDER_MINIMUM            NUMERIC(8,2),
            PERMANENCE_CD                VARCHAR(10),
            POSITION_CD                  VARCHAR(10),
            POSTAL_CD                    VARCHAR(20),
            RNG_PENE                     NUMERIC(8,2),
            SERVICE_START_DT             DATE,
            SERVICE_YEARS                NUMERIC(5,2),
            SOCIAL_SECURITY_NO           VARCHAR(20),
            STATE_REGION_CD              VARCHAR(10),
            STECLASS                     VARCHAR(10),
            UNION_CD                     VARCHAR(10),
            USER_NM                      VARCHAR(60),
            VALID_FROM_DT                DATE,
            VALID_TO_DT                  DATE,
            _LASTREC                     VARCHAR(1),
            VTSCORE                      NUMERIC(8,0),
            VTGROUP                      NUMERIC(2,0),
            P_VTERM_F1                   DOUBLE,
            INDEX (EMPLOYEE_ID)
) ENGINE=InnoDB
        ) by PASSTHRU;

/*************************/
/*    HCMMONTHFORECAST   */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS HCMMONTHFORECAST (
  HCM_UNIQ_ID int(11) NOT NULL auto_increment UNIQUE,
  NHIRES double default NULL,
  CHURN double default NULL,
  HEADCNTCHG double default NULL,
  EEO_CLASS_CD varchar(10) collate utf8_bin default NULL,
  JOB_GROUP_CD varchar(10) collate utf8_bin default NULL,
  UNION_CD varchar(10) collate utf8_bin default NULL,
  EVALUATION_RESULT_CD varchar(10) collate utf8_bin default NULL,
  PERMANENCE_CD varchar(10) collate utf8_bin default NULL,
  INTORG_MGR5 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR4 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR3 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR2 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR1 varchar(32) collate utf8_bin default NULL,
  INTORG_HR5 varchar(32) collate utf8_bin default NULL,
  VALID_DT date default NULL,
  forecast_NHIRES double default NULL,
  lower_NHIRES double default NULL,
  upper_NHIRES double default NULL,
  range_NHIRES double default NULL,
  absoluteUpper_NHIRES double default NULL,
  absoluteLower_NHIRES double default NULL,
  percentageUpper_NHIRES double default NULL,
  percentageLower_NHIRES double default NULL,
  predictstderr_NHIRES double default NULL,
  predicterr_NHIRES double default NULL,
  forecast_CHURN double default NULL,
  lower_CHURN double default NULL,
  upper_CHURN double default NULL,
  range_CHURN double default NULL,
  absoluteUpper_CHURN double default NULL,
  absoluteLower_CHURN double default NULL,
  percentageUpper_CHURN double default NULL,
  percentageLower_CHURN double default NULL,
  predictstderr_CHURN double default NULL,
  predicterr_CHURN double default NULL,
  forecast_HEADCNTCHG double default NULL,
  lower_HEADCNTCHG double default NULL,
  upper_HEADCNTCHG double default NULL,
  range_HEADCNTCHG double default NULL,
  absoluteUpper_HEADCNTCHG double default NULL,
  absoluteLower_HEADCNTCHG double default NULL,
  percentageUpper_HEADCNTCHG double default NULL,
  percentageLower_HEADCNTCHG double default NULL,
  predictstderr_HEADCNTCHG double default NULL,
  predicterr_HEADCNTCHG double default NULL,
  INTORG_HR4 varchar(32) collate utf8_bin default NULL,
  INTORG_HR3 varchar(32) collate utf8_bin default NULL,
  HLEVEL     varchar(32)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
        ) by PASSTHRU;


/*************************/
/*    HCMQTRFORECAST     */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS HCMQTRFORECAST (
  HCM_UNIQ_ID int(11) NOT NULL auto_increment UNIQUE,
  VTERM double default NULL,
  HEADCNT double default NULL,
  EEO_CLASS_CD varchar(10) collate utf8_bin default NULL,
  JOB_GROUP_CD varchar(10) collate utf8_bin default NULL,
  UNION_CD varchar(10) collate utf8_bin default NULL,
  EVALUATION_RESULT_CD varchar(10) collate utf8_bin default NULL,
  PERMANENCE_CD varchar(10) collate utf8_bin default NULL,
  INTORG_MGR5 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR4 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR3 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR2 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR1 varchar(32) collate utf8_bin default NULL,
  INTORG_HR5 varchar(32) collate utf8_bin default NULL,
  VALID_DT date default NULL,
  forecast_VTERM double default NULL,
  lower_VTERM double default NULL,
  upper_VTERM double default NULL,
  range_VTERM double default NULL,
  absoluteUpper_VTERM double default NULL,
  absoluteLower_VTERM double default NULL,
  percentageUpper_VTERM double default NULL,
  percentageLower_VTERM double default NULL,
  predictstderr_VTERM double default NULL,
  predicterr_VTERM double default NULL,
  forecast_HEADCNT double default NULL,
  lower_HEADCNT double default NULL,
  upper_HEADCNT double default NULL,
  range_HEADCNT double default NULL,
  absoluteUpper_HEADCNT double default NULL,
  absoluteLower_HEADCNT double default NULL,
  percentageUpper_HEADCNT double default NULL,
  percentageLower_HEADCNT double default NULL,
  predictstderr_HEADCNT double default NULL,
  predicterr_HEADCNT double default NULL,
  INTORG_HR4 varchar(32) collate utf8_bin default NULL,
  INTORG_HR3 varchar(32) collate utf8_bin default NULL,
  HLEVEL     varchar(32)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
        ) by PASSTHRU;


/*************************/
/*    HCMYEARFORECAST    */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS HCMYEARFORECAST (
  HCM_UNIQ_ID int(11) NOT NULL auto_increment UNIQUE,
  ITERM double default NULL,
  EEO_CLASS_CD varchar(10) collate utf8_bin default NULL,
  JOB_GROUP_CD varchar(10) collate utf8_bin default NULL,
  UNION_CD varchar(10) collate utf8_bin default NULL,
  EVALUATION_RESULT_CD varchar(10) collate utf8_bin default NULL,
  PERMANENCE_CD varchar(10) collate utf8_bin default NULL,
  INTORG_MGR5 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR4 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR3 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR2 varchar(32) collate utf8_bin default NULL,
  INTORG_MGR1 varchar(32) collate utf8_bin default NULL,
  INTORG_HR5 varchar(32) collate utf8_bin default NULL,
  VALID_DT date default NULL,
  forecast_ITERM double default NULL,
  lower_ITERM double default NULL,
  upper_ITERM double default NULL,
  range_ITERM double default NULL,
  absoluteUpper_ITERM double default NULL,
  absoluteLower_ITERM double default NULL,
  percentageUpper_ITERM double default NULL,
  percentageLower_ITERM double default NULL,
  predictstderr_ITERM double default NULL,
  predicterr_ITERM double default NULL,
  INTORG_HR4 varchar(32) collate utf8_bin default NULL,
  INTORG_HR3 varchar(32) collate utf8_bin default NULL,
  HLEVEL     varchar(32)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
        ) by PASSTHRU ;


/*************************/
/*        HRV_GBL        */
/*************************/
execute(
DROP TABLE IF EXISTS HRV_GBL
) by PASSTHRU;

execute(
CREATE TABLE HRV_GBL (
  CONT double default NULL,
  ID double default NULL,
  SEGMENT double default NULL,
  X double default NULL,
  Y double default NULL,
  COUNTRY_CD varchar(40) collate utf8_bin default NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*       HRV_STATE       */
/*************************/
execute(
DROP TABLE IF EXISTS HRV_STATE
) by PASSTHRU;

execute(
CREATE TABLE HRV_STATE (
  COUNTY_NM varchar(25) collate utf8_bin default NULL,
  SEGMENT double default NULL,
  X double default NULL,
  Y double default NULL,
  STATE_REGION_CD varchar(2) collate utf8_bin default NULL,
  COUNTRY_CD varchar(3) collate utf8_bin default NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*        HRV_US         */
/*************************/
execute(
DROP TABLE IF EXISTS HRV_US
) by PASSTHRU;

execute(
CREATE TABLE HRV_US (
  SEGMENT double default NULL,
  X double default NULL,
  Y double default NULL,
  STATE_REGION_CD varchar(2) collate utf8_bin default NULL,
  COUNTRY_CD varchar(3) collate utf8_bin default NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*     MODELSCORES       */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS MODELSCORES
(
  HCM_UNIQ_ID int(11) NOT NULL auto_increment UNIQUE,
  vterm_f double default NULL,
  lstcondt date default NULL,
  EMPLOYEE_ID varchar(32) collate utf8_bin default NULL,
  RNG_PENE06 double default NULL,
  HIRE_DT date default NULL,
  yos_act double default NULL,
  CHGAMT_cur double default NULL,
  p_vterm_f1 double default NULL,
  vtgroup double default NULL,
  chgamt_cur_LP double default NULL,
  rng_pene06_LP double default NULL,
  yos_act_LP double default NULL,
  chgamt_cur_SC double default NULL,
  rng_pene06_SC double default NULL,
  yos_act_SC double default NULL,
  VTSCORE double default NULL,
  chgamt_cur_PERCT double default NULL,
  rng_pene06_PERCT double default NULL,
  yos_act_PERCT double default NULL,
  TOT_PERCT double default NULL,
  INDEX (EMPLOYEE_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
) by PASSTHRU;

/*************************/
/*SAS_ACTIONLEVELSECURITY*/
/*************************/
execute(
DROP TABLE IF EXISTS SAS_ACTIONLEVELSECURITY
) by PASSTHRU;

execute(
CREATE TABLE SAS_ACTIONLEVELSECURITY
(
 ALS_ID             int(11)         NOT NULL    AUTO_INCREMENT,
 IDENTITY_TYPE      varchar(1)      NOT NULL    default '',
 ACTION_ID          varchar(50)     NOT NULL    default '',
 MODULE_ID          int(11)         NOT NULL    default '0',
 ACTOR_NM           varchar(50)     NOT NULL    default '',
 SECURITY_CD        int(11)                     default NULL,
 PRIMARY KEY  ( ALS_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*       SAS_CUBES       */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_CUBES
) by PASSTHRU;

execute(
CREATE TABLE SAS_CUBES
(
 CUBE_ID            int(11)         NOT NULL    default '0',
 BASE_TABLE_NM      varchar(50)     NOT NULL    default '',
 CUBE_NM            varchar(50)     NOT NULL    default '',
 CUBE_DESC          varchar(255)                default NULL,
 USER_DEFINED       tinyint(1)                  default '0',
 DIMENSIONS         varchar(512)                default NULL,
 MEAS_STATS         varchar(512)                default NULL,
 MACRO_CODE         blob                        default NULL,
 SECURITY_CD        int(11)                     default NULL,
 BUILD_DT           datetime                    default NULL,
 METADATA_ID        VARCHAR(18)     NOT NULL    DEFAULT '',
 PRIMARY KEY    ( CUBE_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*   SAS_DIMENSIONS      */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_DIMENSIONS
) by PASSTHRU;

execute(
CREATE TABLE SAS_DIMENSIONS
(
TABLE_ID int(11) NOT NULL default '0',
DIMENSION_NM varchar(32) default NULL,
DIMENSION_LBL varchar(200) default NULL,
DIMENSION_TYPE varchar(10) default NULL,
LEVEL_NM varchar(32) default NULL,
LEVEL_ORDER int(11) NOT NULL default '0',
PRIMARY KEY  (TABLE_ID,DIMENSION_NM,LEVEL_NM)
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*   SAS_DOC_PROPERTIES  */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_DOC_PROPERTIES
) by PASSTHRU;

execute(
CREATE TABLE SAS_DOC_PROPERTIES
(
 DOC_ID         VARCHAR(128)  NOT NULL,
 DOC_NM         VARCHAR(50)   NOT NULL,
 DOC_DESC       VARCHAR(255),
 CREATED_BY     VARCHAR(50)   NOT NULL,
 PROPERTY_XML   VARCHAR(8192) NOT NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*    SAS_GEOMAPPING     */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_GEOMAPPING
) by PASSTHRU;

execute(
CREATE TABLE SAS_GEOMAPPING
(
 LEVEL_NO           int(11)                     default '0',
 LEVEL_NM           varchar(60)                 default NULL,
 MAP_NM             varchar(60)                 default NULL,
 MAPLVL_NM          varchar(60)                 default NULL,
 MAP_DEFAULT_VALUE  varchar(60)                 default NULL,
 LINK_FIELD         varchar(60)                 default NULL,
 PRIMARY KEY  (LEVEL_NO)
) ENGINE=InnoDB
) by PASSTHRU;

/*****************************************************/
/*Planning Tables need to be dropped in the          */
/*correct order because of Foreign Key Constraints   */
/*****************************************************/
execute(
DROP TABLE IF EXISTS SAS_HCM_SUPP_TABLE_MEASURES
                    ,SAS_HCM_SUPPLEMENTAL_TABLE_SLICER
                    ,SAS_HCM_SUPPLEMENTAL_TABLE
                    ,SAS_HCM_PLAN_NEW_EMPLOYEE
                    ,SAS_HCM_USED_PLAN_MEASURE
                    ,SAS_HCM_PLAN
                    ,SAS_HCM_PLAN_MEASURE
                    ,SAS_HCM_PLAN_SAS_CODE
) by PASSTHRU;

/*************************/
/*     SAS_HCM_PLAN      */
/*************************/
execute(
CREATE TABLE SAS_HCM_PLAN
(
  PLAN_ID               INT(11)          NOT NULL AUTO_INCREMENT,
  PLAN_STATUS           INT(11)          NOT NULL DEFAULT '0' COMMENT 'CONSTANTS TBD',
  FM_FORM_SET_NM        VARCHAR(255)     NOT NULL,
  FM_MODEL_CD           VARCHAR(32)      NOT NULL,
  VALUE_TABLE_NM        VARCHAR(255) COMMENT 'PHYSICAL TABLE CREATED TO HOLD VALUES',
  HIER_CD               VARCHAR(32)      NOT NULL COMMENT 'FM Org hierarchy code',
  REPORT_TABLE_NM       VARCHAR(64)      DEFAULT NULL COMMENT 'table or view name generated for report',
  PRIMARY KEY  (PLAN_ID),
  UNIQUE INDEX (FM_FORM_SET_NM)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'Describes an HCM plan.'
  ) by PASSTHRU;


/******************************/
/* SAS_HCM_SUPPLEMENTAL_TABLE */
/******************************/
execute(
CREATE TABLE SAS_HCM_SUPPLEMENTAL_TABLE
(
  SUPPLEMENTAL_TABLE_ID INT(11)      NOT NULL AUTO_INCREMENT,
  PLAN_ID               INT(11)      NOT NULL,
  ROW_DIM_CD            VARCHAR(32)  NOT NULL COMMENT 'FM DIM ON ROWS',
  ROW_HIER_CD           VARCHAR(32)  NOT NULL,
  COL_DIM_CD            VARCHAR(32)  NOT NULL COMMENT 'FM DIM ON COLUMNS',
  CREATE_USERID         VARCHAR(32)  NOT NULL,
  CREATE_DT             DATE         NOT NULL,
  PRIMARY KEY  (SUPPLEMENTAL_TABLE_ID),
  FOREIGN KEY (PLAN_ID) REFERENCES SAS_HCM_PLAN (PLAN_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'Describes an Excel Supplemental Planning Table.'
  ) by PASSTHRU;

/******************************/
/*  SAS_HCM_USED_PLAN_MEASURE */
/******************************/
execute(
CREATE TABLE SAS_HCM_USED_PLAN_MEASURE
(
  MEASURE_ID        int(11)      NOT NULL AUTO_INCREMENT,
  PLAN_ID           int(11)      NOT NULL,
  MEASURE_NM        varchar(255) NOT NULL,
  SOURCE_TABLE_NM   varchar(255) DEFAULT NULL COMMENT 'HCM source table which must have employee_id key, or null if none',
  SOURCE_COLUMN_NM  varchar(255) DEFAULT NULL COMMENT 'HCM source table column, or null if none',
  SAS_MACRO_VAR_NM  varchar(32)  DEFAULT NULL COMMENT 'macro var name passed to sas code',
  EDIT_RULE         int(1)       NOT NULL DEFAULT '0' COMMENT '0=always, 1=never, 2=new employee only',
  COLUMN_OPERATION  int(11)      NOT NULL DEFAULT '0' COMMENT '0=none, 1=totalled, 2=averaged',
  MEASURE_DATA_TYPE int(11)      NOT NULL COMMENT '0=number, 1=string, 2=date, 3=timestamp',
  FM_MEASURE        int(1)       NOT NULL DEFAULT '0' COMMENT '0 for hcm-defined measures',
  PROMPT_TYPE       int(1)       NOT NULL DEFAULT '0' COMMENT '0=none, 1=entry field, 2=table/col lookup, 3=sas target',
  SLICER_DEPENDENT  int(1)       NOT NULL DEFAULT  0  COMMENT 'store based on FM slicer dim values if true',
  FM_DIM_CD         varchar(32)  DEFAULT NULL         COMMENT 'which fm dim this measure will be associated with',
  PRIMARY KEY  (MEASURE_ID),
  FOREIGN KEY (PLAN_ID) REFERENCES SAS_HCM_PLAN (PLAN_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'A measure used in a plans supplementals.'
) by PASSTHRU;

/*************************/
/*  SAS_HCM_PLAN_MEASURE */
/*************************/
execute(
CREATE TABLE SAS_HCM_PLAN_MEASURE
(
  MEASURE_ID        int(11)      NOT NULL AUTO_INCREMENT,
  MEASURE_NM        varchar(255) NOT NULL,
  SOURCE_TABLE_NM   varchar(255) DEFAULT NULL COMMENT 'HCM source table which must have employee_id key, or null if none',
  SOURCE_COLUMN_NM  varchar(255) DEFAULT NULL COMMENT 'HCM source table column, or null if none',
  SAS_MACRO_VAR_NM  varchar(32)  DEFAULT NULL COMMENT 'macro var name passed to sas code',
  ENABLED           int(1)       NOT NULL DEFAULT '1' COMMENT 'to allow disable w/o losing definition',
  EDIT_RULE         int(1)       NOT NULL DEFAULT '0' COMMENT '0=always, 1=never, 2=new employee only',
  COLUMN_OPERATION  int(11)      NOT NULL DEFAULT '0' COMMENT '0=none, 1=totalled, 2=averaged',
  MEASURE_DATA_TYPE int(11)      NOT NULL COMMENT '0=number, 1=string, 2=date, 3=timestamp',
  PROMPT_TYPE       int(1)       NOT NULL DEFAULT '0' COMMENT '0=none, 1=entry field, 2=table/col lookup, 3=sas target',
  SLICER_DEPENDENT  int(1)       NOT NULL DEFAULT  0  COMMENT 'store based on FM slicer dim values if true',
  PRIMARY KEY  (MEASURE_ID),
  UNIQUE INDEX (MEASURE_NM)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'A measure available for planning.'
) by PASSTHRU;

/*************************************/
/*SAS_HCM_SUPP_TABLE_MEASURES*/
/*************************************/
execute(
CREATE TABLE SAS_HCM_SUPP_TABLE_MEASURES
(
  SUPPLEMENTAL_TABLE_ID int(11)               NOT NULL,
  MEASURE_ID            int(11)               NOT NULL,
  UNIQUE INDEX (SUPPLEMENTAL_TABLE_ID, MEASURE_ID),
  FOREIGN KEY (SUPPLEMENTAL_TABLE_ID) REFERENCES SAS_HCM_SUPPLEMENTAL_TABLE (SUPPLEMENTAL_TABLE_ID),
  FOREIGN KEY (MEASURE_ID) REFERENCES SAS_HCM_USED_PLAN_MEASURE (MEASURE_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'Associates used measures with a supplemental table.'
  ) by PASSTHRU;

/*************************************/
/*     SAS_HCM_PLAN_NEW_EMPLOYEE     */
/*************************************/
execute(
CREATE TABLE SAS_HCM_PLAN_NEW_EMPLOYEE
(
  PLAN_ID         INT(11)      NOT NULL,
  EMPLOYEE_ID     VARCHAR(32)  NOT NULL,
  PARENT_ID       VARCHAR(32)  NOT NULL,
  NAME            VARCHAR(60)  NOT NULL,
  UNIQUE INDEX (PLAN_ID, EMPLOYEE_ID),
  FOREIGN KEY (PLAN_ID) REFERENCES SAS_HCM_PLAN (PLAN_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'New employees for a specific plan.'
  ) by PASSTHRU;

/*************************************/
/*     SAS_HCM_PLAN_SAS_CODE         */
/*************************************/
execute(
CREATE TABLE SAS_HCM_PLAN_SAS_CODE
(
  SAS_CODE_ID  INT(11)     NOT NULL AUTO_INCREMENT,
  SAS_CODE     MEDIUMTEXT  NOT NULL,
  RUN_DATE     DATETIME    DEFAULT NULL,
  RETURN_CODE  INT(11)     NOT NULL DEFAULT '0',
  SAS_LOG      MEDIUMTEXT  DEFAULT NULL,
  PRIMARY KEY  (SAS_CODE_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'SAS code used for calculating new employee target measures.'
  ) by PASSTHRU;

/*************************************/
/* SAS_HCM_SUPPLEMENTAL_TABLE_SLICER */
/*************************************/
execute(
CREATE TABLE SAS_HCM_SUPPLEMENTAL_TABLE_SLICER (
  SUPPLEMENTAL_TABLE_ID int(11) NOT NULL,
  DIM_CD varchar(32) NOT NULL,
  HIER_CD varchar(32) NOT NULL,
  VT_COLUMN_NM varchar(32) NOT NULL,
  KEY (SUPPLEMENTAL_TABLE_ID),
  FOREIGN KEY (SUPPLEMENTAL_TABLE_ID) REFERENCES SAS_HCM_SUPPLEMENTAL_TABLE (SUPPLEMENTAL_TABLE_ID)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_bin
COMMENT 'records FM slicers defined for this supplemental table'
  ) by PASSTHRU;

/*************************/
/*  SAS_LOCALE_LIST      */
/*************************/
execute(
CREATE TABLE IF NOT EXISTS SAS_LOCALE_LIST
(
 LOCALE_LANGUAGE_CD       VARCHAR(2) NOT NULL,
 LOCALE_COUNTRY_CD        VARCHAR(2) NOT NULL   default '',
 SOURCE_SYSTEM_CD         VARCHAR(3) NOT NULL   default 'PRP',
 DDS_LANGUAGE_CD          VARCHAR(3),
 DEFAULT_LANGUAGE_FLG     VARCHAR(1),
 PRIMARY KEY  (LOCALE_LANGUAGE_CD,LOCALE_COUNTRY_CD)
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*     SAS_INFOMAPS      */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_INFOMAPS
) by PASSTHRU;

execute(
CREATE TABLE SAS_INFOMAPS
(
 IMAP_ID        int(11)         NOT NULL    default '0',
 IMAP_TYPE      varchar(1)      NOT NULL    default '',
 SOURCE_NM      varchar(50)     NOT NULL    default '',
 IMAP_NM        varchar(50)     NOT NULL    default '',
 IMAP_DESC      varchar(255)                default NULL,
 SECURITY_CD    int(11)                     default NULL,
 BUILD_DT       datetime                    default NULL,
 IMAP_CD        varchar(10)     NOT NULL    default '',
 METADATA_ID    varchar(18)     NOT NULL    default '',
 PRIMARY KEY    ( IMAP_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/* SAS_MEASURE_FORMULAS  */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_MEASURE_FORMULAS
) by PASSTHRU;

execute(
CREATE TABLE SAS_MEASURE_FORMULAS
(
  MEASURE_ID int(11) NOT NULL auto_increment,
  VARIABLE varchar(32) collate utf8_bin NOT NULL,
  SAS_LENGTH varchar(30) collate utf8_bin NOT NULL default '30',
  SAS_FORMAT varchar(30) collate utf8_bin NOT NULL,
  SAS_LABEL varchar(255) collate utf8_bin NOT NULL default '',
  FORMULA varchar(255) collate utf8_bin NOT NULL default '',
  TYPE int(11) NOT NULL default '0',
  USED_VARS varchar(255) collate utf8_bin NOT NULL default '',
  CATEGORY int(1) NOT NULL default '0',
  DESCRIPTION varchar(510) collate utf8_bin NOT NULL default '',
  PRIMARY KEY  (MEASURE_ID)
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*   SAS_MODULEACTIONS   */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_MODULEACTIONS
) by PASSTHRU;

execute(
CREATE TABLE SAS_MODULEACTIONS
(
 ACTION_ID      varchar(50)     NOT NULL    default '0',
 MODULE_ID      int(11)         NOT NULL    default '0',
 ACTION_NM      varchar(50)     NOT NULL    default '',
 ACTION_DESC    varchar(255)                default NULL,
 SECURITY_CD    int(11)                     default NULL,
 ACTION_IMG     varchar(50)                 default NULL,
 PRIMARY KEY  ( ACTION_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*      SAS_MODULES      */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_MODULES
) by PASSTHRU;

execute(
CREATE TABLE SAS_MODULES
(
 MODULE_ID      int(11)         NOT NULL    default '0',
 MODULE_NM      varchar(50)     NOT NULL    default '',
 MODULE_DESC    varchar(255)                default NULL,
 SECURITY_CD    int(11)                     default NULL,
 IS_CUSTOM      int(1)                      default '0',
 PARENT_MODULE  int(11)         NOT NULL    default '0',
 PRIMARY KEY ( MODULE_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*   SAS_PORTAL_CONTENT  */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_PORTAL_CONTENT
) by PASSTHRU;

execute(
CREATE TABLE SAS_PORTAL_CONTENT
(
 ENTRY_ID          int(11)         NOT NULL    AUTO_INCREMENT,
 USER_ID           varchar(20)     NOT NULL,
 SBIP_URL          varchar(512)    NOT NULL,
 SHOW_DESC         tinyint(1)      NOT NULL    default '0',
 SHOW_SBIP         tinyint(1)      NOT NULL    default '0',
 ENTRY_ORDER       int(11)         NOT NULL    default '0',
 ENTRY_TYPE        int(11)         NOT NULL,
 PRIMARY KEY  (ENTRY_ID)
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/* SAS_ROWLEVELSECURITY  */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_ROWLEVELSECURITY
) by PASSTHRU;

execute(
CREATE TABLE SAS_ROWLEVELSECURITY
(
 RLS_ID             integer         NOT NULL    AUTO_INCREMENT,
 FILTER_TYPE        varchar(1)      NOT NULL    default '',
 IDENTITY_TYPE      varchar(1)      NOT NULL    default '',
 ACTOR_NM           varchar(50)     NOT NULL    default '',
 TABLE_NM           varchar(50)     NOT NULL    default '',
 COLUMN_NM          varchar(50)     NOT NULL    default '',
 FILTER_OPERATOR    varchar(2)      NOT NULL    default '',
 FILTER_VALUE       varchar(255)    NOT NULL    default '',
 DATA_TYPE          varchar(1)      NOT NULL    default '',
 PRIMARY KEY  ( RLS_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*     SAS_TEMPLATES     */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_TEMPLATES
) by PASSTHRU;

execute(
CREATE TABLE SAS_TEMPLATES
(
   TEMPLATE_ID    int(11)         NOT NULL    AUTO_INCREMENT,
   TEMPLATE_NM    varchar(50)     NOT NULL    default '',
   TEMPLATE_DESC  varchar(255)                default NULL,
   FILE_NM        varchar(50)     NOT NULL    default '',
   IS_DEFAULT     tinyint(1)      NOT NULL    default '0',
   TEMPLATE_TYPE  varchar(50)     NOT NULL    default '',
   PRIMARY KEY    ( TEMPLATE_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/* SAS_USER_MODULE_SETTINGS  */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_USER_MODULE_SETTINGS
) by PASSTHRU;

execute(
CREATE TABLE SAS_USER_MODULE_SETTINGS
(
 USERID                varchar(20)  NOT NULL,
 MODULETYPE            varchar(20)  NOT NULL,
 PROPERTYNAME          varchar(30)  NOT NULL,
 PROPERTYVALUE         varchar(32)  NOT NULL,
 PROPERTYDESCRIPTION   varchar(100) NOT NULL,
 DOC_ID                varchar(128) NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*  SAS_USER_OPTIONS     */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_USER_OPTIONS
) by PASSTHRU;

execute(
CREATE TABLE SAS_USER_OPTIONS
(
 USER_ID        varchar(60)                 default NULL,
 MODULE_ID      int(11)                     default '0',
 OPTION_NM      varchar(60)                 default NULL,
 OPTION_VALUE   varchar(255)                default NULL
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*   SAS_XACTIONPARAMS   */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_XACTIONPARAMS
) by PASSTHRU;

execute(
CREATE TABLE SAS_XACTIONPARAMS
(
 PARAM_ID           int(11)         NOT NULL    AUTO_INCREMENT,
 ACTION_ID          int(11)         NOT NULL    default '0',
 PARAM_NM           varchar(50)     NOT NULL    default '',
 COLUMN_NM          varchar(50)     NOT NULL    default '',
 PRIMARY KEY  ( PARAM_ID )
) ENGINE=InnoDB
) by PASSTHRU;

/*************************/
/*     SAS_XACTIONS      */
/*************************/
execute(
DROP TABLE IF EXISTS SAS_XACTIONS
) by PASSTHRU;

execute(
CREATE TABLE SAS_XACTIONS
(
 ACTION_ID          int(11)         NOT NULL    AUTO_INCREMENT,
 MODULE_ID          int(11)         NOT NULL    default '0',
 PROFILE_KEY        varchar(36)                 default NULL,
 ACTION_LABEL       varchar(255)    NOT NULL    default  '',
 ACTION_TYPE        int(11)         NOT NULL    default '0',
 ACTION_LINK        varchar(512)    NOT NULL    default '',
 ACTION_EMP_LEVEL   tinyint(1)                  default NULL,
 ACTION_DEPT_LEVEL  tinyint(1)                  default NULL,
 ACTION_IN_USE      tinyint(1)      NOT NULL    default '0',
 PRIMARY KEY  ( ACTION_ID )
) ENGINE=InnoDB
) by PASSTHRU;

disconnect from PASSTHRU;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/***********Modify Data Values: Existing Tables*************/
%local nFoundARecord;
/*************************/
/*   SAS_LOCALE_LIST     */
/*************************/
%If %sysfunc(exist(&hcmlibref..localelist)) %then %do;
Data locale (keep=LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD SOURCE_SYSTEM_CD DDS_LANGUAGE_CD DEFAULT_LANGUAGE_FLG);
 Length LOCALE_LANGUAGE_CD LOCALE_COUNTRY_CD $ 2 SOURCE_SYSTEM_CD DDS_LANGUAGE_CD $ 3 DEFAULT_LANGUAGE_FLG $ 1;
 Retain SOURCE_SYSTEM_CD 'PRP' DDS_LANGUAGE_CD '' DEFAULT_LANGUAGE_FLG '';
 Set &hcmlibref..localelist;
 If (index(locale,'_')) then do;
  locale_language_cd=scan(locale,1,'_');
  locale_country_cd=scan(locale,2,'_');
 End;
 Else do;
  locale_language_cd=strip(locale);
  locale_country_cd='';
 End;
Run;
 %checkErr
 %if &_tmpRC %then %goto exit;

Proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           DELETE FROM SAS_LOCALE_LIST
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

Proc sql;
 Insert into &hcmlibref..SAS_LOCALE_LIST (LOCALE_LANGUAGE_CD, LOCALE_COUNTRY_CD, SOURCE_SYSTEM_CD, DDS_LANGUAGE_CD, DEFAULT_LANGUAGE_FLG)
 Select LOCALE_LANGUAGE_CD, LOCALE_COUNTRY_CD, SOURCE_SYSTEM_CD, DDS_LANGUAGE_CD, DEFAULT_LANGUAGE_FLG
 From locale;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

Proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           DROP TABLE LOCALELIST
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;
%End;

/**************************/
/* SAS_DEFAULT_PROPERTIES */
/**************************/
/*Delete the path to the Properties files so that
  the default path can be set when %setlocs is called below.*/
Proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           UPDATE SAS_DEFAULT_PROPERTIES
           SET DEFAULT_VALUE=''
           WHERE PROPERTY_NM='PropFilePath'
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/* set the locale macro variables */
%setlocs

/*********************************************************************
 * Extract the locales from the conform.code_language table and      *
 * flag these locales with "DDS" in the SAS_LOCALE_LIST table.       *
 *********************************************************************/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from conform.CODE_LANGUAGE;
quit;
%if &nFoundARecord > 0 %then %do;
/*Extract the current locales from conform.code_language.*/
Proc sort data=conform.code_language out=language nodupkey;
 By descending default_language_flg locale_language_cd locale_country_cd;
 Where datetime() between valid_from_dttm and valid_to_dttm;
Run;

Proc sql;
/*Extract the DDS Locales where locale_country_cd is not populated
  and there is a corresponding locale in SAS_LOCALE_LIST.*/
Create table list1 as
Select ll.locale_language_cd, ll.locale_country_cd, lg.language_cd as ddscd length=3,
       lg.default_language_flg as ddsflg length=1
From &hcmlibref..sas_locale_list ll, language lg
Where lg.locale_country_cd='' and lowcase(lg.locale_language_cd)=lowcase(ll.locale_language_cd)
Order by locale_language_cd, locale_country_cd;

/*Extract the DDS Locales where locale_country_cd is populated
  and there is a corresponding locale in SAS_LOCALE_LIST.*/
Create table list2 as
Select ll.locale_language_cd, ll.locale_country_cd, lg.language_cd as ddscd length=3,
       lg.default_language_flg as ddsflg length=1
From &hcmlibref..sas_locale_list ll, language lg
Where lg.locale_country_cd ne ''
 and lowcase(lg.locale_language_cd)=lowcase(ll.locale_language_cd)
 and lowcase(lg.locale_country_cd)=lowcase(ll.locale_country_cd)
Order by locale_language_cd, locale_country_cd;
Quit;

/*Combine the two lists by merging to ensure that there are no duplicates.*/
Data ddslist;
 Merge list1 list2;
 By locale_language_cd locale_country_cd;
Run;

%let dsid=%sysfunc(open(work.ddslist));
%let obs=%sysfunc(attrn(&dsid,NLOBS));
%let rc=%sysfunc(close(&dsid));

%If &obs. > 0 %then %do;
/*Determine if there are any changes between the current DDS list and the new DDS list.*/
%local localechange;
Data _null_;
 Merge &hcmlibref..sas_locale_list (in=c where=(source_system_cd='DDS')) ddslist (in=d);
 By locale_language_cd locale_country_cd;
 If not (c and d) then call symput('localechange','y');
 Else if lowcase(dds_language_cd) ne lowcase(ddscd) or lowcase(default_language_flg) ne lowcase(ddsflg) then call symput('localechange','y');
Run;

/*If there are changes then proceed.*/
%If &localechange.=y %then %do;
Data locale_list;
 Set &hcmlibref..sas_locale_list;
 By locale_language_cd locale_country_cd;
 source_system_cd='PRP';
 drop dds_language_cd default_language_flg;
Run;

/*Create the new updated locale list.*/
Data sas_locale_list;
 Merge locale_list (in=l) ddslist (in=d);
 By locale_language_cd locale_country_cd;
 If l and d then source_system_cd='DDS';
Run;

%let dsid=%sysfunc(open(work.sas_locale_list));
%let obs=%sysfunc(attrn(&dsid,NLOBS));
%let rc=%sysfunc(close(&dsid));

 %If &obs. > 0 %then %do;
Proc sql ;
  connect to &sesetl_EngineType as PASSTHRU
            (&sesetl_BestLogin &sesetl_ConnectionOptions );
  execute (
           DELETE FROM SAS_LOCALE_LIST
           ) by PASSTHRU ;
  disconnect from PASSTHRU ;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;

/*Write out the updated locale list.*/
Proc sql;
 Insert into &hcmlibref..SAS_LOCALE_LIST (LOCALE_LANGUAGE_CD, LOCALE_COUNTRY_CD, SOURCE_SYSTEM_CD, DDS_LANGUAGE_CD, DEFAULT_LANGUAGE_FLG)
 Select LOCALE_LANGUAGE_CD, LOCALE_COUNTRY_CD, SOURCE_SYSTEM_CD, DDSCD, DDSFLG
 From sas_locale_list;
Quit;
 %checkErr
 %if &_tmpRC %then %goto exit;
 %End; /*%If &obs. > 0*/
%End; /*%If &localechange.=y*/
%End; /*%If &obs. > 0*/
%End; /*%if &nFoundARecord > 0*/

/* Define the label macros.*/
%readlbls(basedir=&propspth,basefile=&lblprops,baseext=properties,slash=&slash,defaultpath=)

/*************************/
/* SAS_EMPLOYEE_PROFILE  */
/*************************/
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(
UPDATE SAS_EMPLOYEE_PROFILE
SET IS_DEFAULT=1, DETAIL_TEMPLATE_ID='2', DEFAULT_HIERARCHY=&orghier., HIDE_EMPLOYEE_NODES = 1
WHERE profile_id='EmployeeProfile'
 ) by PASSTHRU;

 execute(
UPDATE SAS_EMPLOYEE_PROFILE
SET IS_DEFAULT=0, DETAIL_TEMPLATE_ID='2', DEFAULT_HIERARCHY=&orghier., HIDE_EMPLOYEE_NODES = 1
WHERE profile_id != 'EmployeeProfile'
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;

/*************************/
/*SAS_DEFAULT_PROPERTIES */
/*************************/
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
execute(Delete from SAS_DEFAULT_PROPERTIES where MODULE_TYPE in ('Geo','EEP')) by PASSTHRU;
execute(Delete from SAS_DEFAULT_PROPERTIES where MODULE_TYPE='Org' AND PROPERTY_NM IN
('Employee_Status_Cd_Column','Employee_Status_Cd_Value','Unassigned','HierarchySource','IsModified','Measures', 'spGUID', 'HideEmployee')
 ) by PASSTHRU;
execute(Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('Geo',1,'GeoMapReferenceTableId','11',"%getlabel(DEFAULTPROPDESC18,Geo Reference Table)"),
('Geo',1,'StartColor','#ADBFEE',"%getlabel(DEFAULTPROPDESC20,Geo Map Start Color)"),
('Geo',1,'EndColor','#515CC9',"%getlabel(DEFAULTPROPDESC17,Geo Map End Color)"),
('Geo',1,'MaplegendLabel','Employee Population',"%getlabel(DEFAULTPROPDESC19,Map Legend Label)"),
('Geo',1,'geo_default_searchable_columns','ANNUAL_SALARY::HIRE_DT::AGE::SERVICE_YEARS',"%getlabel(DEFAULTPROPDESC21,Geo Default Searchable Columns)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPScatterplotResponse_Y';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPScatterplotResponse_Y','ANNUAL_SALARY',"%getlabel(DEFAULTPROPDESC11,Scatterplot Graph response-Y variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPBarCategory';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph', 1, 'BIPBarCategory', &orghier., "%getlabel(DEFAULTPROPDESC1,Bar Graph Category variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPPieCategory';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph', 1, 'BIPPieCategory', &orghier., "%getlabel(DEFAULTPROPDESC8,Pie Graph Category variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPLineCategory';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph', 1, 'BIPLineCategory', &orghier., "%getlabel(DEFAULTPROPDESC6,Line Graph Category variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPBarLine_YResponse';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPBarLine_YResponse','AGE::MEAN',"%getlabel(DEFAULTPROPDESC4,Bar Line - Y  response variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPBarLineResponse';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPBarLineResponse','COUNT',"%getlabel(DEFAULTPROPDESC3,Bar Line Graph response variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPLineResponse';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPLineResponse','COUNT',"%getlabel(DEFAULTPROPDESC7,Line Graph response variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPPieResponse';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPPieResponse','COUNT',"%getlabel(DEFAULTPROPDESC9,Pie Graph response variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPScatterplotResponse_X';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPScatterplotResponse_X','AGE',"%getlabel(DEFAULTPROPDESC10,Scatterplot Graph response-X variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPBarLineCategory';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph', 1, 'BIPBarLineCategory', &orghier., "%getlabel(DEFAULTPROPDESC2,Bar Line Graph Category variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_DEFAULT_PROPERTIES
 where MODULE_TYPE='BIPGraph' and PROPERTY_NM='BIPBarResponse';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute( Insert into SAS_DEFAULT_PROPERTIES(MODULE_TYPE,DATA_TYPE,PROPERTY_NM,DEFAULT_VALUE,PROPERTY_DESC) Values
('BIPGraph',1,'BIPBarResponse','COUNT',"%getlabel(DEFAULTPROPDESC5,Bar Graph response variable)")
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

/*************************/
/*   SAS_WRKGRPHIER      */
/*************************/
/*Seed values to new columns*/
Data wrkgrphier;
 Set &hcmlibref..sas_wrkgrphier end=last;
 By hierarchy_cd;
 If last then IS_SECURITY_DEFAULT=1;
 Else IS_SECURITY_DEFAULT=0;
 IMG_EMP_NODE='employeeImg.gif';
 IMG_DEPT_NODE='orgImg.gif';
 SECURITY_LINK_FIELD=HIERARCHY_CD;
 SECURITY_HIERARCHY=0;
 DIMENSION_CD='ORG';
Run;

/* Delete rows from SAS_WRKGRPHIER. */
proc sql;
connect to &sesetl_EngineType AS PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
execute(delete from SAS_WRKGRPHIER) by PASSTHRU;
quit;

PROC APPEND BASE=&hcmlibref..SAS_WRKGRPHIER DATA=wrkgrphier force; RUN;

/*************************/
/*   SAS_HCMMETACOLUMN   */
/*************************/
/*Update HCM column registrations for currently registered HCM tables.*/
%local rtcount;
Data _null_;
 Set &hcmlibref..SAS_HCMMETATABLE end=last;
 Call symput('rhtable'||compress(put(_n_,3.)),strip(table_nm));
 If last then call symput('rtcount',compress(put(_n_,3.)));
Run;
 %checkErr
 %if &_tmpRC %then %goto exit;

%Do y=1 %to &rtcount.;
 %If %sysfunc(exist(&hcmlibref..&&rhtable&y)) %then %updatedd(tablenm=&&rhtable&y);
 %checkErr
  %if &_tmpRC %then %do;
   %Put ERROR: There has been a failure in updating the column registrations for table %qupcase(&&rhtable&y) in HCM.  See the SAS log for details.;
   %goto exit;
  %end;
%End;

/*************************/
/*   SAS_HCMMETATABLE    */
/*************************/
proc sql noprint;
 select kcompress(put(max(id),6.)) into :maxid
 from &hcmlibref..SAS_HCMMETATABLE;
quit;

/*HRV_GBL*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='HRV_GBL';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'HRV_GBL',"%getlabel(HRV_GBL,Global Map Table)",0,0,0,1,1,1)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=HRV_GBL)
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(update SAS_HCMMETACOLUMN set COLUMN_FMT='$COUNTRY.' where HCM_COLUMN_NM='COUNTRY_CD' and TABLE_ID=&maxid.) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

/*HRV_STATE*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='HRV_STATE';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'HRV_STATE',"%getlabel(HRV_STATE,State Map Table)",0,0,0,1,1,1)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=HRV_STATE)
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(update SAS_HCMMETACOLUMN set COLUMN_FMT='$COUNTRY.' where HCM_COLUMN_NM='COUNTRY_CD' and TABLE_ID=&maxid.) by PASSTHRU;
 execute(update SAS_HCMMETACOLUMN set COLUMN_FMT='$STATE.'   where HCM_COLUMN_NM='STATE_REGION_CD' and TABLE_ID=&maxid.) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

/*HRV_US*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='HRV_US';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'HRV_US',"%getlabel(HRV_US,US Map Table)",0,0,0,1,1,1)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=HRV_US)
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(update SAS_HCMMETACOLUMN set COLUMN_FMT='$COUNTRY.' where HCM_COLUMN_NM='COUNTRY_CD' and TABLE_ID=&maxid.) by PASSTHRU;
 execute(update SAS_HCMMETACOLUMN set COLUMN_FMT='$STATE.'   where HCM_COLUMN_NM='STATE_REGION_CD' and TABLE_ID=&maxid.) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

/*EDUHIST*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='EDUHIST';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'EDUHIST',"%getlabel(EDUHIST,Education History Detail Table)",0,1,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=EDUHIST)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*EMPSCORES*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='EMPSCORES';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'EMPSCORES',"%getlabel(EMPSCORES,Employee Scoring Master Table)",1,1,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=EMPSCORES)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*MODELSCORES*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='MODELSCORES';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'MODELSCORES',"%getlabel(MODELSCORES,Employee Scoring Detail Table)",0,0,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=MODELSCORES)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*HCMMONTHFORECAST*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='HCMMONTHFORECAST';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'HCMMONTHFORECAST',"%getlabel(HCMMONTHFORECAST,Headcount Forecast Table - Month Periods)",0,0,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=HCMMONTHFORECAST)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*HCMQTRFORECAST*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='HCMQTRFORECAST';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'HCMQTRFORECAST',"%getlabel(HCMQTRFORECAST,Headcount Forecast Table - Quarter Periods)",0,0,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=HCMQTRFORECAST)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*HCMYEARFORECAST*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='HCMYEARFORECAST';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'HCMYEARFORECAST',"%getlabel(HCMYEARFORECAST,Headcount Forecast Table - Year Periods)",0,0,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=HCMYEARFORECAST)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*EDUVAL*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_HCMMETATABLE
 where upcase(TABLE_NM)='EDUVAL';
quit;
%if &nFoundARecord EQ 0 %then %do;
%let maxid=%eval(&maxid.+1);
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(Insert into SAS_HCMMETATABLE (ID, TABLE_NM, TABLE_DESC, IS_MASTER, IS_HISTORY, IS_HIDDEN, USE_FMT, BLDORDER, IS_GEO_TABLE) Values
(&maxid.,'EDUVAL',"%getlabel(EDUVAL,Education Assessment Detail Table)",0,1,0,1,1,0)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%updatedd(tablenm=EDUVAL)
%attribdd(tableid=&maxid.)
 %checkErr
  %if &_tmpRC %then %goto exit;
%End;

/*************************/
/*   SAS_USER_EMPLOYEE   */
/*************************/
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute(
UPDATE SAS_USER_EMPLOYEE
SET user_id='sasdemo'
WHERE user_id='SAS Demo User'
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;

/*********************************************************************************************/

/***********Seed Data Values: New Tables*************/
proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
/*************************/
/*SAS_DEFAULT_SEARCH_COLS*/
/*************************/
 execute (Insert into SAS_DEFAULT_SEARCH_COLS (TABLE_NAME,COLUMN_NAME) Values
('empmast', 'EMPLOYEE_NAME'),
('empmast', 'EMPLOYEE_ID'),
('empmast', &orghier.)
) by PASSTHRU;
/*************************/
/*        HRV_GBL        */
/*************************/
Insert into &hcmlibref..HRV_GBL (CONT,ID,SEGMENT,X,Y,COUNTRY_CD)
Select CONT,ID,SEGMENT,X,Y,COUNTRY_CD
From _hcmmap.HRV_GBL;
/*************************/
/*       HRV_STATE       */
/*************************/
Insert into &hcmlibref..HRV_STATE (COUNTY_NM,SEGMENT,X,Y,STATE_REGION_CD,COUNTRY_CD)
Select COUNTY_NM,SEGMENT,X,Y,STATE_REGION_CD,COUNTRY_CD
From _hcmmap.HRV_STATE;
/*************************/
/*        HRV_US         */
/*************************/
Insert into &hcmlibref..HRV_US (SEGMENT,X,Y,STATE_REGION_CD,COUNTRY_CD)
Select SEGMENT,X,Y,STATE_REGION_CD,COUNTRY_CD
From _hcmmap.HRV_US;
/*************************/
/*SAS_ACTIONLEVELSECURITY*/
/*************************/
 execute (Insert into SAS_ACTIONLEVELSECURITY (ALS_ID,IDENTITY_TYPE,ACTION_ID,MODULE_ID,ACTOR_NM,SECURITY_CD) Values
( 1, 'R', 'EEPEMailAll', 1, 'HCM USER', 1 ),
( 2, 'R', 'EEPHierarchy', 1, 'HCM USER', 1 ),
( 3, 'R', 'EEPPrintProfile', 1, 'HCM USER', 1 ),
( 4, 'R', 'EEPPrintListing', 1, 'HCM USER', 1 ),
( 5, 'R', 'EEPProfileListingSaveAsExcel', 1, 'HCM USER', 0 ),
( 6, 'R', 'EEPProfileListingSaveAsPDF', 1, 'HCM USER', 0 ),
( 7, 'R', 'EEPOptions', 1, 'HCM USER', 1 ),
( 8, 'R', 'EEPProfileListingSaveAs', 1, 'HCM USER', 0 ),
( 9, 'R', 'EEPFindPeople', 1, 'HCM USER', 1 ),
( 10, 'R', 'EEPActions', 1, 'HCM USER', 1 ),
( 11, 'R', 'EEPSelectCategory', 1, 'HCM USER', 1 ),
( 15, 'R', 'EEPProfileViewSaveAsPDF', 1, 'HCM USER', 0 ),
( 16, 'R', 'EEPProfileViewSaveAs', 1, 'HCM USER', 0 ),
( 17, 'R', 'GEOCopy', 2, 'HCM USER', 0 ),
( 18, 'R', 'GEOPrint', 2, 'HCM USER', 1 ),
( 19, 'R', 'GEOExport', 2, 'HCM USER', 0 ),
( 20, 'R', 'GEOMap', 2, 'HCM USER', 1 ),
( 21, 'R', 'GEOTableView', 2, 'HCM USER', 1 ),
( 22, 'R', 'GEOWorkspace', 2, 'HCM USER', 1 ),
( 23, 'R', 'GEOOptions', 2, 'HCM USER', 1 ),
( 24, 'R', 'GEODRILLDOWN', 2, 'HCM USER', 1 ),
( 25, 'R', 'ORGMeasures', 3, 'HCM USER', 0 ),
( 26, 'R', 'ORGAnalysisView', 3, 'HCM USER', 1 ),
( 27, 'R', 'ORGCopy', 3, 'HCM USER', 0 ),
( 28, 'R', 'ORGNew', 3, 'HCM USER', 0 ),
( 29, 'R', 'ORGPresentation', 3, 'HCM USER', 1 ),
( 30, 'R', 'ORGExport', 3, 'HCM USER', 0 ),
( 31, 'R', 'ORGExportToPDF', 3, 'HCM USER', 0 ),
( 32, 'R', 'ORGModify', 3, 'HCM USER', 0 ),
( 33, 'R', 'ORGOptions', 3, 'HCM USER', 1 ),
( 34, 'R', 'ORGScorecard', 3, 'HCM USER', 1 ),
( 36, 'R', 'ORGFindPeople', 3, 'HCM USER', 1 ),
( 38, 'R', 'ORGSaveAs', 3, 'HCM USER', 0 ),
( 39, 'R', 'ORGPrint', 3, 'HCM USER', 1 ),
( 40, 'R', 'GSBarChart', 4, 'HCM USER', 1 ),
( 41, 'R', 'GSBarlineChart', 4, 'HCM USER', 1 ),
( 42, 'R', 'GSEmailAll', 4, 'HCM USER', 1 ),
( 43, 'R', 'GSExportToExcel', 4, 'HCM USER', 0 ),
( 44, 'R', 'GSExportToPDF', 4, 'HCM USER', 0 ),
( 45, 'R', 'GSHierarchy', 4, 'HCM USER', 1 ),
( 46, 'R', 'GSLineChart', 4, 'HCM USER', 1 ),
( 47, 'R', 'GSGeoMap', 4, 'HCM USER', 1 ),
( 48, 'R', 'GSOptions', 4, 'HCM USER', 1 ),
( 49, 'R', 'GSPieChart', 4, 'HCM USER', 1 ),
( 50, 'R', 'GSPrint', 4, 'HCM USER', 1 ),
( 51, 'R', 'GSSaveAs', 4, 'HCM USER', 0 ),
( 52, 'R', 'GSScatterPlot', 4, 'HCM USER', 1 ),
( 53, 'R', 'GSTableViewer', 4, 'HCM USER', 1 ),
( 54, 'R', 'GSSearch', 4, 'HCM USER', 1 ),
( 55, 'R', 'GSHistorySearch', 4, 'HCM USER', 1 ),
( 56, 'R', 'GSAdvSearch', 4, 'HCM USER', 1 ),
( 58, 'R', 'HHSearch', 6, 'HCM USER', 1 ),
( 59, 'R', 'HHHistorySearch', 6, 'HCM USER', 1 ),
( 60, 'R', 'HHAdvSearch', 6, 'HCM USER', 1 ),
( 61, 'R', 'HHEmpProfile', 6, 'HCM USER', 1 ),
( 62, 'R', 'HHNewOrg', 6, 'HCM USER', 0 ),
( 63, 'R', 'HHNewGeo', 6, 'HCM USER', 0 ),
( 64, 'R', 'HHAdmin', 6, 'HCM USER', 0 ),
( 65, 'R', 'HHDashboard', 6, 'HCM USER', 0 ),
( 66, 'R', 'HHWorkspace', 6, 'HCM USER', 1 ),
( 67, 'R', 'HHScorecard', 6, 'HCM USER', 0 ),
( 68, 'R', 'HHOlap', 6, 'HCM USER', 0 ),
( 69, 'R', 'HHReport', 6, 'HCM USER', 0 ),
( 70, 'R', 'HHPortal', 6, 'HCM USER', 1 ),
( 71, 'R', 'HHSearchAssist', 6, 'HCM USER', 0 ),
( 72, 'R', 'AdminSecuritySearchUserObject', 12, 'HCM USER', 0 ),
( 73, 'R', 'AdminSecurityViewPermissionObject', 12, 'HCM USER', 0 ),
( 74, 'R', 'AdminSecurityContextMenuObjectPermission', 12, 'HCM USER', 0 ),
( 75, 'R', 'AdminSecurityApplyChangeObject', 12, 'HCM USER', 0 ),
( 76, 'R', 'AdminSecurityViewReportObject', 12, 'HCM USER', 0 ),
( 77, 'R', 'AdminSecuritySearchUserRLS', 12, 'HCM USER', 0 ),
( 78, 'R', 'AdminSecurityAddFilterRLS', 12, 'HCM USER', 0 ),
( 79, 'R', 'AdminSecurityShowFilterRLS', 12, 'HCM USER', 0 ),
( 80, 'R', 'AdminSecurityDeleteRLS', 12, 'HCM USER', 0 ),
( 81, 'R', 'AdminSecurityPropertiesRLS', 12, 'HCM USER', 0 ),
( 82, 'R', 'AdminSecurityContextMenuFilter', 12, 'HCM USER', 0 ),
( 83, 'R', 'AdminSecurityApplyChangesHierarchies', 12, 'HCM USER', 0 ),
( 84, 'R', 'AdminSecurityApplyChangesCLS', 12, 'HCM USER', 0 ),
( 85, 'R', 'AdminSecurityApplyPropChangeCLS', 12, 'HCM USER', 0 ),
( 86, 'R', 'AdminSecurityRemoveIdentityCLS', 12, 'HCM USER', 0 ),
( 87, 'R', 'AdminSecurityContextMenuCLS', 12, 'HCM USER', 0 ),
( 88, 'R', 'AdminSecuritySearchUserCLS', 12, 'HCM USER', 0 ),
( 89, 'R', 'AdminSecurityRefreshCache', 12, 'HCM USER', 0 ),
( 90, 'R', 'AdminSecurityApplyChangesCustomObj', 12, 'HCM USER', 0 ),
( 91, 'R', 'AdminSecurityDeleteCustomObject', 12, 'HCM USER', 0 ),
( 92, 'R', 'AdminSecurityRemovePermissionObject', 12, 'HCM USER', 0 ),
( 93, 'R', 'AdminSecurityContextMenuUserFilters', 12, 'HCM USER', 0 ),
( 94, 'R', 'AdminSecurityContextMenuSearchUserFilters', 12, 'HCM USER', 0 ),
( 95, 'R', 'ADCAddTable', 8, 'HCM USER', 0 ),
( 96, 'R', 'ADCMapHierarchy', 8, 'HCM USER', 0 ),
( 97, 'R', 'ADCNewFormat', 8, 'HCM USER', 0 ),
( 98, 'R', 'ADCNewMetric', 8, 'HCM USER', 0 ),
( 99, 'R', 'ADCLog', 8, 'HCM USER', 0 ),
( 100, 'R', 'ADCRefreshCache', 8, 'HCM USER', 0 ),
( 101, 'R', 'ADApplyChangeTable', 9, 'HCM USER', 0 ),
( 102, 'R', 'ADTableMenu', 9, 'HCM USER', 0 ),
( 103, 'R', 'ADCopyTable', 9, 'HCM USER', 0 ),
( 104, 'R', 'ADViewTable', 9, 'HCM USER', 0 ),
( 105, 'R', 'ADBuildIMAP', 9, 'HCM USER', 0 ),
( 106, 'R', 'ADExportToExcelTable', 9, 'HCM USER', 0 ),
( 107, 'R', 'ADDeleteTable', 9, 'HCM USER', 0 ),
( 108, 'R', 'ADGenerateSymbolTable', 9, 'HCM USER', 0 ),
( 109, 'R', 'ADApplyChangeCol', 9, 'HCM USER', 0 ),
( 110, 'R', 'ADApplyPropChangeCol', 9, 'HCM USER', 0 ),
( 111, 'R', 'ADColumnMenu', 9, 'HCM USER', 0 ),
( 112, 'R', 'ADDeleteColumn', 9, 'HCM USER', 0 ),
( 113, 'R', 'ADGenerateSymbolColumn', 9, 'HCM USER', 0 ),
( 115, 'R', 'ADDeleteHierarchy', 9, 'HCM USER', 0 ),
( 116, 'R', 'ADEditHierarchy', 9, 'HCM USER', 0 ),
( 117, 'R', 'ADFormatMenu', 9, 'HCM USER', 0 ),
( 118, 'R', 'ADDeleteFormat', 9, 'HCM USER', 0 ),
( 119, 'R', 'ADEditFormat', 9, 'HCM USER', 0 ),
( 120, 'R', 'ACINewCubeMenu', 10, 'HCM USER', 0 ),
( 121, 'R', 'ACIRebuildIMAPMenu', 10, 'HCM USER', 0 ),
( 122, 'R', 'ACIRefreshCubeMenu', 10, 'HCM USER', 0 ),
( 123, 'R', 'ACNewEEP', 11, 'HCM USER', 0 ),
( 124, 'R', 'ACRefreshCache', 11, 'HCM USER', 0 ),
( 125, 'R', 'ACEEPMenu', 11, 'HCM USER', 0 ),
( 126, 'R', 'ACCopyEEP', 11, 'HCM USER', 0 ),
( 127, 'R', 'ACDeleteEEP', 11, 'HCM USER', 0 ),
( 128, 'R', 'ACApplyChangeEEP', 11, 'HCM USER', 0 ),
( 129, 'R', 'ACApplyChangeEditEEP', 11, 'HCM USER', 0 ),
( 130, 'R', 'ACApplyChangeListEEP', 11, 'HCM USER', 0 ),
( 131, 'R', 'ACApplyChangeHeaderEEP', 11, 'HCM USER', 0 ),
( 132, 'R', 'ACApplyChangeSearchEEP', 11, 'HCM USER', 0 ),
( 133, 'R', 'ACApplyChangeCategory', 11, 'HCM USER', 0 ),
( 134, 'R', 'ACApplyChangeCategoryDefails', 11, 'HCM USER', 0 ),
( 135, 'R', 'ACEditGEO', 11, 'HCM USER', 0 ),
( 136, 'R', 'ACEditORG', 11, 'HCM USER', 0 ),
( 137, 'R', 'ACDeleteTemplate', 11, 'HCM USER', 0 ),
( 138, 'R', 'ACEditTemplate', 11, 'HCM USER', 0 ),
( 139, 'R', 'ACApplyChangeSearch', 11, 'HCM USER', 0 ),
( 140, 'R', 'EEPEMailAll', 1, 'HCM ANALYST', 1 ),
( 141, 'R', 'EEPHierarchy', 1, 'HCM ANALYST', 1 ),
( 142, 'R', 'EEPPrintProfile', 1, 'HCM ANALYST', 1 ),
( 143, 'R', 'EEPPrintListing', 1, 'HCM ANALYST', 1 ),
( 144, 'R', 'EEPProfileListingSaveAsExcel', 1, 'HCM ANALYST', 1 ),
( 145, 'R', 'EEPProfileListingSaveAsPDF', 1, 'HCM ANALYST', 1 ),
( 146, 'R', 'EEPOptions', 1, 'HCM ANALYST', 1 ),
( 147, 'R', 'EEPProfileListingSaveAs', 1, 'HCM ANALYST', 1 ),
( 148, 'R', 'EEPFindPeople', 1, 'HCM ANALYST', 1 ),
( 149, 'R', 'EEPActions', 1, 'HCM ANALYST', 1 ),
( 150, 'R', 'EEPSelectCategory', 1, 'HCM ANALYST', 1 ),
( 154, 'R', 'EEPProfileViewSaveAsPDF', 1, 'HCM ANALYST', 1 ),
( 155, 'R', 'EEPProfileViewSaveAs', 1, 'HCM ANALYST', 1 ),
( 156, 'R', 'ORGMeasures', 3, 'HCM ANALYST', 1 ),
( 157, 'R', 'ORGAnalysisView', 3, 'HCM ANALYST', 1 ),
( 158, 'R', 'ORGCopy', 3, 'HCM ANALYST', 1 ),
( 159, 'R', 'ORGNew', 3, 'HCM ANALYST', 1 ),
( 160, 'R', 'ORGPresentation', 3, 'HCM ANALYST', 1 ),
( 161, 'R', 'ORGExport', 3, 'HCM ANALYST', 1 ),
( 162, 'R', 'ORGExportToPDF', 3, 'HCM ANALYST', 1 ),
( 163, 'R', 'ORGModify', 3, 'HCM ANALYST', 1 ),
( 164, 'R', 'ORGOptions', 3, 'HCM ANALYST', 1 ),
( 166, 'R', 'ORGScorecard', 3, 'HCM ANALYST', 1 ),
( 167, 'R', 'ORGFindPeople', 3, 'HCM ANALYST', 1 ),
( 169, 'R', 'ORGSaveAs', 3, 'HCM ANALYST', 1 ),
( 170, 'R', 'ORGPrint', 3, 'HCM ANALYST', 1 ),
( 171, 'R', 'GSBarChart', 4, 'HCM ANALYST', 1 ),
( 172, 'R', 'GSBarlineChart', 4, 'HCM ANALYST', 1 ),
( 173, 'R', 'GSEmailAll', 4, 'HCM ANALYST', 1 ),
( 174, 'R', 'GSExportToExcel', 4, 'HCM ANALYST', 1 ),
( 175, 'R', 'GSExportToPDF', 4, 'HCM ANALYST', 1 ),
( 176, 'R', 'GSHierarchy', 4, 'HCM ANALYST', 1 ),
( 177, 'R', 'GSLineChart', 4, 'HCM ANALYST', 1 ),
( 178, 'R', 'GSGeoMap', 4, 'HCM ANALYST', 1 ),
( 179, 'R', 'GSOptions', 4, 'HCM ANALYST', 1 ),
( 180, 'R', 'GSPieChart', 4, 'HCM ANALYST', 1 ),
( 181, 'R', 'GSPrint', 4, 'HCM ANALYST', 1 ),
( 182, 'R', 'GSSaveAs', 4, 'HCM ANALYST', 1 ),
( 183, 'R', 'GSScatterPlot', 4, 'HCM ANALYST', 1 ),
( 184, 'R', 'GSTableViewer', 4, 'HCM ANALYST', 1 ),
( 185, 'R', 'GSSearch', 4, 'HCM ANALYST', 1 ),
( 186, 'R', 'GSHistorySearch', 4, 'HCM ANALYST', 1 ),
( 187, 'R', 'GSAdvSearch', 4, 'HCM ANALYST', 1 ),
( 189, 'R', 'HHSearch', 6, 'HCM ANALYST', 1 ),
( 190, 'R', 'HHHistorySearch', 6, 'HCM ANALYST', 1 ),
( 191, 'R', 'HHAdvSearch', 6, 'HCM ANALYST', 1 ),
( 192, 'R', 'HHEmpProfile', 6, 'HCM ANALYST', 1 ),
( 193, 'R', 'HHNewOrg', 6, 'HCM ANALYST', 1 ),
( 194, 'R', 'HHNewGeo', 6, 'HCM ANALYST', 1 ),
( 195, 'R', 'HHAdmin', 6, 'HCM ANALYST', 0 ),
( 196, 'R', 'HHDashboard', 6, 'HCM ANALYST', 1 ),
( 197, 'R', 'HHWorkspace', 6, 'HCM ANALYST', 1 ),
( 198, 'R', 'HHScorecard', 6, 'HCM ANALYST', 1 ),
( 199, 'R', 'HHOlap', 6, 'HCM ANALYST', 1 ),
( 200, 'R', 'HHReport', 6, 'HCM ANALYST', 1 ),
( 201, 'R', 'HHPortal', 6, 'HCM ANALYST', 1 ),
( 202, 'R', 'HHSearchAssist', 6, 'HCM ANALYST', 1 ),
( 203, 'R', 'AdminSecuritySearchUserObject', 12, 'HCM ANALYST', 0 ),
( 204, 'R', 'AdminSecurityViewPermissionObject', 12, 'HCM ANALYST', 0 ),
( 205, 'R', 'AdminSecurityContextMenuObjectPermission', 12, 'HCM ANALYST', 0 ),
( 206, 'R', 'AdminSecurityApplyChangeObject', 12, 'HCM ANALYST', 0 ),
( 207, 'R', 'AdminSecurityViewReportObject', 12, 'HCM ANALYST', 0 ),
( 208, 'R', 'AdminSecuritySearchUserRLS', 12, 'HCM ANALYST', 0 ),
( 209, 'R', 'AdminSecurityAddFilterRLS', 12, 'HCM ANALYST', 0 ),
( 210, 'R', 'AdminSecurityShowFilterRLS', 12, 'HCM ANALYST', 0 ),
( 211, 'R', 'AdminSecurityDeleteRLS', 12, 'HCM ANALYST', 0 ),
( 212, 'R', 'AdminSecurityPropertiesRLS', 12, 'HCM ANALYST', 0 ),
( 213, 'R', 'AdminSecurityContextMenuFilter', 12, 'HCM ANALYST', 0 ),
( 214, 'R', 'AdminSecurityApplyChangesHierarchies', 12, 'HCM ANALYST', 0 ),
( 215, 'R', 'AdminSecurityApplyChangesCLS', 12, 'HCM ANALYST', 0 ),
( 216, 'R', 'AdminSecurityApplyPropChangeCLS', 12, 'HCM ANALYST', 0 ),
( 217, 'R', 'AdminSecurityRemoveIdentityCLS', 12, 'HCM ANALYST', 0 ),
( 218, 'R', 'AdminSecurityContextMenuCLS', 12, 'HCM ANALYST', 0 ),
( 219, 'R', 'AdminSecuritySearchUserCLS', 12, 'HCM ANALYST', 0 ),
( 220, 'R', 'AdminSecurityRefreshCache', 12, 'HCM ANALYST', 0 ),
( 221, 'R', 'AdminSecurityApplyChangesCustomObj', 12, 'HCM ANALYST', 0 ),
( 222, 'R', 'AdminSecurityRemovePermissionObject', 12, 'HCM ANALYST', 0 ),
( 223, 'R', 'AdminSecurityDeleteCustomObject', 12, 'HCM ANALYST', 0 ),
( 224, 'R', 'AdminSecurityContextMenuUserFilters', 12, 'HCM ANALYST', 0 ),
( 225, 'R', 'AdminSecurityContextMenuSearchUserFilters', 12, 'HCM ANALYST', 0 ),
( 226, 'R', 'GEOCopy', 2, 'HCM ANALYST', 1 ),
( 227, 'R', 'GEOPrint', 2, 'HCM ANALYST', 1 ),
( 228, 'R', 'GEOExport', 2, 'HCM ANALYST', 1 ),
( 229, 'R', 'GEOMap', 2, 'HCM ANALYST', 1 ),
( 230, 'R', 'GEOTableView', 2, 'HCM ANALYST', 1 ),
( 231, 'R', 'GEOWorkspace', 2, 'HCM ANALYST', 1 ),
( 232, 'R', 'GEOOptions', 2, 'HCM ANALYST', 1 ),
( 233, 'R', 'GEODRILLDOWN', 2, 'HCM ANALYST', 1 ),
( 234, 'R', 'ADCAddTable', 8, 'HCM ANALYST', 0 ),
( 235, 'R', 'ADCMapHierarchy', 8, 'HCM ANALYST', 0 ),
( 236, 'R', 'ADCNewFormat', 8, 'HCM ANALYST', 0 ),
( 237, 'R', 'ADCNewMetric', 8, 'HCM ANALYST', 0 ),
( 238, 'R', 'ADCLog', 8, 'HCM ANALYST', 0 ),
( 239, 'R', 'ADCRefreshCache', 8, 'HCM ANALYST', 0 ),
( 240, 'R', 'ADApplyChangeTable', 9, 'HCM ANALYST', 0 ),
( 241, 'R', 'ADTableMenu', 9, 'HCM ANALYST', 0 ),
( 242, 'R', 'ADCopyTable', 9, 'HCM ANALYST', 0 ),
( 243, 'R', 'ADViewTable', 9, 'HCM ANALYST', 0 ),
( 244, 'R', 'ADBuildIMAP', 9, 'HCM ANALYST', 0 ),
( 245, 'R', 'ADExportToExcelTable', 9, 'HCM ANALYST', 0 ),
( 246, 'R', 'ADDeleteTable', 9, 'HCM ANALYST', 0 ),
( 247, 'R', 'ADGenerateSymbolTable', 9, 'HCM ANALYST', 0 ),
( 248, 'R', 'ADApplyChangeCol', 9, 'HCM ANALYST', 0 ),
( 249, 'R', 'ADApplyPropChangeCol', 9, 'HCM ANALYST', 0 ),
( 250, 'R', 'ADColumnMenu', 9, 'HCM ANALYST', 0 ),
( 251, 'R', 'ADDeleteColumn', 9, 'HCM ANALYST', 0 ),
( 252, 'R', 'ADGenerateSymbolColumn', 9, 'HCM ANALYST', 0 ),
( 254, 'R', 'ADDeleteHierarchy', 9, 'HCM ANALYST', 0 ),
( 255, 'R', 'ADEditHierarchy', 9, 'HCM ANALYST', 0 ),
( 256, 'R', 'ADFormatMenu', 9, 'HCM ANALYST', 0 ),
( 257, 'R', 'ADDeleteFormat', 9, 'HCM ANALYST', 0 ),
( 258, 'R', 'ADEditFormat', 9, 'HCM ANALYST', 0 ),
( 259, 'R', 'ACINewCubeMenu', 10, 'HCM ANALYST', 0 ),
( 260, 'R', 'ACIRebuildIMAPMenu', 10, 'HCM ANALYST', 0 ),
( 261, 'R', 'ACIRefreshCubeMenu', 10, 'HCM ANALYST', 0 ),
( 262, 'R', 'ACNewEEP', 11, 'HCM ANALYST', 0 ),
( 263, 'R', 'ACRefreshCache', 11, 'HCM ANALYST', 0 ),
( 264, 'R', 'ACEEPMenu', 11, 'HCM ANALYST', 0 ),
( 265, 'R', 'ACCopyEEP', 11, 'HCM ANALYST', 0 ),
( 266, 'R', 'ACDeleteEEP', 11, 'HCM ANALYST', 0 ),
( 267, 'R', 'ACApplyChangeEEP', 11, 'HCM ANALYST', 0 ),
( 268, 'R', 'ACApplyChangeEditEEP', 11, 'HCM ANALYST', 0 ),
( 269, 'R', 'ACApplyChangeListEEP', 11, 'HCM ANALYST', 0 ),
( 270, 'R', 'ACApplyChangeHeaderEEP', 11, 'HCM ANALYST', 0 ),
( 271, 'R', 'ACApplyChangeSearchEEP', 11, 'HCM ANALYST', 0 ),
( 272, 'R', 'ACApplyChangeCategory', 11, 'HCM ANALYST', 0 ),
( 273, 'R', 'ACApplyChangeCategoryDefails', 11, 'HCM ANALYST', 0 ),
( 274, 'R', 'ACEditGEO', 11, 'HCM ANALYST', 0 ),
( 275, 'R', 'ACEditORG', 11, 'HCM ANALYST', 0 ),
( 276, 'R', 'ACDeleteTemplate', 11, 'HCM ANALYST', 0 ),
( 277, 'R', 'ACEditTemplate', 11, 'HCM ANALYST', 0 ),
( 278, 'R', 'ACApplyChangeSearch', 11, 'HCM ANALYST', 0 ),
( 279, 'R', 'EEPEMailAll', 1, 'HCM ADMINISTRATOR', 1 ),
( 280, 'R', 'EEPHierarchy', 1, 'HCM ADMINISTRATOR', 1 ),
( 281, 'R', 'EEPPrintProfile', 1, 'HCM ADMINISTRATOR', 1 ),
( 282, 'R', 'EEPPrintListing', 1, 'HCM ADMINISTRATOR', 1 ),
( 283, 'R', 'EEPProfileListingSaveAsExcel', 1, 'HCM ADMINISTRATOR', 1 ),
( 284, 'R', 'EEPProfileListingSaveAsPDF', 1, 'HCM ADMINISTRATOR', 1 ),
( 285, 'R', 'EEPOptions', 1, 'HCM ADMINISTRATOR', 1 ),
( 286, 'R', 'EEPProfileListingSaveAs', 1, 'HCM ADMINISTRATOR', 1 ),
( 287, 'R', 'EEPFindPeople', 1, 'HCM ADMINISTRATOR', 1 ),
( 288, 'R', 'EEPActions', 1, 'HCM ADMINISTRATOR', 1 ),
( 289, 'R', 'EEPSelectCategory', 1, 'HCM ADMINISTRATOR', 1 ),
( 293, 'R', 'EEPProfileViewSaveAsPDF', 1, 'HCM ADMINISTRATOR', 1 ),
( 294, 'R', 'EEPProfileViewSaveAs', 1, 'HCM ADMINISTRATOR', 1 ),
( 295, 'R', 'GEOCopy', 2, 'HCM ADMINISTRATOR', 1 ),
( 296, 'R', 'GEOPrint', 2, 'HCM ADMINISTRATOR', 1 ),
( 297, 'R', 'GEOExport', 2, 'HCM ADMINISTRATOR', 1 ),
( 298, 'R', 'GEOMap', 2, 'HCM ADMINISTRATOR', 1 ),
( 299, 'R', 'GEOTableView', 2, 'HCM ADMINISTRATOR', 1 ),
( 300, 'R', 'GEOWorkspace', 2, 'HCM ADMINISTRATOR', 1 ),
( 301, 'R', 'GEOOptions', 2, 'HCM ADMINISTRATOR', 1 ),
( 302, 'R', 'GEODRILLDOWN', 2, 'HCM ADMINISTRATOR', 1 ),
( 303, 'R', 'ORGMeasures', 3, 'HCM ADMINISTRATOR', 1 ),
( 304, 'R', 'ORGAnalysisView', 3, 'HCM ADMINISTRATOR', 1 ),
( 305, 'R', 'ORGCopy', 3, 'HCM ADMINISTRATOR', 1 ),
( 306, 'R', 'ORGNew', 3, 'HCM ADMINISTRATOR', 1 ),
( 307, 'R', 'ORGPresentation', 3, 'HCM ADMINISTRATOR', 1 ),
( 308, 'R', 'ORGExport', 3, 'HCM ADMINISTRATOR', 1 ),
( 309, 'R', 'ORGExportToPDF', 3, 'HCM ADMINISTRATOR', 1 ),
( 310, 'R', 'ORGModify', 3, 'HCM ADMINISTRATOR', 1 ),
( 311, 'R', 'ORGOptions', 3, 'HCM ADMINISTRATOR', 1 ),
( 312, 'R', 'ORGScorecard', 3, 'HCM ADMINISTRATOR', 1 ),
( 314, 'R', 'ORGFindPeople', 3, 'HCM ADMINISTRATOR', 1 ),
( 316, 'R', 'ORGSaveAs', 3, 'HCM ADMINISTRATOR', 1 ),
( 317, 'R', 'ORGPrint', 3, 'HCM ADMINISTRATOR', 1 ),
( 318, 'R', 'GSBarChart', 4, 'HCM ADMINISTRATOR', 1 ),
( 319, 'R', 'GSBarlineChart', 4, 'HCM ADMINISTRATOR', 1 ),
( 320, 'R', 'GSEmailAll', 4, 'HCM ADMINISTRATOR', 1 ),
( 321, 'R', 'GSExportToExcel', 4, 'HCM ADMINISTRATOR', 1 ),
( 322, 'R', 'GSExportToPDF', 4, 'HCM ADMINISTRATOR', 1 ),
( 323, 'R', 'GSHierarchy', 4, 'HCM ADMINISTRATOR', 1 ),
( 324, 'R', 'GSLineChart', 4, 'HCM ADMINISTRATOR', 1 ),
( 325, 'R', 'GSGeoMap', 4, 'HCM ADMINISTRATOR', 1 ),
( 326, 'R', 'GSOptions', 4, 'HCM ADMINISTRATOR', 1 ),
( 327, 'R', 'GSPieChart', 4, 'HCM ADMINISTRATOR', 1 ),
( 328, 'R', 'GSPrint', 4, 'HCM ADMINISTRATOR', 1 ),
( 329, 'R', 'GSSaveAs', 4, 'HCM ADMINISTRATOR', 1 ),
( 330, 'R', 'GSScatterPlot', 4, 'HCM ADMINISTRATOR', 1 ),
( 331, 'R', 'GSTableViewer', 4, 'HCM ADMINISTRATOR', 1 ),
( 332, 'R', 'GSSearch', 4, 'HCM ADMINISTRATOR', 1 ),
( 333, 'R', 'GSHistorySearch', 4, 'HCM ADMINISTRATOR', 1 ),
( 334, 'R', 'GSAdvSearch', 4, 'HCM ADMINISTRATOR', 1 ),
( 336, 'R', 'HHSearch', 6, 'HCM ADMINISTRATOR', 1 ),
( 337, 'R', 'HHHistorySearch', 6, 'HCM ADMINISTRATOR', 1 ),
( 338, 'R', 'HHAdvSearch', 6, 'HCM ADMINISTRATOR', 1 ),
( 339, 'R', 'HHEmpProfile', 6, 'HCM ADMINISTRATOR', 1 ),
( 340, 'R', 'HHNewOrg', 6, 'HCM ADMINISTRATOR', 1 ),
( 341, 'R', 'HHNewGeo', 6, 'HCM ADMINISTRATOR', 1 ),
( 342, 'R', 'HHAdmin', 6, 'HCM ADMINISTRATOR', 1 ),
( 343, 'R', 'HHDashboard', 6, 'HCM ADMINISTRATOR', 1 ),
( 344, 'R', 'HHWorkspace', 6, 'HCM ADMINISTRATOR', 1 ),
( 345, 'R', 'HHScorecard', 6, 'HCM ADMINISTRATOR', 1 ),
( 346, 'R', 'HHOlap', 6, 'HCM ADMINISTRATOR', 1 ),
( 347, 'R', 'HHReport', 6, 'HCM ADMINISTRATOR', 1 ),
( 348, 'R', 'HHPortal', 6, 'HCM ADMINISTRATOR', 1 ),
( 349, 'R', 'HHSearchAssist', 6, 'HCM ADMINISTRATOR', 1 ),
( 350, 'R', 'ADCAddTable', 8, 'HCM ADMINISTRATOR', 1 ),
( 351, 'R', 'ADCMapHierarchy', 8, 'HCM ADMINISTRATOR', 1 ),
( 352, 'R', 'ADCNewFormat', 8, 'HCM ADMINISTRATOR', 1 ),
( 353, 'R', 'ADCNewMetric', 8, 'HCM ADMINISTRATOR', 1 ),
( 354, 'R', 'ADCLog', 8, 'HCM ADMINISTRATOR', 1 ),
( 355, 'R', 'ADCRefreshCache', 8, 'HCM ADMINISTRATOR', 1 ),
( 356, 'R', 'ADApplyChangeTable', 9, 'HCM ADMINISTRATOR', 1 ),
( 357, 'R', 'ADTableMenu', 9, 'HCM ADMINISTRATOR', 1 ),
( 358, 'R', 'ADCopyTable', 9, 'HCM ADMINISTRATOR', 1 ),
( 359, 'R', 'ADViewTable', 9, 'HCM ADMINISTRATOR', 1 ),
( 360, 'R', 'ADBuildIMAP', 9, 'HCM ADMINISTRATOR', 1 ),
( 361, 'R', 'ADExportToExcelTable', 9, 'HCM ADMINISTRATOR', 1 ),
( 362, 'R', 'ADDeleteTable', 9, 'HCM ADMINISTRATOR', 1 ),
( 363, 'R', 'ADGenerateSymbolTable', 9, 'HCM ADMINISTRATOR', 1 ),
( 364, 'R', 'ADApplyChangeCol', 9, 'HCM ADMINISTRATOR', 1 ),
( 365, 'R', 'ADApplyPropChangeCol', 9, 'HCM ADMINISTRATOR', 1 ),
( 366, 'R', 'ADColumnMenu', 9, 'HCM ADMINISTRATOR', 1 ),
( 367, 'R', 'ADDeleteColumn', 9, 'HCM ADMINISTRATOR', 1 ),
( 368, 'R', 'ADGenerateSymbolColumn', 9, 'HCM ADMINISTRATOR', 1 ),
( 370, 'R', 'ADDeleteHierarchy', 9, 'HCM ADMINISTRATOR', 1 ),
( 371, 'R', 'ADEditHierarchy', 9, 'HCM ADMINISTRATOR', 1 ),
( 372, 'R', 'ADFormatMenu', 9, 'HCM ADMINISTRATOR', 1 ),
( 373, 'R', 'ADDeleteFormat', 9, 'HCM ADMINISTRATOR', 1 ),
( 374, 'R', 'ADEditFormat', 9, 'HCM ADMINISTRATOR', 1 ),
( 375, 'R', 'ACINewCubeMenu', 10, 'HCM ADMINISTRATOR', 1 ),
( 376, 'R', 'ACIRebuildIMAPMenu', 10, 'HCM ADMINISTRATOR', 1 ),
( 377, 'R', 'ACIRefreshCubeMenu', 10, 'HCM ADMINISTRATOR', 1 ),
( 378, 'R', 'ACNewEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 379, 'R', 'ACRefreshCache', 11, 'HCM ADMINISTRATOR', 1 ),
( 380, 'R', 'ACEEPMenu', 11, 'HCM ADMINISTRATOR', 1 ),
( 381, 'R', 'ACCopyEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 382, 'R', 'ACDeleteEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 383, 'R', 'ACApplyChangeEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 384, 'R', 'ACApplyChangeEditEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 385, 'R', 'ACApplyChangeListEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 386, 'R', 'ACApplyChangeHeaderEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 387, 'R', 'ACApplyChangeSearchEEP', 11, 'HCM ADMINISTRATOR', 1 ),
( 388, 'R', 'ACApplyChangeCategory', 11, 'HCM ADMINISTRATOR', 1 ),
( 389, 'R', 'ACApplyChangeCategoryDefails', 11, 'HCM ADMINISTRATOR', 1 ),
( 390, 'R', 'ACEditGEO', 11, 'HCM ADMINISTRATOR', 1 ),
( 391, 'R', 'ACEditORG', 11, 'HCM ADMINISTRATOR', 1 ),
( 392, 'R', 'ACDeleteTemplate', 11, 'HCM ADMINISTRATOR', 1 ),
( 393, 'R', 'ACEditTemplate', 11, 'HCM ADMINISTRATOR', 1 ),
( 394, 'R', 'ACApplyChangeSearch', 11, 'HCM ADMINISTRATOR', 1 ),
( 395, 'R', 'AdminSecuritySearchUserObject', 12, 'HCM ADMINISTRATOR', 1 ),
( 396, 'R', 'AdminSecurityViewPermissionObject', 12, 'HCM ADMINISTRATOR', 1 ),
( 397, 'R', 'AdminSecurityContextMenuObjectPermission', 12, 'HCM ADMINISTRATOR', 1 ),
( 398, 'R', 'AdminSecurityApplyChangeObject', 12, 'HCM ADMINISTRATOR', 1 ),
( 399, 'R', 'AdminSecurityViewReportObject', 12, 'HCM ADMINISTRATOR', 1 ),
( 400, 'R', 'AdminSecuritySearchUserRLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 401, 'R', 'AdminSecurityAddFilterRLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 402, 'R', 'AdminSecurityShowFilterRLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 403, 'R', 'AdminSecurityDeleteRLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 404, 'R', 'AdminSecurityPropertiesRLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 405, 'R', 'AdminSecurityContextMenuFilter', 12, 'HCM ADMINISTRATOR', 1 ),
( 406, 'R', 'AdminSecurityApplyChangesHierarchies', 12, 'HCM ADMINISTRATOR', 1 ),
( 407, 'R', 'AdminSecurityApplyChangesCLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 408, 'R', 'AdminSecurityApplyPropChangeCLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 409, 'R', 'AdminSecurityRemoveIdentityCLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 410, 'R', 'AdminSecurityContextMenuCLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 411, 'R', 'AdminSecuritySearchUserCLS', 12, 'HCM ADMINISTRATOR', 1 ),
( 412, 'R', 'AdminSecurityRefreshCache', 12, 'HCM ADMINISTRATOR', 1 ),
( 413, 'R', 'AdminSecurityApplyChangesCustomObj', 12, 'HCM ADMINISTRATOR', 1 ),
( 414, 'R', 'AdminSecurityDeleteCustomObject', 12, 'HCM ADMINISTRATOR', 1 ),
( 415, 'R', 'AdminSecurityRemovePermissionObject', 12, 'HCM ADMINISTRATOR', 1 ),
( 416, 'R', 'AdminSecurityContextMenuUserFilters', 12, 'HCM ADMINISTRATOR', 1 ),
( 417, 'R', 'AdminSecurityContextMenuSearchUserFilters', 12, 'HCM ADMINISTRATOR', 1 ),
( 418, 'R', 'ACGProp', 13, 'HCM ADMINISTRATOR', 1 ),
( 419, 'R', 'ACGProp', 13, 'HCM ANALYST', 0 ),
( 420, 'R', 'ACGProp', 13, 'HCM USER', 0 ),
(639, 'R', 'AdminSecurityApplyChangesRLSExtension', 12, 'HCM ADMINISTRATOR', 1 ),
(640, 'R', 'AdminSecurityApplyChangesRLSExtension', 12, 'HCM USER', 0 ),
(641, 'R', 'AdminSecurityApplyChangesRLSExtension', 12, 'HCM ANALYST', 0 ),
(642,'R','ADViewCube',10,'HCM USER',0),
(643,'R','ADRebuildCube',10,'HCM USER',0),
(644,'R','CubeRefresh',10,'HCM USER',0),
(645,'R','ADBuildCubeIMAP',10,'HCM USER',0),
(646,'R','ADDeleteCube',10,'HCM USER',0),
(647,'R','ADViewCube',10,'HCM ADMINISTRATOR',1),
(648,'R','ADRebuildCube',10,'HCM ADMINISTRATOR',1),
(649,'R','CubeRefresh',10,'HCM ADMINISTRATOR',1),
(650,'R','ADBuildCubeIMAP',10,'HCM ADMINISTRATOR',1),
(651,'R','ADDeleteCube',10,'HCM ADMINISTRATOR',1),
(652,'R','ADViewCube',10,'HCM ANALYST',0),
(653,'R','ADRebuildCube',10,'HCM ANALYST',0),
(654,'R','CubeRefresh',10,'HCM ANALYST',0),
(655,'R','ADBuildCubeIMAP',10,'HCM ANALYST',0),
(656,'R','ADDeleteCube',10,'HCM ANALYST',0),
(657,'R','ADCubeMenu',10,'HCM USER',0),
(658,'R','ADCubeMenu',10,'HCM ANALYST',0),
(659,'R','ADCubeMenu',10,'HCM ADMINISTRATOR',1),
(660,'R','CubeRebuildButtonBar',10,'HCM USER',0),
(661,'R','CubeRebuildButtonBar',10,'HCM ADMINISTRATOR',1),
(662,'R','CubeRebuildButtonBar',10,'HCM ANALYST',0),
(663,'R','AddDimensionButtonBar',10,'HCM USER',0),
(664,'R','AddDimensionButtonBar',10,'HCM ADMINISTRATOR',1),
(665,'R','AddDimensionButtonBar',10,'HCM ANALYST',0),
(667,'R', 'ACNewTemplate', 11, 'HCM USER', 0),
(668,'R', 'ACNewTemplate', 11, 'HCM ANALYST', 0),
(669,'R', 'ACNewTemplate', 11, 'HCM ADMINISTRATOR', 1),
(670,'R', 'EEPWorkspaceManager', 1, 'HCM USER', 1 ),
(671,'R', 'EEPWorkspaceManager', 1, 'HCM ANALYST', 1 ),
(672,'R', 'EEPWorkspaceManager', 1, 'HCM ADMINISTRATOR', 1 ),
(674,'R', 'ACGRefreshCache', 13, 'HCM USER', 0 ),
(675,'R', 'ACGRefreshCache', 13, 'HCM ANALYST', 0 ),
(676,'R', 'ACGRefreshCache', 13, 'HCM ADMINISTRATOR', 1 ),
(677,'R', 'ACGDiagnosis', 13, 'HCM USER', 0 ),
(678,'R', 'ACGDiagnosis', 13, 'HCM ANALYST', 0 ),
(679,'R', 'ACGDiagnosis', 13, 'HCM ADMINISTRATOR', 1 ),
(680,'R', 'AdminSecurityImportUsers' ,12, 'HCM USER', 0),
(681,'R', 'AdminSecurityImportUsers' ,12, 'HCM ADMINISTRATOR', 1),
(682,'R', 'AdminSecurityImportUsers' ,12, 'HCM ANALYST', 1),
(683,'R', 'ADCFormatCatalog' ,8, 'HCM USER', 0),
(684,'R', 'ADCFormatCatalog' ,8, 'HCM ADMINISTRATOR', 1),
(685,'R', 'ADCFormatCatalog' ,8, 'HCM ANALYST', 1),
(686,'R', 'ADCRunJob' ,8, 'HCM USER', 0),
(687,'R', 'ADCRunJob' ,8, 'HCM ADMINISTRATOR', 1),
(688,'R', 'ADCRunJob' ,8, 'HCM ANALYST', 1),
(689,'R', 'ADRebuildImapMenu' ,10, 'HCM USER', 0),
(690,'R', 'ADRebuildImapMenu' ,10, 'HCM ADMINISTRATOR', 1),
(691,'R', 'ADRebuildImapMenu' ,10, 'HCM ANALYST', 1),
(692,'R', 'ADViewImapMenu' ,10, 'HCM USER', 0),
(693,'R', 'ADViewImapMenu' ,10, 'HCM ADMINISTRATOR', 1),
(694,'R', 'ADViewImapMenu' ,10, 'HCM ANALYST', 1),
(695,'R', 'ADDeleteImapMenu' ,10, 'HCM USER', 0),
(696,'R', 'ADDeleteImapMenu' ,10, 'HCM ADMINISTRATOR', 1),
(697,'R', 'ADDeleteImapMenu' ,10, 'HCM ANALYST', 1),
(698,'R', 'ADEditMeasureMenu' ,8, 'HCM USER', 0),
(699,'R', 'ADEditMeasureMenu' ,8, 'HCM ADMINISTRATOR', 1),
(700,'R', 'ADEditMeasureMenu' ,8, 'HCM ANALYST', 1),
(701,'R', 'ADDeleteMeasureMenu' ,8, 'HCM USER', 0),
(702,'R', 'ADDeleteMeasureMenu' ,8, 'HCM ADMINISTRATOR', 1),
(703,'R', 'ADDeleteMeasureMenu' ,8, 'HCM ANALYST', 1),
(704,'R', 'EEPViewEmployeeDetails', 1, 'HCM USER', 1 ),
(705,'R', 'EEPViewEmployeeDetails', 1, 'HCM ANALYST', 1 ),
(706,'R', 'EEPViewEmployeeDetails', 1, 'HCM ADMINISTRATOR', 1 ),
(707,'U', 'EEPViewEmployeeDetails', 1, 'SASDEMO', 1 ),
(708,'R', 'ADPlanApplyChanges' ,8, 'HCM USER', 0),
(709,'R', 'ADPlanApplyChanges' ,8, 'HCM ADMINISTRATOR', 1),
(710,'R', 'ADPlanApplyChanges' ,8, 'HCM ANALYST', 1),
(711,'R', 'ADPlanSASCode' ,8, 'HCM USER', 0),
(712,'R', 'ADPlanSASCode' ,8, 'HCM ADMINISTRATOR', 1),
(713,'R', 'ADPlanSASCode' ,8, 'HCM ANALYST', 1)
 ) by PASSTHRU;
/*************************/
/*       SAS_CUBES       */
/*************************/
 execute (Insert into SAS_CUBES (CUBE_ID,BASE_TABLE_NM,CUBE_NM,CUBE_DESC,USER_DEFINED,DIMENSIONS,MEAS_STATS,MACRO_CODE,SECURITY_CD,BUILD_DT) Values
('1','ABSHMAST','ABSHCUBE',"%getlabel(ABSHCUBE,Absence History Cube)",'0',NULL,NULL,'%ABSHCUBE;','0',NULL),
('2','ACTHMAST','ACTHCUBE',"%getlabel(ACTHCUBE,Action History Cube)",'0',NULL,NULL,'%ACTHCUBE;','0',NULL),
('3','APPHMAST','APPHCUBE',"%getlabel(APPHCUBE,Applicant History Cube)",'0',NULL,NULL,'%APPHCUBE;','0',NULL),
('4','EMPMAST','EMPCUBE',"%getlabel(EMPCUBE,Employee Master Cube)",'0',NULL,NULL,'%EMPCUBE;','0',NULL),
('5','HEADSUM','HDSMCUBE',"%getlabel(HDSMCUBE,Headcount Summary Cube)",'0',NULL,NULL,'%HDSMCUBE;','0',NULL),
('6','OPOSMAST','OPOSCUBE',"%getlabel(OPOSCUBE,Open Position Cube)",'0',NULL,NULL,'%OPOSCUBE;','0',NULL),
('7','OPOSSUM','OPSMCUBE',"%getlabel(OPSMCUBE,Open Position Summary Cube)",'0',NULL,NULL,'%OPSMCUBE;','0',NULL),
('8','SALHIST','SALHCUBE',"%getlabel(SALHCUBE,Salary History Cube)",'0',NULL,NULL,'%SALHCUBE;','0',NULL),
('9','TERMMAST','TERMCUBE',"%getlabel(TERMCUBE,Terminations Cube)",'0',NULL,NULL,'%TERMCUBE;','0',NULL),
('10','TIP','TIPCUBE',"%getlabel(TIPCUBE,Time in Position Cube)",'0',NULL,NULL,'%TIPCUBE;','0',NULL)
 ) by PASSTHRU;
/*************************/
/*   SAS_DIMENSIONS      */
/*************************/
 execute (insert into SAS_DIMENSIONS (TABLE_ID,Dimension_nm,Dimension_lbl,Dimension_type,Level_nm,Level_order) Values
(1,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(1,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(1,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(1,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(1,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(1,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(1,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(1,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(1,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(1,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(1,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(1,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(1,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(2,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(2,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(2,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(2,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(2,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(2,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(2,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(2,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(2,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(2,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(2,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(2,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(2,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(3,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(3,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(3,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(3,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(3,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(3,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(3,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(3,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(3,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(3,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(3,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(3,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(3,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(4,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(4,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(4,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(4,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(4,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(4,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(4,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(4,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(4,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(4,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(4,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(4,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(4,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(5,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(5,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(5,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(5,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(5,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(5,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(5,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(5,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(5,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(5,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(5,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(5,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(5,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(6,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(6,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(6,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(6,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(6,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(6,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(6,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(6,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(6,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(6,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(6,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(6,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(6,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(8,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(8,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(8,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(8,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(8,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(8,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(8,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(8,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(8,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(8,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(8,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(8,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(8,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(9,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(9,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(9,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(9,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(9,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(9,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(9,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(9,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(9,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(9,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(9,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(9,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(9,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(11,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(11,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(11,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(11,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(11,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(11,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(11,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(11,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(11,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(11,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(11,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(11,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(11,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(16,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(16,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(16,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(16,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(16,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(16,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(16,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(16,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(16,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(16,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(16,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(16,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(16,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(18,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(18,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(18,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(18,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(18,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(18,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(18,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(18,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(18,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(18,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(18,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(18,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(18,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
,(20,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','CITY_NM',3)
,(20,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','COUNTRY_CD',1)
,(20,'GEODIM',"%getlabel(DIM_GEODIM,Geography Dimension)",'0','STATE_REGION_CD',2)
,(20,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR1',5)
,(20,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR2',4)
,(20,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR3',3)
,(20,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR4',2)
,(20,'MGRDIM',"%getlabel(DIM_MGRDIM,Manager Dimension)",'0','INTORG_MGR5',1)
,(20,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR1',5)
,(20,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR2',4)
,(20,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR3',3)
,(20,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR4',2)
,(20,'ORGDIM',"%getlabel(DIM_ORGDIM,Organization Dimension)",'0','INTORG_HR5',1)
 ) by PASSTHRU;
/*************************/
/*  SAS_DOC_PROPERTIES   */
/*************************/
 execute (Insert into SAS_DOC_PROPERTIES Values
('70106e87-ac1a-4b9c-00be-76c78c788091','EmployeeProfile','','sasdemo','<?xml version=''1.0'' encoding=''UTF-8''?>\n<document type=''Employee Profile''><profileId>EmployeeProfile</profileId></document>')
 ) by PASSTHRU;
/*************************/
/*     SAS_GEOMAPPING    */
/*************************/
 execute (Insert into SAS_GEOMAPPING (LEVEL_NO,LEVEL_NM,MAP_NM,MAPLVL_NM,MAP_DEFAULT_VALUE,LINK_FIELD) Values
(1,"%getlabel(GEOMAPLEVLWORLD,World)",'HRV_GBL','COUNTRY_CD','USA','COUNTRY_CD')
,(2,"%getlabel(GEOMAPLEVLCOUNTRY,Country)",'HRV_US','STATE_REGION_CD','NC','STATE_REGION_CD')
,(3,"%getlabel(GEOMAPLEVLSTATE,State)",'HRV_STATE','COUNTY_NM','Wake','COUNTY_NM')
 ) by PASSTHRU;
/*************************/
/* SAS_HCM_PLAN_MEASURE  */
/*************************/
 execute (Insert into SAS_HCM_PLAN_MEASURE Values
(1,"%getlabel(MEAS_EMPLOYEETYPE,Employee Type)",'EMPMAST','EMPLOYEE_TYPE_CD','employeeType',1,2,0,1,2,0)
,(2,"%getlabel(MEAS_JOBGROUP,Job Group)",'EMPMAST','JOB_GROUP_DESC','jobGroup',1,2,0,1,2,0)
,(3,"%getlabel(MEAS_JOBCODE,Job Code)",'EMPMAST','JOB_CD','JobCode',1,2,0,1,2,0)
,(4,"%getlabel(MEAS_YEARSOFEXPERIENCE,Years of Experience)",'','','yearsOfExperience',1,2,0,0,1,0)
,(5,"%getlabel(MEAS_TENURE,Tenure)",'EMPMAST','SERVICE_YEARS','tenure',1,1,0,0,0,0)
,(6,"%getlabel(MEAS_JOBTITLE,Job Title)",'EMPMAST','JOB_TITLE_TXT','jobTitle',1,2,0,1,0,0)
,(7,"%getlabel(MEAS_ANNUALSALARY,Annual Salary)",'EMPMAST','ANNUAL_SALARY','currentAnnualSalary',1,2,1,0,3,0)
,(8,"%getlabel(MEAS_EMPLOYEEID,Employee Id)",'EMPMAST','EMPLOYEE_ID','empId',1,1,0,1,0,0)
,(9,"%getlabel(MEAS_HIREDATE,Hire Date)",'EMPMAST','HIRE_DT','hireDate',1,2,0,2,0,0)
,(10,"%getlabel(MEAS_SERVICEBEGINDATE,Service Begin Date)",'EMPMAST','SERVICE_START_DT','serviceBeginDate',1,2,0,2,0,0)
,(11,"%getlabel(MEAS_GENDER,Gender)",'EMPMAST','GENDER_CD','gender',1,1,0,1,0,0)
,(12,"%getlabel(MEAS_RANGEPENETRATION,Range Penetration)",'EMPMAST','RNG_PENE','rangePenetration',1,1,0,0,0,0)
,(13,"%getlabel(MEAS_EMPLOYEESTATUS,Employee Status)",'EMPMAST','EMPLOYEE_STATUS_CD','empStatus',1,1,0,1,0,0)
,(14,"%getlabel(MEAS_EMPLOYEEEDUCATION,Employee Education)",'EDUHIST','DEGREE_DESCRIPTION','empEducation',1,1,0,1,0,0)
,(15,"%getlabel(MEAS_MANAGERNAME,Manager Name)",'EMPMAST','MANAGER_NAME','managerName',1,1,0,1,0,0)
,(16,"%getlabel(MEAS_EEOCLASS,EEO Class)",'EMPMAST','EEO_CLASS_CD','eeoClass',1,1,0,1,0,0)
,(17,"%getlabel(MEAS_PAYROLLCURRENCY,Payroll Currency)",'EMPMAST','CURRENCY_CD','payCurrency',1,1,0,1,0,0)
,(18,"%getlabel(MEAS_PROFMEMBERSHIPCOST,Prof Membership Cost)",'','','profMembershipCost',1,0,0,0,3,0)
,(19,"%getlabel(MEAS_TRAININGCOST,Training Cost)",'','','trainingCost',1,0,0,0,3,0)
,(20,"%getlabel(MEAS_TRAVELCOST,Travel Cost)",'','','travelCost',1,0,0,0,3,0)
 ) by PASSTHRU;
/*************************/
/* SAS_HCM_PLAN_SAS_CODE */
/*************************/
 execute (Insert into SAS_HCM_PLAN_SAS_CODE Values
(1,'/*\r\nFollowing code is provided as a sample code snippet. A site
\r\nwould have to alter it to fit to their needs. Please consult
\r\nHCM administration guide for help.
\r\n*/
\r\n
\r\n/* Allocate HCM libraries */
\r\n%hcmlib;
\r\n
\r\n/*
\r\nNew Employee Suggested Salary Calculation
\r\n Computes mid-point annual salary amount based on
\r\n years of experience, job group nd employee type
\r\n*/
\r\n%nuempsal(nexp=&yearsOfExperience, jobgrp=&jobGroup, emptype=&employeeType);
\r\n
\r\n/*
\r\nNew Employee Suggested Bonus Calculation
\r\n Computes mid-point bonus amount with-in same job group
\r\n*/
\r\n%nuempbns(jobgrp=&jobGroup);
\r\n
\r\n/*
\r\nNew Employee Suggested Travel Calculation
\r\n Computes mid-point travel amount with-in same job group
\r\n*/
\r\n%nuemptvl(jobgrp=&jobGroup);\r\n',NULL,0,'')
 ) by PASSTHRU;
/*************************/
/*     SAS_INFOMAPS      */
/*************************/
 execute (Insert into SAS_INFOMAPS (IMAP_ID,IMAP_TYPE,SOURCE_NM,IMAP_NM,IMAP_DESC,SECURITY_CD,BUILD_DT,IMAP_CD,METADATA_ID) Values
('1','0','EMPMAST','EMPMAST_MAP',"%getlabel(EMPMAST_MAP,Employee Master Table Map)",NULL,NULL, 'CD1', ''),
('2','0','ABSHMAST','ABSHMAST_MAP',"%getlabel(ABSHMAST_MAP,Absence History Table Map)",NULL,NULL, 'CD2', ''),
('3','0','ACTHMAST','ACTHMAST_MAP',"%getlabel(ACTHMAST_MAP,Action History Table Map)",NULL,NULL, 'CD3', ''),
('4','0','HEADSUM','HEADSUM_MAP',"%getlabel(HEADSUM_MAP,Headcount Summary Table Map)",NULL,NULL, 'CD4', ''),
('5','0','OPOSMAST','OPOSMAST_MAP',"%getlabel(OPOSMAST_MAP,Open Position Table Map)",NULL,NULL, 'CD5', ''),
('6','0','OPOSSUM','OPOSSUM_MAP',"%getlabel(OPOSSUM_MAP,Open Position Summary Table Map)",NULL,NULL, 'CD6', ''),
('7','0','TERMMAST','TERMMAST_MAP',"%getlabel(TERMMAST_MAP,Terminations Table Map)",NULL,NULL, 'CD7', ''),
('8','0','TIP','TIP_MAP',"%getlabel(TIP_MAP,Time in Position Table Map)",NULL,NULL, 'CD8', ''),
('9','0','SALHIST','SALHIST_MAP',"%getlabel(SALHIST_MAP,Salary History Table Map)",NULL,NULL, 'CD9', ''),
('10','0','SALHSUM','SALHSUM_MAP',"%getlabel(SALHSUM_MAP,Salary Summary Table Map)",NULL,NULL, 'CD10', ''),
('11','1','ABSHCUBE','ABSHCUBE_MAP',"%getlabel(ABSHCUBE_MAP,Absence History Cube Map)",NULL,NULL, 'CD11', ''),
('12','1','ACTHCUBE','ACTHCUBE_MAP',"%getlabel(ACTHCUBE_MAP,Action History Cube Map)",NULL,NULL, 'CD12', ''),
('13','1','APPHCUBE','APPHCUBE_MAP',"%getlabel(APPHCUBE_MAP,Applicant History Cube Map)",NULL,NULL, 'CD13', ''),
('14','1','EMPCUBE','EMPCUBE_MAP',"%getlabel(EMPCUBE_MAP,Employee Master Cube Map)",NULL,NULL, 'CD14', ''),
('15','1','HDSMCUBE','HDSMCUBE_MAP',"%getlabel(HDSMCUBE_MAP,Headcount Summary Cube Map)",NULL,NULL, 'CD15', ''),
('16','1','OPOSCUBE','OPOSCUBE_MAP',"%getlabel(OPOSCUBE_MAP,Open Position Cube Map)",NULL,NULL, 'CD16', ''),
('17','1','OPSMCUBE','OPSMCUBE_MAP',"%getlabel(OPSMCUBE_MAP,Open Position Summary Cube Map)",NULL,NULL, 'CD17', ''),
('18','1','SALHCUBE','SALHCUBE_MAP',"%getlabel(SALHCUBE_MAP,Salary History Cube Map)",NULL,NULL, 'CD18', ''),
('19','1','TERMCUBE','TERMCUBE_MAP',"%getlabel(TERMCUBE_MAP,Terminations Cube Map)",NULL,NULL, 'CD19', ''),
('20','1','TIPCUBE','TIPCUBE_MAP',"%getlabel(TIPCUBE_MAP,Time in Position Cube Map)",NULL,NULL, 'CD20', '')
 ) by PASSTHRU;
/*************************/
/* SAS_MEASURE_FORMULAS  */
/*************************/
 execute (Insert into SAS_MEASURE_FORMULAS Values
(1,'FTE','30','','','',0,'',0,'')
,(2,'FTECont','30','','','',0,'',0,'')
,(3,'FTEContWFOffP','30','','','',0,'',0,'')
,(4,'FTEContWFOnP','30','','','',0,'',0,'')
,(5,'FTEEx','30','','','',0,'',0,'')
,(6,'FTENonEx','30','','','',0,'',0,'')
,(7,'FTEOfficeand','30','','','',0,'',0,'')
,(8,'FTEOfficials','30','','','',0,'',0,'')
,(9,'FTEProfession','30','','','',0,'',0,'')
,(10,'FTEReg','30','','','',0,'',0,'')
,(11,'FTERegWFOffP','30','','','',0,'',0,'')
,(12,'FTERegWFOnP','30','','','',0,'',0,'')
,(13,'FTEService_Ma','30','','','',0,'',0,'')
,(14,'FTESkilledCr','30','','','',0,'',0,'')
,(15,'FTETechnician','30','','','',0,'',0,'')
,(16,'FTEWFOffP','30','','','',0,'',0,'')
,(17,'FTEWFOnP','30','','','',0,'',0,'')
,(18,'HC','30','',"%getlabel(FACT_HC,Employee Headcount)",'',0,'',0,'')
,(19,'HCCont','30','','','',0,'',0,'')
,(20,'HCContEx','30','','','',0,'',0,'')
,(21,'HCContNonEx','30','','','',0,'',0,'')
,(22,'HCContWFOffP','30','','','',0,'',0,'')
,(23,'HCContWFOnP','30','','','',0,'',0,'')
,(24,'HCOfficeand','30','','','',0,'',0,'')
,(25,'HCOfficials','30','','','',0,'',0,'')
,(26,'HCProfession','30','','','',0,'',0,'')
,(27,'HCReg','30','','','',0,'',0,'')
,(28,'HCRegEx','30','','','',0,'',0,'')
,(29,'HCRegNonEx','30','','','',0,'',0,'')
,(30,'HCRegWFOffP','30','','','',0,'',0,'')
,(31,'HCRegWFOnP','30','','','',0,'',0,'')
,(32,'HCService_Ma','30','','','',0,'',0,'')
,(33,'HCSkilledCr','30','','','',0,'',0,'')
,(34,'HCTechnician','30','','','',0,'',0,'')
,(35,'HCWFOffP','30','','','',0,'',0,'')
,(36,'HCWFOnP','30','','','',0,'',0,'')
,(37,'Hires','30','','','',0,'',0,'')
,(38,'HiresEx','30','','','',0,'',0,'')
,(39,'HiresNonEx','30','','','',0,'',0,'')
,(40,'Seps','30','',"%getlabel(FACT_SEPS,Total Separations)",'',0,'',0,'')
,(41,'SepsEx','30','','','',0,'',0,'')
,(42,'SepsInvol','30','',"%getlabel(FACT_SEPSINVOL,Total Involuntary Separations)",'',0,'',0,'')
,(43,'SepsInvolEx','30','','','',0,'',0,'')
,(44,'SepsInvolNonEx','30','','','',0,'',0,'')
,(45,'SepsInvolOfficeand','30','','','',0,'',0,'')
,(46,'SepsInvolOfficials','30','','','',0,'',0,'')
,(47,'SepsInvolProfession','30','','','',0,'',0,'')
,(48,'SepsInvolService_Ma','30','','','',0,'',0,'')
,(49,'SepsInvolSkilledCr','30','','','',0,'',0,'')
,(50,'SepsInvolTechnician','30','','','',0,'',0,'')
,(51,'SepsNonEx','30','','','',0,'',0,'')
,(52,'SepsOfficeand','30','','','',0,'',0,'')
,(53,'SepsOfficials','30','','','',0,'',0,'')
,(54,'SepsProfession','30','','','',0,'',0,'')
,(55,'SepsService_Ma','30','','','',0,'',0,'')
,(56,'SepsSkilledCr','30','','','',0,'',0,'')
,(57,'SepsTechnician','30','','','',0,'',0,'')
,(58,'SepsVol','30','',"%getlabel(FACT_SEPSVOL,Total Voluntary Separations)",'',0,'',0,'')
,(59,'SepsVol0to1','30','',"%getlabel(FACT_SEPSVOL0TO1,Voluntary Separations by LOS - 0 to 1 Year)",'',0,'',0,'')
,(60,'SepsVol10plus','30','',"%getlabel(FACT_SEPSVOL10PLUS,Voluntary Separations by LOS - 10+ Years)",'',0,'',0,'')
,(61,'SepsVol1to3','30','',"%getlabel(FACT_SEPSVOL1TO3,Voluntary Separations by LOS - 1+ to 3 Years)",'',0,'',0,'')
,(62,'SepsVol3to5','30','',"%getlabel(FACT_SEPSVOL3TO5,Voluntary Separations by LOS - 3+ to 5 Years)",'',0,'',0,'')
,(63,'SepsVol5to10','30','',"%getlabel(FACT_SEPSVOL5TO10,Voluntary Separations by LOS - 5+ to 10 Years)",'',0,'',0,'')
,(64,'SepsVolEx','30','','','',0,'',0,'')
,(65,'SepsVolEx0to1','30','','','',0,'',0,'')
,(66,'SepsVolEx10plus','30','','','',0,'',0,'')
,(67,'SepsVolEx1to3','30','','','',0,'',0,'')
,(68,'SepsVolEx3to5','30','','','',0,'',0,'')
,(69,'SepsVolEx5to10','30','','','',0,'',0,'')
,(70,'SepsVolNonEx','30','','','',0,'',0,'')
,(71,'SepsVolNonEx0to1','30','','','',0,'',0,'')
,(72,'SepsVolNonEx10plus','30','','','',0,'',0,'')
,(73,'SepsVolNonEx1to3','30','','','',0,'',0,'')
,(74,'SepsVolNonEx3to5','30','','','',0,'',0,'')
,(75,'SepsVolNonEx5to10','30','','','',0,'',0,'')
,(76,'SepsVolOfficeand','30','','','',0,'',0,'')
,(77,'SepsVolOfficeand0to1','30','','','',0,'',0,'')
,(78,'SepsVolOfficeand10plus','30','','','',0,'',0,'')
,(79,'SepsVolOfficeand1to3','30','','','',0,'',0,'')
,(80,'SepsVolOfficeand3to5','30','','','',0,'',0,'')
,(81,'SepsVolOfficeand5to10','30','','','',0,'',0,'')
,(82,'SepsVolOfficials','30','','','',0,'',0,'')
,(83,'SepsVolOfficials0to1','30','','','',0,'',0,'')
,(84,'SepsVolOfficials10plus','30','','','',0,'',0,'')
,(85,'SepsVolOfficials1to3','30','','','',0,'',0,'')
,(86,'SepsVolOfficials3to5','30','','','',0,'',0,'')
,(87,'SepsVolOfficials5to10','30','','','',0,'',0,'')
,(88,'SepsVolProfession','30','','','',0,'',0,'')
,(89,'SepsVolProfession0to1','30','','','',0,'',0,'')
,(90,'SepsVolProfession10plus','30','','','',0,'',0,'')
,(91,'SepsVolProfession1to3','30','','','',0,'',0,'')
,(92,'SepsVolProfession3to5','30','','','',0,'',0,'')
,(93,'SepsVolProfession5to10','30','','','',0,'',0,'')
,(94,'SepsVolService_Ma','30','','','',0,'',0,'')
,(95,'SepsVolService_Ma0to1','30','','','',0,'',0,'')
,(96,'SepsVolService_Ma10plus','30','','','',0,'',0,'')
,(97,'SepsVolService_Ma1to3','30','','','',0,'',0,'')
,(98,'SepsVolService_Ma3to5','30','','','',0,'',0,'')
,(99,'SepsVolService_Ma5to10','30','','','',0,'',0,'')
,(100,'SepsVolSkilledCr','30','','','',0,'',0,'')
,(101,'SepsVolSkilledCr0to1','30','','','',0,'',0,'')
,(102,'SepsVolSkilledCr10plus','30','','','',0,'',0,'')
,(103,'SepsVolSkilledCr1to3','30','','','',0,'',0,'')
,(104,'SepsVolSkilledCr5to10','30','','','',0,'',0,'')
,(105,'SepsVolTechnician','30','','','',0,'',0,'')
,(106,'SepsVolTechnician0to1','30','','','',0,'',0,'')
,(107,'SepsVolTechnician10plus','30','','','',0,'',0,'')
,(108,'SepsVolTechnician1to3','30','','','',0,'',0,'')
,(109,'SepsVolTechnician3to5','30','','','',0,'',0,'')
,(110,'SepsVolTechnician5to10','30','','','',0,'',0,'')
,(111,'Tenure','30','','','',0,'',0,'')
,(112,'TenureEx','30','','','',0,'',0,'')
,(113,'TenureNonEx','30','','','',0,'',0,'')
,(114,'Xfrs','30','','','',0,'',0,'')
,(115,'XfrsEx','30','','','',0,'',0,'')
,(116,'XfrsNonEx','30','','','',0,'',0,'')
,(117,'Comp','30','','','',0,'',0,'')
,(118,'CompCont','30','',"%getlabel(FACT_COMPCONT,Compensation - Contingent - Total)",'',0,'',0,'')
,(119,'CompOfficeand','30','','','',0,'',0,'')
,(120,'CompOfficials','30','','','',0,'',0,'')
,(121,'CompProfession','30','','','',0,'',0,'')
,(122,'CompReg','30','','','',0,'',0,'')
,(123,'CompService_Ma','30','','','',0,'',0,'')
,(124,'CompSkilledCr','30','','','',0,'',0,'')
,(125,'CompTechnician','30','','','',0,'',0,'')
,(126,'CompVar','30','','','',0,'',0,'')
,(127,'CompWFOnP','30','','','',0,'',0,'')
,(128,'SepsVolSkilledCr3to5','30','','','',0,'',0,'')
,(129,'BenCost','30','11.',"%getlabel(FACT_BENCOST,Cost of Benefits)",'',0,'',0,'')
,(130,'BenCostEPTNW','30','11.','','',0,'',0,'')
,(131,'CapCost','30','11.','','',0,'',0,'')
,(132,'NOPAT','30','11.','','',0,'',0,'')
,(133,'OpExp','30','11.',"%getlabel(FACT_OPEXP,Operating Expenses)",'',0,'',0,'')
,(134,'Revenue','30','11.','','',0,'',0,'')
,(135,'end_dt','30','DATE9.',"%getlabel(FACT_END_DT,End Date)",'',0,'',0,'')
,(137,'AccRateEx','4','percent6.2',"%getlabel(ACCRATEEX,Accession Rate - Exempt)",'HiresEx / HCRegEx',1,'',6,'')
,(138,'AccRateNonex','4','percent6.2',"%getlabel(ACCRATENONEX,Accession Rate - Nonexempt)",'HiresNonex / HCRegNonex',1,'',6,'')
,(139,'AccRateExt','4','percent6.2',"%getlabel(ACCRATEEXT,Accession Rate - External - Total)",'HiresExt / HCReg',1,'',6,'')
,(140,'AccRateExtEx','4','percent6.2',"%getlabel(ACCRATEEXTEX,Accession Rate - External - Exempt)",'HiresExExt / HCRegEx',1,'',6,'')
,(141,'AccRateExtNonex','4','percent6.2',"%getlabel(ACCRATEEXTNONEX,Accession Rate - External - Nonexempt)",'HiresNonexExt / HCRegNonex',1,'',6,'')
,(142,'AccRateInt','4','percent6.2',"%getlabel(ACCRATEINT,Accession Rate - Internal - Total)",'HiresInt / HCReg',1,'',6,'')
,(143,'AccRateIntEx','4','percent6.2',"%getlabel(ACCRATEINTEX,Accession Rate - Internal - Exempt)",'HiresExInt / HCRegEx',1,'',6,'')
,(144,'AccRateIntNonex','4','percent6.2',"%getlabel(ACCRATEINTNONEX,Accession Rate - Internal - Nonexempt)",'HiresNonexInt / HCRegNonex',1,'',6,'')
,(146,'AddRate','4','percent6.2',"%getlabel(ADDRATE,Add Rate - Total)",'HiresAdd / HCReg',1,'',6,'')
,(147,'AddRateEx','4','percent6.2',"%getlabel(ADDRATEEX,Add Rate - Exempt)",'HiresExAdd / HCRegEx',1,'',6,'')
,(148,'AddRateNonex','4','percent6.2',"%getlabel(ADDRATENONEX,Add Rate - Nonexempt)",'HiresNonexAdd / HCRegNonex',1,'',6,'')
,(149,'AddRateExt','4','percent6.2',"%getlabel(ADDRATEEXT,Add Rate - External - Total)",'HiresExtAdd / HCReg',1,'',6,'')
,(150,'AddRateExtEx','4','percent6.2',"%getlabel(ADDRATEEXTEX,Add Rate - External - Exempt)",'HiresExExtAdd / HCRegEx',1,'',6,'')
,(151,'AddRateExtNonex','4','percent6.2',"%getlabel(ADDRATEEXTNONEX,Add Rate - External - Nonexempt)",'HiresNonexExtAdd / HCRegNonex',1,'',6,'')
,(152,'AddRateInt','4','percent6.2',"%getlabel(ADDRATEINT,Add Rate - Internal - Total)",'HiresIntAdd / HCReg',1,'',6,'')
,(153,'AddRateIntEx','4','percent6.2',"%getlabel(ADDRATEINTEX,Add Rate - Internal - Exempt)",'HiresExIntAdd / HCRegEx',1,'',6,'')
,(154,'AddRateIntNonex','4','percent6.2',"%getlabel(ADDRATEINTNONEX,Add Rate - Internal - Nonexempt)",'HiresNonexIntAdd / HCRegNonex',1,'',6,'')
,(155,'ReplRate','4','percent6.2',"%getlabel(REPLRATE,Replacement Rate - Total)",'HiresRepl / HCReg',1,'',6,'')
,(156,'ReplRateEx','4','percent6.2',"%getlabel(REPLRATEEX,Replacement Rate - Exempt)",'HiresExlRepl / HCRegEx',1,'',6,'')
,(157,'ReplRateNonex','4','percent6.2',"%getlabel(REPLRATENONEX,Replacement Rate - Nonexempt)",'HiresNonexRepl / HCRegNonex',1,'',6,'')
,(158,'ReplRateExt','4','percent6.2',"%getlabel(REPLRATEEXT,Replacement Rate - External - Total)",'HiresExtRepl / HCReg',1,'',6,'')
,(159,'ReplRateExtEx','4','percent6.2',"%getlabel(REPLRATEEXTEX,Replacement Rate - External - Exempt)",'HiresExExtRepl / HCRegEx',1,'',6,'')
,(160,'ReplRateExtNonex','4','percent6.2',"%getlabel(REPLRATEEXTNONEX,Replacement Rate - External - Nonexempt)",'HiresNonexExtRepl / HCRegNonex',1,'',6,'')
,(161,'ReplRateInt','4','percent6.2',"%getlabel(REPLRATEINT,Replacement Rate - Internal - Total)",'HiresIntRepl / HCReg',1,'',6,'')
,(162,'ReplRateIntEx','4','percent6.2',"%getlabel(REPLRATEINTEX,Replacement Rate - Internal - Exempt)",'HiresExIntRepl / HCRegEx',1,'',6,'')
,(163,'ReplRateIntNonex','4','percent6.2',"%getlabel(REPLRATEINTNONEX,Replacement Rate - Internal - Nonexempt)",'HiresNonexIntRepl / HCRegNonex',1,'',6,'')
,(164,'CPR','4','percent6.2',"%getlabel(CPR,Career Path Ratio - Total)",'Prom / ( Prom + Xfrs )',1,'',6,'')
,(165,'CPREx','4','percent6.2',"%getlabel(CPREX,Career Path Ratio - Exempt)",'PromEx / ( PromEx + XfrsEx )',1,'',6,'')
,(166,'CPRNonex','4','percent6.2',"%getlabel(CPRNONEX,Career Path Ratio - Nonexempt)",'PromNonex / sum( PromNonex, XfrsNonex)',1,'',6,'')
,(167,'CPH','4','dollar10.2',"%getlabel(CPH,Cost Per Hire - Total)",'( HiringCost * 1.1 ) / Hires',1,'',6,'')
,(168,'CPHEx','4','dollar10.2',"%getlabel(CPHEX,Cost Per Hire - Exempt)",'( HiringCostEx * 1.1 ) / HiresEx',1,'',6,'')
,(169,'CPHNonex','4','dollar10.2',"%getlabel(CPHNONEX,Cost Per Hire - Nonexempt)",'( HiringCostNonex * 1.1 ) / HiresNonex',1,'',6,'')
,(170,'CPHExt','4','dollar10.2',"%getlabel(CPHEXT,Cost Per Hire - External - Total)",'( HiringCostExt * 1.1 ) / HiresExt',1,'',6,'')
,(171,'CPHExtEx','4','dollar10.2',"%getlabel(CPHEXTEX,Cost Per Hire - External - Exempt)",'( HiringCostExExt * 1.1 ) / HiresExExt',1,'',6,'')
,(172,'CPHExtNonex','4','dollar10.2',"%getlabel(CPHEXTNONEX,Cost Per Hire - External Nonexempt)",'( HiringCostNonexExt * 1.1 ) / HiresNonexExt',1,'',6,'')
,(173,'CPHInt','4','dollar10.2',"%getlabel(CPHINT,Cost Per Hire - Internal - Total)",'( HiringCostInt * 1.1 ) / HiresInt',1,'',6,'')
,(174,'CPHIntEx','4','dollar10.2',"%getlabel(CPHINTEX,Cost Per Hire - Internal - Exempt)",'( HiringCostExInt * 1.1 ) / HiresExInt',1,'',6,'')
,(175,'CPHIntNonex','4','dollar10.2',"%getlabel(CPHINTNONEX,Cost Per Hire - Internal Nonexempt)",'( HiringCostNonexInt * 1.1 ) / HiresNonexInt',1,'',6,'')
,(176,'CPHClge','4','dollar10.2',"%getlabel(CPHCLGE,Cost per Hire - College - Total)",'( HiringCostClge * 1.1 ) / HiresClge',1,'',6,'')
,(177,'CPHAd','4','percent6.2',"%getlabel(CPHAD,Cost per Hire - Advertising)",'HiringCostAd / HiringCost',1,'',6,'')
,(178,'CPHAg','4','percent6.2',"%getlabel(CPHAG,Cost per Hire - Agency)",'HiringCostAg / HiringCost',1,'',6,'')
,(179,'CPHRfrrlBns','4','percent6.2',"%getlabel(CPHRFRRLBNS,Cost per Hire - Referral Bonuses)",'HiringCostRfrrlBns / HiringCost',1,'',6,'')
,(180,'CPHTrvl','4','percent6.2',"%getlabel(CPHTRVL,Cost per Hire - Travel)",'HiringCostTrvl / HiringCost',1,'',6,'')
,(181,'CPHReloc','4','percent6.2',"%getlabel(CPHRELOC,Cost per Hire - Relocation)",'HiringCostReloc / HiringCost',1,'',6,'')
,(182,'CPHRcrter','4','percent6.2',"%getlabel(CPHRCRTER,Cost per Hire - Recruiter)",'HiringCostRcrtr / HiringCost',1,'',6,'')
,(183,'CPHExtAd','4','percent6.2',"%getlabel(CPHEXTAD,Cost per Hire - External -Advertising)",'HiringCostExtAd / HiringCostExt',1,'',6,'')
,(184,'CPHExtAg','4','percent6.2',"%getlabel(CPHEXTAG,Cost per Hire - External - Agency)",'HiringCostExtAg / HiringCostExt',1,'',6,'')
,(185,'CPHExtRfrrlBns','4','percent6.2',"%getlabel(CPHEXTRFRRLBNS,Cost per Hire - External - Referral Bonuses)",'HiringCostExtRfrrlBns / HiringCostExt',1,'',6,'')
,(186,'CPHExtTrvl','4','percent6.2',"%getlabel(CPHEXTTRVL,Cost per Hire - External-Travel)",'HiringCostExtTrvl / HiringCostExt',1,'',6,'')
,(187,'CPHExtReloc','4','percent6.2',"%getlabel(CPHEXTRELOC,Cost per Hire - External-Relocation)",'HiringCostExtReloc / HiringCostExt',1,'',6,'')
,(188,'CPHExtRcrter','4','percent6.2',"%getlabel(CPHEXTRCRTER,Cost per Hire - External-Recruiter)",'HiringCostExtRcrtr / HiringCostExt',1,'',6,'')
,(189,'CPHIntAd','4','percent6.2',"%getlabel(CPHINTAD,Cost per Hire - Internal-Advertising)",'HiringCostIntAd / HiringCostInt',1,'',6,'')
,(190,'CPHIntTrvl','4','percent6.2',"%getlabel(CPHINTTRVL,Cost per Hire - Internal-Travel)",'HiringCostsIntTrvl / HiringCostInt',1,'',6,'')
,(191,'CPHIntReloc','4','percent6.2',"%getlabel(CPHINTRELOC,Cost per Hire - Internal-Relocation)",'HiringCostIntReloc / HiringCostInt',1,'',6,'')
,(192,'CPHIntRcrter','4','percent6.2',"%getlabel(CPHINTRCRTER,Cost per Hire - Internal-Recruiter)",'HiringCostIntRcrtr / HiringCostInt',1,'',6,'')
,(193,'TTF','4','comma6.',"%getlabel(TTF,Time to Fill - Total)",'DTF / Hires',1,'',6,'')
,(194,'TTFEx','4','comma6.',"%getlabel(TTFEX,Time to Fill - Exempt)",'DTFEx / HiresEx',1,'',6,'')
,(195,'TTFNonex','4','comma6.',"%getlabel(TTFNONEX,Time to Fill - Nonexempt)",'DTFNonex / HiresNonex',1,'',6,'')
,(196,'TTFExt','4','comma6.',"%getlabel(TTFEXT,Time to Fill - External - Total)",'DTFExt / HiresExt',1,'',6,'')
,(197,'TTFExtEx','4','comma6.',"%getlabel(TTFEXTEX,Time to Fill - External - Exempt)",'DTFExExt / HiresExExt',1,'',6,'')
,(198,'TTFExtNonex','4','comma6.',"%getlabel(TTFEXTNONEX,Time to Fill - External - Nonexempt)",'DTFNonexExt / HiresNonexExt',1,'',6,'')
,(199,'TTFInt','4','comma6.',"%getlabel(TTFINT,Time to Fill - Internal - Total)",'DTFInt / HiresInt',1,'',6,'')
,(200,'TTFIntEx','4','comma6.',"%getlabel(TTFINTEX,Time to Fill - Internal - Exempt)",'DTFExInt / HiresExInt',1,'',6,'')
,(201,'TTFIntNonex','4','comma6.',"%getlabel(TTFINTNONEX,Time to Fill - Internal - Nonexempt)",'DTFNonexInt / HiresNonexInt',1,'',6,'')
,(202,'TTS','4','comma6.',"%getlabel(TTS,Time to Start - Total)",'DTS / Hires',1,'',6,'')
,(203,'TTSEx','4','comma6.',"%getlabel(TTSEX,Time to Start - Exempt)",'DTSEx/ HiresEx',1,'',6,'')
,(204,'TTSNonex','4','comma6.',"%getlabel(TTSNONEX,Time to Start - Nonexempt)",'DTSNonex/ HiresNonex',1,'',6,'')
,(205,'TTSExt','4','comma6.',"%getlabel(TTSEXT,Time to Start - External - Total)",'DTSExt / HiresExt',1,'',6,'')
,(206,'TTSExtEx','4','comma6.',"%getlabel(TTSEXTEX,Time to Start - External - Exempt)",'DTSExExt / HiresExExt',1,'',6,'')
,(207,'TTSExtNonex','4','comma6.',"%getlabel(TTSEXTNONEX,Time to Start - External - Nonexempt)",'DTSNonexExt / HiresNonexExt',1,'',6,'')
,(208,'TTSInt','4','comma6.',"%getlabel(TTSINT,Time to Start - Internal - Total)",'DTSInt / HiresExt',1,'',6,'')
,(209,'TTSIntEx','4','comma6.',"%getlabel(TTSINTEX,Time to Start - Internal - Exempt)",'DTSExInt / HiresExInt',1,'',6,'')
,(210,'TTSIntNonex','4','comma6.',"%getlabel(TTSINTNONEX,Time to Start - Internal - Nonexempt)",'DTSNonexInt / HiresNonexInt',1,'',6,'')
,(211,'OAR','4','percent6.2',"%getlabel(OAR,Offer Acceptance Rate)",'OffersAccepted / OffersExtended',1,'',6,'')
,(212,'OARExt','4','percent6.2',"%getlabel(OAREXT,Offer Acceptance Rate - External)",'OffersAcceptedExt / OffersExtendedExt',1,'',6,'')
,(213,'OARClge','4','percent6.2',"%getlabel(OARCLGE,Offer Acceptance Rate - College)",'OffersAcceptedClge / OffersExtendedClge',1,'',6,'')
,(214,'SOBPct','4','percent6.2',"%getlabel(SOBPCT,Sign-On Bonus Percent)",'HiresRSOB / sum ( HiresExt,HiresClge )',1,'',6,'')
,(215,'SOBPctExec','4','percent6.2',"%getlabel(SOBPCTEXEC,Sign-On Bonus Percent - Executive)",'HiresExecRSOB / HiresExec',1,'',6,'')
,(216,'SOBPctMgr','4','percent6.2',"%getlabel(SOBPCTMGR,Sign-On Bonus Percent - Manager)",'HiresMgrRSOB / HiresMgr',1,'',6,'')
,(217,'SOBFctr','4','dollar8.2',"%getlabel(SOBFCTR,Sign-On Bonus Factor)",'SOBCost/ HiresRSOB',1,'',6,'')
,(218,'SOBFctrExec','4','dollar8.2',"%getlabel(SOBFCTREXEC,Sign-On Bonus Factor - Executive)",'SOBCostExec / HiresExecRSOB',1,'',6,'')
,(219,'SOBFctrMgr','4','dollar8.2',"%getlabel(SOBFCTRMGR,Sign-On Bonus Factor - Manager)",'SOBCostMgr / HiresMgrRSOB',1,'',6,'')
,(220,'RevFctr','4','dollar12.',"%getlabel(REVFCTR,Revenue Factor)",'Revenue / FTE',1,'',1,'')
,(221,'RevFctrReg','4','dollar12.',"%getlabel(REVFCTRREG,Revenue Factor - Regular Employees)",'Revenue / FTEReg',1,'',1,'')
,(222,'RevFctrWFonP','4','dollar12.',"%getlabel(REVFCTRWFONP,Revenue Factor - Workforce Employees)",'Revenue / FTEWFonP',1,'',1,'')
,(223,'ExpFctr','4','dollar12.',"%getlabel(EXPFCTR,Expense Factor)",'OpExp / FTE',1,'',1,'')
,(224,'ExpFctrReg','4','dollar12.',"%getlabel(EXPFCTRREG,Expense Factor - Regular Employees)",'OpExp / FTEReg',1,'',1,'')
,(225,'ExpFctrWFonP','4','dollar12.',"%getlabel(EXPFCTRWFONP,Expense Factor - Workforce Employees)",'OpExp / FTEWFonP',1,'',1,'')
,(226,'IncFctr','4','dollar12.',"%getlabel(INCFCTR,Income Factor)",'( Revenue - OpExp ) / FTE',1,'',1,'')
,(227,'IncFctrReg','4','dollar12.',"%getlabel(INCFCTRREG,Income Factor - Regular Employees)",'( Revenue - OpExp ) / FTEReg',1,'',1,'')
,(228,'IncFctrWFonP','4','dollar12.',"%getlabel(INCFCTRWFONP,Income Factor - Workforce Employees)",'( Revenue - OpExp ) / FTEWFonP',1,'',1,'')
,(229,'HCVA','4','dollar12.',"%getlabel(HCVA,Human Capital Value Added)",'( Revenue - ( OpExp - ( CompWFOnP  + BenCostEPTNW ))) / FTE',1,'',1,'')
,(230,'HCROI','4','comma10.2',"%getlabel(HCROI,Human Capital ROI)",'( Revenue - ( OpExp - ( CompWFOnP  + BenCostEPTNW ))) / sum( CompWFOnP, BenCostEPTNW )',1,'',1,'')
,(231,'HEVA','4','dollar12.',"%getlabel(HEVA,Human Economic Value Added)",'( NOPAT - CapCost ) / FTE',1,'',1,'')
,(232,'OEPct','4','percent6.2',"%getlabel(OEPCT,Outsourcing Expense Percent)",'ExpenseOutsrce / OpExp',1,'',1,'')
,(233,'MgrRatio','4','comma10.2',"%getlabel(MGRRATIO,Management Ratio)",'HC / HCMgmt',1,'',1,'')
,(234,'MgrRatioReg','4','comma10.2',"%getlabel(MGRRATIOREG,Management Ratio-Regular Employees)",'HCReg / HCMgmt',1,'',1,'')
,(235,'MgrInvstFctr','4','dollar12.',"%getlabel(MGRINVSTFCTR,Management Investment Factor)",'CompMgmt / FTE',1,'',1,'')
,(236,'MgrInvstFctrReg','4','dollar12.',"%getlabel(MGRINVSTFCTRREG,Management Investment Factor-Regular Employees)",'CompMgmt / FTEReg',1,'',1,'')
,(237,'AvgTenure','4','comma10.2',"%getlabel(AVGTENURE,Average Tenure - Total)",'Tenure / HCReg',1,'',1,'')
,(238,'AvgTenureEx','4','comma10.2',"%getlabel(AVGTENUREEX,Average Tenure - Exempt)",'TenureEx / HCRegEx',1,'',1,'')
,(239,'AvgTenureNonex','4','comma10.2',"%getlabel(AVGTENURENONEX,Average Tenure - Nonexempt)",'TenureNonex / HCRegNonex',1,'',1,'')
,(240,'HCPctCont','4','percent6.2',"%getlabel(HCPCTCONT,Headcount Percent - Contingent - Total)",'HCCont / HC',1,'',1,'')
,(241,'HCPctContReg','4','percent6.2',"%getlabel(HCPCTCONTREG,Headcount Percent - Contingent - Regular Employees)",'HCCont / HCReg',1,'',1,'')
,(242,'HCPctContOnP','4','percent6.2',"%getlabel(HCPCTCONTONP,Headcount Percent - Contingent - On Payroll)",'HCContWFonP / HC',1,'',1,'')
,(243,'HCPctContOffP','4','percent6.2',"%getlabel(HCPCTCONTOFFP,Headcount Percent - Contingent - Off Payroll)",'HCContWFOffP / HC',1,'',1,'')
,(244,'FTEPctCont','4','percent6.2',"%getlabel(FTEPCTCONT,FTE Percent - Contingent - Total)",'FTECont / FTE',1,'',1,'')
,(245,'FTEPctContReg','4','percent6.2',"%getlabel(FTEPCTCONTREG,FTE Percent - Contingent - Regular Employees)",'FTEContReg / FTE',1,'',1,'')
,(246,'FTEPctContOnP','4','percent6.2',"%getlabel(FTEPCTCONTONP,FTE Percent - Contingent - On Payroll)",'FTEContWFOnP / FTE',1,'',1,'')
,(247,'FTEPctContOffP','4','percent6.2',"%getlabel(FTEPCTCONTOFFP,FTE Percent - Contingent - Off Payroll)",'FTEContWFOffP / FTE',1,'',1,'')
,(248,'FTEPctMgmt','4','percent6.2',"%getlabel(FTEPCTMGMT,FTE Percent - Management)",'FTEMgmt / FTE',1,'',1,'')
,(249,'FTEPctProf','4','percent6.2',"%getlabel(FTEPCTPROF,FTE Percent - Professionals)",'FTEProf / FTE',1,'',1,'')
,(250,'FTEPctSales','4','percent6.2',"%getlabel(FTEPCTSALES,FTE Percent - Sales)",'FTESales / FTE',1,'',1,'')
,(251,'FTEPctOanC','4','percent6.2',"%getlabel(FTEPCTOANC,FTE Percent - Office & Clerical)",'FTEOandC / FTE',1,'',1,'')
,(252,'FTEPctOp','4','percent6.2',"%getlabel(FTEPCTOP,FTE Percent - Operatives)",'FTEOp / FTE',1,'',1,'')
,(253,'HRExpPct','4','percent6.2',"%getlabel(HREXPPCT,HR Expense Percent)",'ExpenseHR / OpExp',1,'',2,'')
,(254,'HRFTERatio','4','comma10.',"%getlabel(HRFTERATIO,HR FTE Ratio)",'FTE / FTEHR',1,'',2,'')
,(255,'HRFTERatioReg','4','comma10.',"%getlabel(HRFTERATIOREG,HR FTE Ratio - Regular Employees)",'FTEReg / FTEHR',1,'',2,'')
,(256,'HRExPct','4','percent6.2',"%getlabel(HREXPCT,HR Exempt Percent)",'FTEHREx / FTEHR',1,'',2,'')
,(257,'HRExPctReg','4','percent6.2',"%getlabel(HREXPCTREG,HR Exempt Percent - Regular Employees)",'FTEHREx / FTEHRReg',1,'',2,'')
,(258,'HRFTEInvFctr','4','dollar12.',"%getlabel(HRFTEINVFCTR,HR FTE Investment Factor)",'ExpenseHR / FTE',1,'',2,'')
,(259,'HRFTEInvFctrReg','4','dollar12.',"%getlabel(HRFTEINVFCTRREG,HR FTE Investment Factor - Regular Employees)",'ExpenseHR / FTEReg',1,'',2,'')
,(260,'HRHCInvFctr','4','dollar12.',"%getlabel(HRHCINVFCTR,HR Headcount Investment Factor)",'ExpenseHR / HC',1,'',2,'')
,(261,'HRHCInvFctrReg','4','dollar12.',"%getlabel(HRHCINVFCTRREG,HR Headcount Investment Factor - Regular Employees)",'ExpenseHR / HCReg',1,'',2,'')
,(262,'HROutsrcePct','4','percent6.2',"%getlabel(HROUTSRCEPCT,HR Outsourcing Percent)",'OutsrceCostHR / ExpenseHR',1,'',2,'')
,(263,'HRConsPct','4','percent6.2',"%getlabel(HRCONSPCT,HR Consulting Percent)",'ConsCostHR / ExpenseHR',1,'',2,'')
,(264,'HRCompExpPct','4','percent6.2',"%getlabel(HRCOMPEXPPCT,HR Compensation Expense Percent)",'CompHR / OpExp',1,'',2,'')
,(265,'HREmpCostFctr','4','dollar12.',"%getlabel(HREMPCOSTFCTR,HR Employee Cost Factor)",'CompHR / FTEHRReg',1,'',2,'')
,(266,'HREECostFctr','4','dollar12.',"%getlabel(HREECOSTFCTR,HR Total Employee Cost Factor)",'CompHR * ( 1.0 + BenCostEPTNW / CompWFOnP ) / FTEHRReg',1,'',2,'')
,(267,'HRSepsRate','4','percent6.2',"%getlabel(HRSEPSRATE,HR Separation Rate - Total)",'SepsHR / HCHR',1,'',2,'')
,(268,'HRSepsRateEx','4','percent6.2',"%getlabel(HRSEPSRATEEX,HR Separation Rate - Exempt)",'SepsHREx / HCHREx',1,'',2,'')
,(269,'HRSepsRateNonex','4','percent6.2',"%getlabel(HRSEPSRATENONEX,HR Separation Rate - Nonexempt)",'SepsHRNonex / HCHRNonex',1,'',2,'')
,(270,'HRStructAdmin','4','percent6.2',"%getlabel(HRSTRUCTADMIN,HR Structure Breakdown - Administrative)",'FTEHRAdmin / FTEHR',1,'',2,'')
,(271,'HRStructBen','4','percent6.2',"%getlabel(HRSTRUCTBEN,HR Structure Breakdown - Benefits)",'FTEHRBen / FTEHR',1,'',2,'')
,(272,'HRStructComp','4','percent6.2',"%getlabel(HRSTRUCTCOMP,HR Structure Breakdown - Compensation)",'FTEHRComp / FTEHR',1,'',2,'')
,(273,'HRStructEmprel','4','percent6.2',"%getlabel(HRSTRUCTEMPREL,HR Structure Breakdown - Employee Relations)",'FTEHREmprel / FTEHR',1,'',2,'')
,(274,'HRStructHRIS','4','percent6.2',"%getlabel(HRSTRUCTHRIS,HR Structure Breakdown - HRIS)",'FTEHRHRIS / FTEHR',1,'',2,'')
,(275,'HRStructMgmt','4','percent6.2',"%getlabel(HRSTRUCTMGMT,HR Structure Breakdown - HR Management)",'FTEHRMgmt / FTEHR',1,'',2,'')
,(276,'HRStructLgl','4','percent6.2',"%getlabel(HRSTRUCTLGL,HR Structure Breakdown - Legal)",'FTEHRLgl / FTEHR',1,'',2,'')
,(277,'HRStructStaff','4','percent6.2',"%getlabel(HRSTRUCTSTAFF,HR Structure Breakdown - Staffing)",'FTEHRStaff / FTEHR',1,'',2,'')
,(278,'CompRevPct','4','percent6.2',"%getlabel(COMPREVPCT,Compensation Revenue Percent)",'CompWFOnP  / Revenue',1,'',3,'')
,(279,'CompRevPctReg','4','percent6.2',"%getlabel(COMPREVPCTREG,Compensation Revenue Percent-Regular Employees)",'CompReg  / Revenue',1,'',3,'')
,(280,'TotCompRevPct','4','percent6.2',"%getlabel(TOTCOMPREVPCT,Total Compensation Revenue Percent)",'( CompWFOnP + BenCostEPTNW ) / Revenue',1,'',3,'')
,(281,'LaborCostRevPct','4','percent6.2',"%getlabel(LABORCOSTREVPCT,Total labor Cost Revenue Percent)",'( Comp - BenCostEPTNW ) / Revenue',1,'',3,'')
,(282,'CompExpPct','4','percent6.2',"%getlabel(COMPEXPPCT,Compensation Expense Percent)",'CompWFOnP  / OpExp',1,'',3,'')
,(283,'CompExpPctReg','4','percent6.2',"%getlabel(COMPEXPPCTREG,Compensation Expense Percent-Regular Employees)",'CompReg  / OpExp',1,'',3,'')
,(284,'TotCompExpPct','4','percent6.2',"%getlabel(TOTCOMPEXPPCT,Total Compensation Expense Percent)",'( CompWFOnP  + BenCostEPTNW ) / OpExp',1,'',3,'')
,(285,'LaborCostExpPct','4','percent6.2',"%getlabel(LABORCOSTEXPPCT,Total Labor Cost Expense Percent)",'( Comp + BenCostEPTNW )  / OpExp',1,'',3,'')
,(286,'CompPctExec','4','percent6.2',"%getlabel(COMPPCTEXEC,Compensation Percent - Executive)",'CompExec / CompWFOnP',1,'',3,'')
,(287,'CompPctMgr','4','percent6.2',"%getlabel(COMPPCTMGR,Compensation Percent - Manager)",'CompMgr / CompWFOnP',1,'',3,'')
,(288,'CompPctStaff','4','percent6.2',"%getlabel(COMPPCTSTAFF,Compensation Percent - Staff)",'CompStaff / CompWFOnP',1,'',3,'')
,(289,'CompPctVar','4','percent6.2',"%getlabel(COMPPCTVAR,Compensation Percent - Variable)",'CompVar / CompWFOnP',1,'',3,'')
,(290,'CompCont','4','percent6.2',"%getlabel(COMPCONT,Compensation - Contingent - Total)",'ContCost / CompWFOnP',1,'',3,'')
,(291,'CompContOnP','4','percent6.2',"%getlabel(COMPCONTONP,Compensation - Contingent - On Payroll)",'ContCostWFOnP / CompWFOnP',1,'',3,'')
,(292,'CompContOffP','4','percent6.2',"%getlabel(COMPCONTOFFP,Compensation - Contingent - Off Payroll)",'ContCostWFOffP / CompWFOnP',1,'',3,'')
,(293,'ContCostRevPct','4','percent6.2',"%getlabel(CONTCOSTREVPCT,Contingent Cost Revenue Percent - Total)",'ContCost / Revenue',1,'',3,'')
,(294,'ContCostRevPctOnP','4','percent6.2',"%getlabel(CONTCOSTREVPCTONP,Contingent Cost Revenue Percent - On Payroll)",'ContCostWFOnP / Revenue',1,'',3,'')
,(295,'ContCostRevPctOffP','4','percent6.2',"%getlabel(CONTCOSTREVPCTOFFP,Contingent Cost Revenue Percent - Off Payroll)",'ContCostWFOffP / Revenue',1,'',3,'')
,(296,'ContCostExpPct','4','percent6.2',"%getlabel(CONTCOSTEXPPCT,Contingent Cost Expense Percent - Total)",'ContCost / OpExp',1,'',3,'')
,(297,'ContCostExpPctOnP','4','percent6.2',"%getlabel(CONTCOSTEXPPCTONP,Contingent Cost Expense Percent - On Payroll)",'ContCostWFOnP / OpExp',1,'',3,'')
,(298,'ContCostExpPctOffP','4','percent6.2',"%getlabel(CONTCOSTEXPPCTOFFP,Contingent Cost Expense Percent - Off Payroll)",'ContCostWFOffP / OpExp',1,'',3,'')
,(299,'EECostFctrReg','4','dollar12.',"%getlabel(EECOSTFCTRREG,Employee Cost Factor - Regular Employees)",'CompReg / FTEReg',1,'',3,'')
,(300,'EECostFctrWF','4','dollar12.',"%getlabel(EECOSTFCTRWF,Employee Cost Factor - Workforce)",'CompWFOnP  / FTEWFonP',1,'',3,'')
,(301,'EECostFctrExec','4','dollar12.',"%getlabel(EECOSTFCTREXEC,Employee Cost Factor - Executive)",'CompExec / FTEWFonP',1,'',3,'')
,(302,'EECostFctrMgr','4','dollar12.',"%getlabel(EECOSTFCTRMGR,Employee Cost Factor - Manager)",'CompMgr / FTEWFonP',1,'',3,'')
,(303,'EECostFctrStaff','4','dollar12.',"%getlabel(EECOSTFCTRSTAFF,Employee Cost Factor - Staff)",'CompStaff / FTEWFonP',1,'',3,'')
,(304,'CostFctr','4','dollar12.',"%getlabel(COSTFCTR,Total Employee Cost Factor)",'( CompReg + BenCostEPTNW ) / FTEReg',1,'',3,'')
,(305,'CostFctrLabor','4','dollar12.',"%getlabel(COSTFCTRLABOR,Total Labor Cost Factor)",'( Comp + BenCostEPTNW ) / FTE',1,'',3,'')
,(306,'ContFctr','4','dollar12.',"%getlabel(CONTFCTR,Contingent Cost Factor - Total)",'ContCost / FTECont',1,'',3,'')
,(307,'ContFctrOnP','4','dollar12.',"%getlabel(CONTFCTRONP,Contingent Cost Factor - On Payroll)",'ContCostWFOnP / FTEContWFOnP',1,'',3,'')
,(308,'ContFctrOffP','4','dollar12.',"%getlabel(CONTFCTROFFP,Contingent Cost Factor - Off Payroll)",'ContCostWFOffP / FTEContWFOffP',1,'',3,'')
,(309,'BenRevPct','4','percent6.2',"%getlabel(BENREVPCT,Benefit Revenue Percent)",'BenCost / Revenue',1,'',4,'')
,(310,'BenExpPct','4','percent6.2',"%getlabel(BENEXPPCT,Benefit Expense Percent)",'BenCost / OpExp',1,'',4,'')
,(311,'BenCompPct','4','percent6.2',"%getlabel(BENCOMPPCT,Benefit Compensation Percent)",'BenCostEPTNW / CompWFOnP',1,'',4,'')
,(312,'BenCompPctReg','4','percent6.2',"%getlabel(BENCOMPPCTREG,Benefit Compensation Percent-Regular Employees)",'BenCostEPTNW / CompReg',1,'',4,'')
,(313,'BenFctr','4','dollar12.',"%getlabel(BENFCTR,Benefit Factor)",'BenCost / HCWFOnP',1,'',4,'')
,(314,'BenFctrReg','4','dollar12.',"%getlabel(BENFCTRREG,Benefit Factor-Regular Employees)",'BenCost / HCReg',1,'',4,'')
,(315,'HealthcareFctr','4','dollar12.',"%getlabel(HEALTHCAREFCTR,Healthcare Factor)",'MedBenPmt / CoveredEE',1,'',4,'')
,(316,'WorkersCompFctr','4','dollar12.',"%getlabel(WORKERSCOMPFCTR,Workers Compensation Factor)",'CompWorkers / HCWFOnP',1,'',4,'')
,(317,'WorkersCompFctrReg','4','dollar12.',"%getlabel(WORKERSCOMPFCTRREG,Workers Compensation Factor-Regular Employees)",'CompWorkers / HCReg',1,'',4,'')
,(318,'BenCostPmntsLglReqPm','4','percent6.2',"%getlabel(BENCOSTPMNTSLGLREQPM,Benefit Cost Breakdown - Legally Required Payments)",'PmntsLglReqPmnts / BenCost',1,'',4,'')
,(319,'BenCostPmntsRetSav','4','percent6.2',"%getlabel(BENCOSTPMNTSRETSAV,Benefit Cost Breakdown - Retirement & Savings Plan Payments)",'PmntsRetSav / BenCost',1,'',4,'')
,(320,'BenCostPmntsLifeInsD','4','percent6.2',"%getlabel(BENCOSTPMNTSLIFEINSD,Benefit Cost Breakdown - Life Insurance & Death Benefit Payments)",'PmntsLifeInsDthBen / BenCost',1,'',4,'')
,(321,'BenCostMedBenPmnts','4','percent6.2',"%getlabel(BENCOSTMEDBENPMNTS,Benefit Cost Breakdown - Medical & Medically Related Benefit Payments)",'MedBenPmt / BenCost',1,'',4,'')
,(322,'BenCostPmntsTmeNotWorked','4','percent6.2',"%getlabel(BENCOSTPMNTSTMENOTWORKED,Benefit Cost Breakdown - Payments for Time Not Worked Payments)",'PmntsMiscBen / BenCost',1,'',4,'')
,(323,'BenCostMiscBenPmnts','4','percent6.2',"%getlabel(BENCOSTMISCBENPMNTS,Benefit Cost Breakdown - Miscellaneous Benefit Payments)",'PmntsMiscBen / BenCost',1,'',4,'')
,(324,'SepsRate','4','percent6.2',"%getlabel(SEPSRATE,Separation Rate - Total)",'Seps / HCReg',1,'',5,'')
,(325,'SepsRateEx','4','percent6.2',"%getlabel(SEPSRATEEX,Separation Rate - Exempt)",'SepsEx / HCRegEx',1,'',5,'')
,(326,'SepsRateNonex','4','atenonex percent6.2',"%getlabel(SEPSRATENONEX,Separation Rate - Nonexempt)",'SepsNonex / HCRegNonex',1,'',5,'')
,(327,'SepsVolRate','4','percent6.2',"%getlabel(SEPSVOLRATE,Voluntary Separation Rate - Total)",'SepsVol / HCReg',1,'',5,'')
,(328,'SepsVolRateEx','4','percent6.2',"%getlabel(SEPSVOLRATEEX,Voluntary Separation Rate - Exempt)",'SepsVolEx / HCRegEx',1,'',5,'')
,(329,'SepsVolRateNonex','4','percent6.2',"%getlabel(SEPSVOLRATENONEX,Voluntary Separation Rate - Nonexempt)",'SepsVolNonex / HCRegNonex',1,'',5,'')
,(330,'SepsInvolRate','4','nvolRate percent6.2',"%getlabel(SEPSINVOLRATE,Involuntary Separation Rate - Total)",'SepsInvol / HCReg',1,'',5,'')
,(331,'SepsInvolRateEx','4','percent6.2',"%getlabel(SEPSINVOLRATEEX,Involuntary Separation Rate - Exempt)",'SepsInvolEx / HCRegEx',1,'',5,'')
,(332,'SepsInvolRateNonex','4','percent6.2',"%getlabel(SEPSINVOLRATENONEX,Involuntary Separation Rate - Nonexempt)",'SepsInvolNonex / HCRegNonex',1,'',5,'')
,(333,'SepsVol0to1','4','percent6.2',"%getlabel(SEPSVOL0TO1,Voluntary Separations by LOS - 0 to 1 Year)",'SepsVol0to1  / SepsVol',1,'',5,'')
,(334,'SepsVol1to3','4','percent6.2',"%getlabel(SEPSVOL1TO3,Voluntary Separations by LOS - 1+ to 3 Years)",'SepsVol1to3  / SepsVol',1,'',5,'')
,(335,'SepsVol3to5','4','percent6.2',"%getlabel(SEPSVOL3TO5,Voluntary Separations by LOS - 3+ to 5 Years)",'SepsVol3to5  / SepsVol',1,'',5,'')
,(336,'SepsVol5to10','4','percent6.2',"%getlabel(SEPSVOL5TO10,Voluntary Separations by LOS - 5+ to 10 Years)",'SepsVol5to10  / SepsVol',1,'',5,'')
,(337,'SepsVol10plus','4','percent6.2',"%getlabel(SEPSVOL10PLUS,Voluntary Separations by LOS - 10+ Years)",'SepsVol10plus  / SepsVol',1,'',5,'')
,(338,'SepsVol0to1Ex','4','percent6.2',"%getlabel(SEPSVOL0TO1EX,Voluntary Separations by LOS - 0 to 1 Year- Exempt)",'SepsVolEx0to1  / SepsVolEx',1,'',5,'')
,(339,'SepsVol1to3Ex','4','percent6.2',"%getlabel(SEPSVOL1TO3EX,Voluntary Separations by LOS - 1+ to 3 Years - Exempt)",'SepsVolEx1to3  / SepsVolEx',1,'',5,'')
,(340,'SepsVol3to5Ex','4','percent6.2',"%getlabel(SEPSVOL3TO5EX,Voluntary Separations by LOS - 3+ to 5 Years - Exempt)",'SepsVolEx3to5  / SepsVolEx',1,'',5,'')
,(341,'SepsVol5to10Ex','4','percent6.2',"%getlabel(SEPSVOL5TO10EX,Voluntary Separations by LOS - 5+ to 10 Years - Exempt)",'SepsVolEx5to10  / SepsVolEx',1,'',5,'')
,(342,'SepsVol10plusEx','4','percent6.2',"%getlabel(SEPSVOL10PLUSEX,Voluntary Separations by LOS - 10+ Years - Exempt)",'SepsVolEx10plus  / SepsVolEx',1,'',5,'')
,(343,'SepsVol0to1Nonex','4','percent6.2',"%getlabel(SEPSVOL0TO1NONEX,Voluntary Separations by LOS - 0 to 1 Year- Nonexempt)",'SepsVolNonex0to1  / SepsVolNonex',1,'',5,'')
,(344,'SepsVol1to3Nonex','4','percent6.2',"%getlabel(SEPSVOL1TO3NONEX,Voluntary Separations by LOS - 1+ to 3 Years - Nonexempt)",'SepsVolNonex1to3  / SepsVolNonex',1,'',5,'')
,(345,'SepsVol3to5Nonex','4','percent6.2',"%getlabel(SEPSVOL3TO5NONEX,Voluntary Separations by LOS - 3+ to 5 Years - Nonexempt)",'SepsVolNonex3to5  / SepsVolNonex',1,'',5,'')
,(346,'SepsVol5to10Nonex','4','percent6.2',"%getlabel(SEPSVOL5TO10NONEX,Voluntary Separations by LOS - 5+ to 10 Years - Nonexempt)",'SepsVolNonex5to10  / SepsVolNonex',1,'',5,'')
,(347,'SepsVol10plusNonex','4','percent6.2',"%getlabel(SEPSVOL10PLUSNONEX,Voluntary Separations by LOS - 10+ Years - Nonexempt)",'SepsVolNonex10plus  / SepsVolNonex',1,'',5,'')
,(348,'SepsRateMgmt','4','percent6.2',"%getlabel(SEPSRATEMGMT,Separation Rate - Management)",'SepsMgmt / HCMgmt',1,'',5,'')
,(349,'SepsRateProf','4','percent6.2',"%getlabel(SEPSRATEPROF,Separation Rate - Professionals)",'SepsProf / HCProf',1,'',5,'')
,(350,'SepsRateSales','4','percent6.2',"%getlabel(SEPSRATESALES,Separation Rate - Sales)",'SepsSales / HCSales',1,'',5,'')
,(351,'SepsRateOandC','4','percent6.2',"%getlabel(SEPSRATEOANDC,Separation Rate - Office & Clerical)",'SepsOandC / HCOandC',1,'',5,'')
,(352,'SepsRateOp','4','percent6.2',"%getlabel(SEPSRATEOP,Separation Rate - Operatives)",'SepsOp / HCOp',1,'',5,'')
,(353,'SepsVolRateMgmt','4','percent6.2',"%getlabel(SEPSVOLRATEMGMT,Voluntary Separation Rate - Management)",'SepsVolMgmt / HCMgmt',1,'',5,'')
,(354,'SepsVolRateProf','4','percent6.2',"%getlabel(SEPSVOLRATEPROF,Voluntary Separation Rate - Professionals)",'SepsVolProf / HCProf',1,'',5,'')
,(355,'SepsVolRateSales','4','percent6.2',"%getlabel(SEPSVOLRATESALES,Voluntary Separation Rate - Sales)",'SepsVolSales / HCSales',1,'',5,'')
,(356,'SepsVolRateOandC','4','percent6.2',"%getlabel(SEPSVOLRATEOANDC,Voluntary Separation Rate - Office & Clerical)",'SepsVolOandC / HCOandC',1,'',5,'')
,(357,'SepsVolRateOp','4','percent6.2',"%getlabel(SEPSVOLRATEOP,Voluntary Separation Rate - Operatives)",'SepsVolOp / HCOp',1,'',5,'')
,(358,'SepsInvolRateMgmt','4','percent6.2',"%getlabel(SEPSINVOLRATEMGMT,Involuntary Separation Rate - Management)",'SepsInvolMgmt / HCMgmt',1,'',5,'')
,(359,'SepsInvolRateProf','4','percent6.2',"%getlabel(SEPSINVOLRATEPROF,Involuntary Separation Rate - Professionals)",'SepsInvolProf / HCProf',1,'',5,'')
,(360,'SepsInvolRateSales','4','percent6.2',"%getlabel(SEPSINVOLRATESALES,Involuntary Separation Rate - Sales)",'SepsInvolSales / HCSales',1,'',5,'')
,(361,'SepsInvolRateOandC','4','percent6.2',"%getlabel(SEPSINVOLRATEOANDC,Involuntary Separation Rate - Office & Clerical)",'SepsInvolOandC / HCOandC',1,'',5,'')
,(362,'SepsInvolRateOp','4','percent6.2',"%getlabel(SEPSINVOLRATEOP,Involuntary Separation Rate - Operatives)",'SepsInvolOp / HCOp',1,'',5,'')
,(363,'SepsVol0to1Mgmt','4','percent6.2',"%getlabel(SEPSVOL0TO1MGMT,Voluntary Separations by LOS - 0 to 1 Year - Management)",'SepsVolMgmt0to1 / SepsVolMgmt',1,'',5,'')
,(364,'SepsVol1to3Mgmt','4','percent6.2',"%getlabel(SEPSVOL1TO3MGMT,Voluntary Separations by LOS - 1+ to 3 Years - Management)",'SepsVolMgmt1to3 / SepsVolMgmt',1,'',5,'')
,(365,'SepsVol3to5Mgmt','4','percent6.2',"%getlabel(SEPSVOL3TO5MGMT,Voluntary Separations by LOS - 3+ to 5 Years - Management)",'SepsVolMgmt3to5 / SepsVolMgmt',1,'',5,'')
,(366,'SepsVol5to10Mgmt','4','percent6.2',"%getlabel(SEPSVOL5TO10MGMT,Voluntary Separations by LOS - 5+ to 10 Years - Management)",'SepsVolMgmt5to10 / SepsVolMgmt',1,'',5,'')
,(367,'SepsVol10plusMgmt','4','percent6.2',"%getlabel(SEPSVOL10PLUSMGMT,Voluntary Separations by LOS - 10+ Years - Management)",'SepsVolMgmt10plus / SepsVolMgmt',1,'',5,'')
,(368,'SepsVol0to1Prof','4','percent6.2',"%getlabel(SEPSVOL0TO1PROF,Voluntary Separations by LOS - 0 to 1 Year - Professionals)",'SepsVolProf0to1 / SepsVolProf',1,'',5,'')
,(369,'SepsVol1to3Prof','4','percent6.2',"%getlabel(SEPSVOL1TO3PROF,Voluntary Separations by LOS - 1+ to 3 Years - Professionals)",'SepsVolProf1to3 / SepsVolProf',1,'',5,'')
,(370,'SepsVol3to5Prof','4','percent6.2',"%getlabel(SEPSVOL3TO5PROF,Voluntary Separations by LOS - 3+ to 5 Years - Professionals)",'SepsVolProf3to5 / SepsVolProf',1,'',5,'')
,(371,'SepsVol5to10Prof','4','percent6.2',"%getlabel(SEPSVOL5TO10PROF,Voluntary Separations by LOS - 5+ to 10 Years - Professionals)",'SepsVolProf5to10 / SepsVolProf',1,'',5,'')
,(372,'SepsVol10plusProf','4','percent6.2',"%getlabel(SEPSVOL10PLUSPROF,Voluntary Separations by LOS - 10+ Years - Professionals)",'SepsVolProf10plus / SepsVolProf',1,'',5,'')
,(373,'SepsVol0to1Sales','4','percent6.2',"%getlabel(SEPSVOL0TO1SALES,Voluntary Separations by LOS - 0 to 1 Year - Sales)",'SepsVolSales0to1 / SepsVolSales',1,'',5,'')
,(374,'SepsVol1to3Sales','4','percent6.2',"%getlabel(SEPSVOL1TO3SALES,Voluntary Separations by LOS - 1+ to 3 Years - Sales)",'SepsVolSales1to3 / SepsVolSales',1,'',5,'')
,(375,'SepsVol3to5Sales','4','percent6.2',"%getlabel(SEPSVOL3TO5SALES,Voluntary Separations by LOS - 3+ to 5 Years - Sales)",'SepsVolSales3to5 / SepsVolSales',1,'',5,'')
,(376,'SepsVol5to10Sales','4','percent6.2',"%getlabel(SEPSVOL5TO10SALES,Voluntary Separations by LOS - 5+ to 10 Years - Sales)",'SepsVolSales1to10 / SepsVolSales',1,'',5,'')
,(377,'SepsVol10plusSales','4','percent6.2',"%getlabel(SEPSVOL10PLUSSALES,Voluntary Separations by LOS - 10+ Years - Sales)",'SepsVolSales10plus / SepsVolSales',1,'',5,'')
,(378,'SepsVol0to1OandC','4','percent6.2',"%getlabel(SEPSVOL0TO1OANDC,Voluntary Separations by LOS - 0 to 1 Year - Office & Clerical)",'SepsVolOandC0to1 / SepsVolOandC',1,'',5,'')
,(379,'SepsVol1to3OandC','4','percent6.2',"%getlabel(SEPSVOL1TO3OANDC,Voluntary Separations by LOS - 1+ to 3 Years - Office & Clerical)",'SepsVolOandC1to3 / SepsVolOandC',1,'',5,'')
,(380,'SepsVol3to5OandC','4','percent6.2',"%getlabel(SEPSVOL3TO5OANDC,Voluntary Separations by LOS - 3+ to 5 Years - Office & Clerical)",'SepsVolOandC3to5 / SepsVolOandC',1,'',5,'')
,(381,'SepsVol5to10OandC','4','percent6.2',"%getlabel(SEPSVOL5TO10OANDC,Voluntary Separations by LOS - 5+ to 10 Years - Office & Clerical)",'SepsVolOandC5to10 / SepsVolOandC',1,'',5,'')
,(382,'SepsVol10plusOandC','4','percent6.2',"%getlabel(SEPSVOL10PLUSOANDC,Voluntary Separations by LOS - 10+ Years - Office & Clerical)",'SepsVolOandC10plus / SepsVolOandC',1,'',5,'')
,(383,'SepsVol0to1Op','4','percent6.2',"%getlabel(SEPSVOL0TO1OP,Voluntary Separations by LOS - 0 to 1 Year - Operatives)",'SepsVolOp0to1 / SepsVolOp',1,'',5,'')
,(384,'SepsVol1to3Op','4','percent6.2',"%getlabel(SEPSVOL1TO3OP,Voluntary Separations by LOS - 1+ to 3 Years - Operatives)",'SepsVolOp1to3 / SepsVolOp',1,'',5,'')
,(385,'SepsVol3to5Op','4','percent6.2',"%getlabel(SEPSVOL3TO5OP,Voluntary Separations by LOS - 3+ to 5 Years - Operatives)",'SepsVolOp3to5 / SepsVolOp',1,'',5,'')
,(386,'SepsVol5to10Op','4','percent6.2',"%getlabel(SEPSVOL5TO10OP,Voluntary Separations by LOS - 5+ to 10 Years - Operatives)",'SepsVolOp5to10 / SepsVolOp',1,'',5,'')
,(387,'SepsVol10plusOp','4','percent6.2',"%getlabel(SEPSVOL10PLUSOP,Voluntary Separations by LOS - 10+ Years - Operatives)",'SepsVolOp10plus / SepsVolOp',1,'',5,'')
,(388,'EETrainPct','4','percent6.2',"%getlabel(EETRAINPCT,Employees Trained Percent)",'WFTrain / HC',1,'',7,'')
,(389,'EETrainPctReg','4','percent6.2',"%getlabel(EETRAINPCTREG,Employees Trained Percent - Regular Employees)",'WFTrain / HCReg',1,'',7,'')
,(390,'TrainCostFctr','4','dollar12.',"%getlabel(TRAINCOSTFCTR,Training Cost Factor - Total)",'WFTrainCost / WFTrain',1,'',7,'')
,(391,'TrainCostFctrETPB','4','dollar12.',"%getlabel(TRAINCOSTFCTRETPB,Training Cost Factor - Excluding Trainee Pay & Benefits)",'WFTrainCostETPB / WFTrain',1,'',7,'')
,(392,'TrainCostPct','4','percent6.2',"%getlabel(TRAINCOSTPCT,Training Cost Percent - Total)",'WFTrainCost / OpExp',1,'',7,'')
,(393,'TrainCostPctETPB','4','percent6.2',"%getlabel(TRAINCOSTPCTETPB,Training Cost Percent - Excluding Trainee Pay & Benefits - Total)",'WFTrainCostETPB / OpExp',1,'',7,'')
,(394,'TrainCostPctETPBExt','4','percent6.2',"%getlabel(TRAINCOSTPCTETPBEXT,Training Cost Percent - Excluding Trainee Pay & Benefits - External)",'TrainCostETPBExt / WFTrainCostETPB',1,'',7,'')
,(395,'TrainCostPctETPBInt','4','percent6.2',"%getlabel(TRAINCOSTPCTETPBINT,Training Cost Percent - Excluding Trainee Pay & Benefits - Internal)",'TrainCostETPBInt / WFTrainCostETPB',1,'',7,'')
,(396,'TrainCostHRExpPct','4','percent6.2',"%getlabel(TRAINCOSTHREXPPCT,Training Cost HR Expense Percent - Total)",'WFTrainCost / ExpenseHR',1,'',7,'')
,(397,'TrainCostHRExpPctETPB','4','percent6.2',"%getlabel(TRAINCOSTHREXPPCTETPB,Training Cost HR Expense Percent - Excluding Trainee Pay & Benefits)",'WFTrainCostETPB / ExpenseHR',1,'',7,'')
,(398,'TrainCompPct','4','percent6.2',"%getlabel(TRAINCOMPPCT,Training Compensation Percent - Total)",'WFTrainCost / CompWFOnP',1,'',7,'')
,(399,'TrainCompPctReg','4','percent6.2',"%getlabel(TRAINCOMPPCTREG,Training Compensation Percent - Regular Employees)",'WFTrainCost / CompReg',1,'',7,'')
,(400,'TrainCompPctETPB','4','percent6.2',"%getlabel(TRAINCOMPPCTETPB,Training Compensation Percent - Excluding Trainee Pay & Benefits)",'WFTrainCostETPB / CompWFOnP',1,'',7,'')
,(401,'TrainCompPctETPBReg','4','percent6.2',"%getlabel(TRAINCOMPPCTETPBREG,Training Compensation Percent - Regular Employees - Excluding Trainee Pay & Benefits)",'WFTrainCostETPB / CompReg',1,'',7,'')
,(402,'TrainHCInvFctr','4','dollar12.',"%getlabel(TRAINHCINVFCTR,Training Headcount Investment Factor - Total)",'WFTrainCost / HC',1,'',7,'')
,(403,'TrainHCInvFctrReg','4','dollar12.',"%getlabel(TRAINHCINVFCTRREG,Training Headcount Investment Factor - Regular Employees)",'WFTrainCost / HCReg',1,'',7,'')
,(404,'TrainHCInvFctrETPB','4','dollar12.',"%getlabel(TRAINHCINVFCTRETPB,Training Headcount Investment Factor - Excluding Trainee Pay & Benefits)",'WFTrainCostETPB / HC',1,'',7,'')
,(405,'TrainHCInvFctrETPBReg','4','dollar12.',"%getlabel(TRAINHCINVFCTRETPBREG,Training Headcount Investment Factor - Regular Employees - Excluding Trainee Pay & Ben)",'WFTrainCostETPB / HCReg',1,'',7,'')
,(406,'HCTrainFctr','4','comma10.2',"%getlabel(HCTRAINFCTR,Headcount Training Factor)",'TrainHrs / HC',1,'',7,'')
,(407,'HCTrainFctrReg','4','comma10.2',"%getlabel(HCTRAINFCTRREG,Headcount Training Factor - Regular Employees)",'TrainHrs / HCReg',1,'',7,'')
,(408,'FTETrainFctr','4','comma10.2',"%getlabel(FTETRAINFCTR,FTE Training Factor)",'TrainHrs / FTE',1,'',7,'')
,(409,'FTETrainFctrReg','4','comma10.2',"%getlabel(FTETRAINFCTRREG,FTE Training Factor - Regular Employees)",'TrainHrsReg / FTEReg',1,'',7,'')
,(410,'TrainFTEInvFctr','4','dollar12.',"%getlabel(TRAINFTEINVFCTR,Training FTE Investment Factor - Total)",'WFTrainCost / FTE',1,'',7,'')
,(411,'TrainFTEInvFctrEx','4','dollar12.',"%getlabel(TRAINFTEINVFCTREX,Training FTE Investment Factor - Exempt Total)",'WFTrainCostEx / FTEEx',1,'',7,'')
,(412,'TrainFTEInvFctrNonex','4','dollar12.',"%getlabel(TRAINFTEINVFCTRNONEX,Training FTE Investment Factor - Nonexempt Total)",'WFTrainCostNonex / FTENonex',1,'',7,'')
,(413,'TrainFTEInvFctrReg','4','dollar12.',"%getlabel(TRAINFTEINVFCTRREG,Training FTE Investment Factor - Regular)",'WFTrainCost / FTEReg',1,'',7,'')
,(414,'TrainFTEInvFctrETPB','4','dollar12.',"%getlabel(TRAINFTEINVFCTRETPB,Training FTE Investment Factor - Excluding Trainee Pay & Benefits)",'WFTrainCostETPB / FTE',1,'',7,'')
,(415,'TrainFTEInvFctrETPBEx','4','dollar12.',"%getlabel(TRAINFTEINVFCTRETPBEX,Training FTE Investment Factor - Excluding Trainee Pay & Benefits - Exempt)",'TrainCostETPBEx / FTEEx',1,'',7,'')
,(416,'TrainFTEInvFctrETPBNonex','4','dollar12.',"%getlabel(TRAINFTEINVFCTRETPBNONEX,Training FTE Investment Factor - Excluding Trainee Pay & Benefits - Nonexempt)",'TrainCostETPBNonex / FTENonex',1,'',7,'')
,(417,'TrainFTEInvFctrETPBReg','4','dollar12.',"%getlabel(TRAINFTEINVFCTRETPBREG,Training FTE Investment Factor - Regular Employees - Excluding Trainee Pay & Benefits)",'TrainCostETPB / FTEReg',1,'',7,'')
,(418,'TrainStaffRatio','4','comma6.2',"%getlabel(TRAINSTAFFRATIO,Training Staff Ratio)",'FTE / TrainStaffFTE',1,'',7,'')
,(419,'TrainStaffRatioReg','4','comma6.2',"%getlabel(TRAINSTAFFRATIOREG,Training Staff Ratio - Regular Employees)",'FTEReg / TrainStaffFTE',1,'',7,'')
,(420,'TrainCPH','4','dollar12.',"%getlabel(TRAINCPH,Training Cost per Hour - Total)",'WFTrainCost / TrainHrs',1,'',7,'')
,(421,'TrainCPHETPB','4','dollar12.',"%getlabel(TRAINCPHETPB,Training Cost per Hour - Excluding Trainee Pay & Ben)",'WFTrainCostETPB / TrainHrs',1,'',7,'')
,(422,'TrainHrsPctInt','4','percent6.2',"%getlabel(TRAINHRSPCTINT,Training Hours Percent - Internal)",'StaffTrainHrsInt / TrainHrs',1,'',7,'')
,(423,'TrainHrsPctExt','4','percent6.2',"%getlabel(TRAINHRSPCTEXT,Training Hours Percent - External)",'StaffTrainHrsExt / TrainHrs',1,'',7,'')
 ) by PASSTHRU;
/*************************/
/*   SAS_MODULEACTIONS   */
/*************************/
 execute (Insert into SAS_MODULEACTIONS (ACTION_ID,MODULE_ID,ACTION_NM,ACTION_DESC,SECURITY_CD,ACTION_IMG) Values
('ACApplyChangeCategory',11,"%getlabel(MODACTSNAME1,ApplyChangesCategory)","%getlabel(MODACTSDESC1,Fixed Category Apply Changes)",1024,'SaveFile.gif')
,('ACApplyChangeCategoryDefails',11,"%getlabel(MODACTSNAME2,ApplyChangesCategoryDetails)","%getlabel(MODACTSDESC2,Customize Employee Profile Fixed Category Apply Changes)",2048,'SaveFile.gif')
,('ACApplyChangeEEP',11,"%getlabel(MODACTSNAME3,ApplyChangesEEP)","%getlabel(MODACTSDESC3,Employee Profile Apply Changes)",32,'SaveFile.gif')
,('ACApplyChangeEditEEP',11,"%getlabel(MODACTSNAME4,ApplyChangesEditEEP)","%getlabel(MODACTSDESC4,Customize Employee Profile Details Apply Changes)",64,'SaveFile.gif')
,('ACApplyChangeHeaderEEP',11,"%getlabel(MODACTSNAME5,ApplyChangesHeaderEEP)","%getlabel(MODACTSDESC5,Customize Employee Profile Header Apply Changes)",256,'SaveFile.gif')
,('ACApplyChangeListEEP',11,"%getlabel(MODACTSNAME6,ApplyChangesListEEP)","%getlabel(MODACTSDESC6,Customize Employee Profile List Apply Changes)",128,'SaveFile.gif')
,('ACApplyChangeSearch',11,"%getlabel(MODACTSNAME7,ApplyChangesSearch)","%getlabel(MODACTSDESC7,Customize Search Category Apply Changes)",65536,'SaveFile.gif')
,('ACApplyChangeSearchEEP',11,"%getlabel(MODACTSNAME8,ApplyChangesSearchEEP)","%getlabel(MODACTSDESC8,Customize Employee Profile Search Apply Changes)",512,'SaveFile.gif')
,('ACCopyEEP',11,"%getlabel(MODACTSNAME9,CopyEEP)","%getlabel(MODACTSDESC9,Copy Profile)",8,'CopyNew.gif')
,('ACDeleteEEP',11,"%getlabel(MODACTSNAME10,DeleteEEP)","%getlabel(MODACTSDESC10,Delete Profile)",16,'DeleteBlack.gif')
,('ACDeleteTemplate',11,"%getlabel(MODACTSNAME11,Delete template)","%getlabel(MODACTSDESC11,Customize Delete Template)",16384,'DeleteBlack.gif')
,('ACEEPMenu',11,"%getlabel(MODACTSNAME12,ProfileContextMenu)","%getlabel(MODACTSDESC12,Profile Context Menu)",4,'')
,('ACEditGEO',11,"%getlabel(MODACTSNAME13,Edit Geo ApplyChanges)","%getlabel(MODACTSDESC13,Customize Geo Analysis Category Apply Changes)",4096,'SaveFile.gif')
,('ACEditORG',11,"%getlabel(MODACTSNAME14,Edit Org ApplyChanges)","%getlabel(MODACTSDESC14,Customize Org Analysis Apply Changes)",8192,'SaveFile.gif')
,('ACEditTemplate',11,"%getlabel(MODACTSNAME15,Edit template)","%getlabel(MODACTSDESC15,Customize Template Apply Changes)",32768,'SaveFile.gif')
,('ACGDiagnosis',13,"%getlabel(MODACTSNAME16,Run Diagnostics)","%getlabel(MODACTSDESC16,Run Diagnostics)",1,'runDiagnosis.gif')
,('ACGProp',13,"%getlabel(MODACTSNAME17,Apply Changes Configuration)","%getlabel(MODACTSDESC17,Apply Changes Configuration)",1,'SaveFile.gif')
,('ACGRefreshCache',13,"%getlabel(MODACTSNAME18,Refresh Cache)","%getlabel(MODACTSDESC18,Refresh Cache)",1,'CacheRefresh.gif')
,('ACINewCubeMenu',10,"%getlabel(MODACTSNAME19,New Cube)","%getlabel(MODACTSDESC19,New Cube)",1,'CubeNew.gif')
,('ACIRebuildIMAPMenu',10,"%getlabel(MODACTSNAME20,Rebuild Information Map)","%getlabel(MODACTSDESC20,Rebuild Information Map)",2,'InformationMapRebuild.gif')
,('ACIRefreshCubeMenu',10,"%getlabel(MODACTSNAME21,Refresh Cube)","%getlabel(MODACTSDESC21,Refresh Cube)",4,'CubeRefresh.gif')
,('ACNewEEP',11,"%getlabel(MODACTSNAME22,NewEmployeeProfile)","%getlabel(MODACTSDESC22,New Employee Profile)",1,'NewEmployeeProfile.gif')
,('ACNewTemplate',11,"%getlabel(MODACTSNAME23,New Template)","%getlabel(MODACTSDESC23,New Template)",1,'')
,('ACRefreshCache',11,"%getlabel(MODACTSNAME24,RefreshCacheCustomize)","%getlabel(MODACTSDESC24,Refresh Customize Cache)",2,'CacheRefresh.gif')
,('ADApplyChangeCol',9,"%getlabel(MODACTSNAME25,Table Columns Apply Changes)","%getlabel(MODACTSDESC25,Table Columns Apply Changes)",256,'SaveFile.gif')
,('ADApplyChangeTable',9,"%getlabel(MODACTSNAME27,Table Apply Changes)","%getlabel(MODACTSDESC27,Table Apply Changes)",1,'SaveFile.gif')
,('ADApplyPropChangeCol',9,"%getlabel(MODACTSNAME28,Table Columns Apply and Propagate Changes)","%getlabel(MODACTSDESC28,Table Columns Apply and Propagate Changes)",512,'ApplyAndPropogate.gif')
,('ADBuildCubeIMAP',10,"%getlabel(MODACTSNAME29,AdminDataTabCubeContextMenu Build CubeIMAP)","%getlabel(MODACTSDESC29,AdminDataTabCubeContextMenu Build CubeIMAP)",64,'InformationMapRebuild.gif')
,('ADBuildIMAP',9,"%getlabel(MODACTSNAME30,AdminDataContextMenu BuildInfoMap)","%getlabel(MODACTSDESC30,AdminDataContextMenu BuildInfoMap)",16,'InformationMapRebuild.gif')
,('ADCAddTable',8,"%getlabel(MODACTSNAME31,Add Table)","%getlabel(MODACTSDESC31,Add Table)",1,'AddTable.gif')
,('ADCFormatCatalog',8,"%getlabel(MODACTSNAME32,Build Formats Catalog)","%getlabel(MODACTSDESC32,Build Formats Catalog)",1024,'FormatNew.gif')
,('ADCLog',8,"%getlabel(MODACTSNAME33,SAS Log)","%getlabel(MODACTSDESC33,SAS Log)",16,'SasLog.gif')
,('ADCMapHierarchy',8,"%getlabel(MODACTSNAME34,Map Hierarchies)","%getlabel(MODACTSDESC34,Map Hierarchies)",2,'MapHierarchies.gif')
,('ADCNewFormat',8,"%getlabel(MODACTSNAME35,New Format)","%getlabel(MODACTSDESC35,New Format)",4,'FormatNew.gif')
,('ADCNewMetric',8,"%getlabel(MODACTSNAME36,New Metric)","%getlabel(MODACTSDESC36,New Metric)",8,'HRMetrics.gif')
,('ADCRefreshCache',8,"%getlabel(MODACTSNAME37,Refresh Data Cache)","%getlabel(MODACTSDESC37,Refresh Data Cache)",32,'CacheRefresh.gif')
,('ADCRunJob',8,"%getlabel(MODACTSNAME38,Run Job)","%getlabel(MODACTSDESC38,Run Job)",1024,'runJob.gif')
,('ADColumnMenu',9,"%getlabel(MODACTSNAME39,TableDetails Context Menu)","%getlabel(MODACTSDESC39,TableDetails Context Menu)",1024,'')
,('ADCopyTable',9,"%getlabel(MODACTSNAME40,AdminDataContextMenu CopyTable)","%getlabel(MODACTSDESC40,AdminDataContextMenu CopyTable)",4,'CopyNew.gif')
,('ADDeleteColumn',9,"%getlabel(MODACTSNAME41,Delete Column)","%getlabel(MODACTSDESC41,Delete Column)",2048,'DeleteBlack.gif')
,('ADDeleteCube',10,"%getlabel(MODACTSNAME42,AdminDataTabCubeContextMenu Delete Cube)","%getlabel(MODACTSDESC42,AdminDataTabCubeContextMenu Delete Cube)",128,'DeleteBlack.gif')
,('ADDeleteFormat',9,"%getlabel(MODACTSNAME43,Delete Formats)","%getlabel(MODACTSDESC43,Delete Formats)",131072,'DeleteBlack.gif')
,('ADDeleteHierarchy',9,"%getlabel(MODACTSNAME44,DeleteHierarchy)","%getlabel(MODACTSDESC44,Delete Hierarchy)",16384,'DeleteBlack.gif')
,('ADDeleteImapMenu',10,"%getlabel(MODACTSNAME45,IMAP Menu - Delete)","%getlabel(MODACTSDESC45,IMAP Menu - Delete)",1024,'DeleteBlack.gif')
,('ADDeleteMeasureMenu',8,"%getlabel(MODACTSNAME46,Measures Menu - Delete)","%getlabel(MODACTSDESC46,Measures Menu - Delete)",1024,'DeleteBlack.gif')
,('ADDeleteTable',9,"%getlabel(MODACTSNAME47,AdminDataContextMenu DeleteTable)","%getlabel(MODACTSDESC47,AdminDataContextMenu DeleteTable)",64,'DeleteBlack.gif')
,('ADEditFormat',9,"%getlabel(MODACTSNAME48,Format Properties)","%getlabel(MODACTSDESC48,Format Properties)",262144,'Properties.gif')
,('ADEditHierarchy',9,"%getlabel(MODACTSNAME49,Admin Data Map Hierarchies Details)","%getlabel(MODACTSDESC49,Admin Data Map Hierarchies Details)",32768,'SaveFile.gif')
,('ADEditMeasureMenu',8,"%getlabel(MODACTSNAME50,Measures Menu - Edit)","%getlabel(MODACTSDESC50,Measures Menu - Edit)",1024,'Edit.gif')
,('ADExportToExcelTable',9,"%getlabel(MODACTSNAME51,AdminDataContextMenu ExportToExcel)","%getlabel(MODACTSDESC51,AdminDataContextMenu ExportToExcel)",32,'ExportToExcel.gif')
,('ADFormatMenu',9,"%getlabel(MODACTSNAME52,Formats Context Menu)","%getlabel(MODACTSDESC52,Formats Context Menu)",65536,'')
,('ADGenerateSymbolColumn',9,"%getlabel(MODACTSNAME53,Generate Symbol)","%getlabel(MODACTSDESC53,Generate Symbol)",4096,'ExportToExcel.gif')
,('ADGenerateSymbolTable',9,"%getlabel(MODACTSNAME54,AdminDataContextMenu GenerateSymbol)","%getlabel(MODACTSDESC54,AdminDataContextMenu GenerateSymbol)",128,'ExportToExcel.gif')
,('ADRebuildCube',10,"%getlabel(MODACTSNAME55,AdminDataTabCubeContextMenu Rebuild Cube)","%getlabel(MODACTSDESC55,AdminDataTabCubeContextMenu Rebuild Cube)",16,'CubeRefresh.gif')
,('ADRebuildImapMenu',10,"%getlabel(MODACTSNAME56,IMAP Menu - Rebuild)","%getlabel(MODACTSDESC56,IMAP Menu - Rebuild)",1024,'InformationMapRebuild.gif')
,('ADTableMenu',9,"%getlabel(MODACTSNAME57,TableContextMenu)","%getlabel(MODACTSDESC57,TableContextMenu)",2,'')
,('ADViewCube',10,"%getlabel(MODACTSNAME58,AdminDataTabCubeContextMenu ViewCube)","%getlabel(MODACTSDESC58,AdminDataTabCubeContextMenu ViewTable)",8,'CubeRefresh.gif')
,('ADViewImapMenu',10,"%getlabel(MODACTSNAME59,IMAP Menu - View)","%getlabel(MODACTSDESC59,IMAP Menu - View)",1024,'InformationMapRebuild.gif')
,('ADViewTable',9,"%getlabel(MODACTSNAME60,AdminDataContextMenu ViewTable)","%getlabel(MODACTSDESC60,AdminDataContextMenu ViewTable)",8,'NormalTable.gif')
,('AddDimensionButtonBar',10,"%getlabel(MODACTSNAME61,Add Cube Dimension ButtonBar)","%getlabel(MODACTSDESC61,Add Cube Dimension ButtonBar)",16,'InformationMapRebuild.gif')
,('AdminSecurityAddFilterRLS',12,"%getlabel(MODACTSNAME62,Add Filter)","%getlabel(MODACTSDESC62,Add Filter)",64,'AddFilter.gif')
,('AdminSecurityApplyChangeObject',12,"%getlabel(MODACTSNAME63,Edit Object Permissions)","%getlabel(MODACTSDESC63,Edit Object Permissions)",8,'SaveFile.gif')
,('AdminSecurityApplyChangesCLS',12,"%getlabel(MODACTSNAME64,Apply Changes to Column Permissions)","%getlabel(MODACTSDESC64,Apply Changes to Column Permissions)",4096,'SaveFile.gif')
,('AdminSecurityApplyChangesCustomObj',12,"%getlabel(MODACTSNAME65,Apply Changes to Custom Objects)","%getlabel(MODACTSDESC65,Apply Changes to Custom Objects)",262144,'SaveFile.gif')
,('AdminSecurityApplyChangesHierarchies',12,"%getlabel(MODACTSNAME66,Edit Hierarchy)","%getlabel(MODACTSDESC66,Edit Hierarchy)",2048,'SaveFile.gif')
,('AdminSecurityApplyChangesRLSExtension',12,"%getlabel(MODACTSNAME67,Apply Changes to Enable Row Level Security)","%getlabel(MODACTSDESC67,Apply Changes to Enable Row Level Security)",1,'SaveFile.gif')
,('AdminSecurityApplyPropChangeCLS',12,"%getlabel(MODACTSNAME68,Column Permissions Apply and Propagate Changes)","%getlabel(MODACTSDESC68,Column Permissions Apply and Propagate Changes)",8192,'ApplyAndPropogate.gif')
,('AdminSecurityContextMenuCLS',12,"%getlabel(MODACTSNAME69,Column Permission Context Menu)","%getlabel(MODACTSDESC69,Column Permission Context Menu)",32768,'')
,('AdminSecurityContextMenuFilter',12,"%getlabel(MODACTSNAME70,Table Filters Context Menu)","%getlabel(MODACTSDESC70,Table Filters Context Menu)",1024,'')
,('AdminSecurityContextMenuObjectPermission',12,"%getlabel(MODACTSNAME71,Object Permissions Context Menu)","%getlabel(MODACTSDESC71,Object Permissions Context Menu)",4,'')
,('AdminSecurityContextMenuSearchUserFilters',12,"%getlabel(MODACTSNAME72,Search User Filters Context Menu)","%getlabel(MODACTSDESC72,Search User Filters Context Menu)",2097152,'')
,('AdminSecurityContextMenuUserFilters',12,"%getlabel(MODACTSNAME73,Show User Filters Context Menu)","%getlabel(MODACTSDESC73,Show User Filters Context Menu)",1048576,'')
,('AdminSecurityDeleteCustomObject',12,"%getlabel(MODACTSNAME74,Delete Custom Objects)","%getlabel(MODACTSDESC74,Delete Custom Objects)",524288,'DeleteBlack.gif')
,('AdminSecurityDeleteRLS',12,"%getlabel(MODACTSNAME75,Delete Filter)","%getlabel(MODACTSDESC75,Delete Filter)",256,'Delete.gif')
,('AdminSecurityImportUsers',12,"%getlabel(MODACTSNAME76,Import Users)","%getlabel(MODACTSDESC76,Import Users)",1024,'empImg.gif')
,('AdminSecurityPropertiesRLS',12,"%getlabel(MODACTSNAME77,View Filter)","%getlabel(MODACTSDESC77,View Filter)",512,'MemberProperties.gif')
,('AdminSecurityRefreshCache',12,"%getlabel(MODACTSNAME78,Refresh Security Cache)","%getlabel(MODACTSDESC78,Refresh Security Cache)",131072,'CacheRefresh.gif')
,('AdminSecurityRemoveIdentityCLS',12,"%getlabel(MODACTSNAME79,Remove Column Permissions)","%getlabel(MODACTSDESC79,Remove Column Permissions)",16384,'DeleteBlack.gif')
,('AdminSecurityRemovePermissionObject',12,"%getlabel(MODACTSNAME80,Remove Object Permissions)","%getlabel(MODACTSDESC80,Remove Object Permissions)",1,'DeleteBlack.gif')
,('AdminSecuritySearchUserCLS',12,"%getlabel(MODACTSNAME81,Column Permissions Search Users)","%getlabel(MODACTSDESC81,Column Permissions Search Users)",65536,'Search.gif')
,('AdminSecuritySearchUserObject',12,"%getlabel(MODACTSNAME82,Object Permissions Search Users)","%getlabel(MODACTSDESC82,Object Permissions Search Users)",1,'Search.gif')
,('AdminSecuritySearchUserRLS',12,"%getlabel(MODACTSNAME83,Filters Search Users)","%getlabel(MODACTSDESC83,Filters Search Users)",32,'Search.gif')
,('AdminSecurityShowFilterRLS',12,"%getlabel(MODACTSNAME84,Show Filters)","%getlabel(MODACTSDESC84,Show Filters)",128,'Filter.gif')
,('AdminSecurityViewPermissionObject',12,"%getlabel(MODACTSNAME85,View Object Permissions)","%getlabel(MODACTSDESC85,View Object Permissions)",2,'AddFilter.gif')
,('AdminSecurityViewReportObject',12,"%getlabel(MODACTSNAME86,View Report)","%getlabel(MODACTSDESC86,View Report)",16,'')
,('CubeRebuildButtonBar',10,"%getlabel(MODACTSNAME87,Rebuild Cube ButtonBar)","%getlabel(MODACTSDESC87,Rebuild Cube ButtonBar)",8,'CubeRefresh.gif')
,('CubeRefresh',10,"%getlabel(MODACTSNAME88,AdminDataTabCubeContextMenu Cube Refresh)","%getlabel(MODACTSDESC88,AdminDataTabCubeContextMenu Cube Refresh)",32,'refresh_list.gif')
,('EEPActions',1,"%getlabel(MODACTSNAME89,External Actions)","%getlabel(MODACTSDESC89,External Actions)",512,'')
,('EEPEMailAll',1,"%getlabel(MODACTSNAME90,EMail All)","%getlabel(MODACTSDESC90,EMail All)",1,'email.gif')
,('EEPFindPeople',1,"%getlabel(MODACTSNAME91,Find People)","%getlabel(MODACTSDESC91,Find People)",256,'')
,('EEPHierarchy',1,"%getlabel(MODACTSNAME92,Hierarchy)","%getlabel(MODACTSDESC92,Hierarchy)",2,'MapHierarchies.gif')
,('EEPOptions',1,"%getlabel(MODACTSNAME94,Options)","%getlabel(MODACTSDESC94,ProfileOptions)",64,'Options.gif')
,('EEPPrintListing',1,"%getlabel(MODACTSNAME95,Print Profile Listing)","%getlabel(MODACTSDESC95,Print Profile Listing)",8,'Print.gif')
,('EEPPrintProfile',1,"%getlabel(MODACTSNAME96,Print Profile)","%getlabel(MODACTSDESC96,Print Profile)",4,'Print.gif')
,('EEPProfileListingSaveAs',1,"%getlabel(MODACTSNAME97,ProfileListing SaveAs)","%getlabel(MODACTSDESC97,ProfileListing SaveAs)",128,'')
,('EEPProfileListingSaveAsExcel',1,"%getlabel(MODACTSNAME98,ProfileListing SaveAs Excel)","%getlabel(MODACTSDESC98,ProfileListing SaveAs Excel)",16,'Export_Org.gif')
,('EEPProfileListingSaveAsPDF',1,"%getlabel(MODACTSNAME99,ProfileListing SaveAs PDF)","%getlabel(MODACTSDESC99,ProfileListing SaveAs PDF)",32,'ExportData.gif')
,('EEPProfileViewSaveAs',1,"%getlabel(MODACTSNAME100,ProfileView SaveAs)","%getlabel(MODACTSDESC100,ProfileView SaveAs)",32768,'')
,('EEPProfileViewSaveAsPDF',1,"%getlabel(MODACTSNAME101,ProfileView SaveAs PDF)","%getlabel(MODACTSDESC101,ProfileView SaveAs PDF)",16384,'ExportData.gif')
,('EEPSelectCategory',1,"%getlabel(MODACTSNAME102,Employee Details Category Selector)","%getlabel(MODACTSDESC102,Employee Details Category Selector)",1024,'')
,('EEPViewEmployeeDetails',1,"%getlabel(MODACTSNAME103,View Employee Details)","%getlabel(MODACTSDESC103,View Employee Details)",0,'EmployeeDetailTemplate.gif')
,('EEPWorkspaceManager',1,"%getlabel(MODACTSNAME104,Open Workspace Manager)","%getlabel(MODACTSDESC104,Open Workspace Manager)",0,'workspace.gif')
,('GEOCopy',2,"%getlabel(MODACTSNAME105,Geographic Copy To)","%getlabel(MODACTSDESC105,Copy To)",1,'Copy_Org.gif')
,('GEODRILLDOWN',2,"%getlabel(MODACTSNAME106,Geographic DrillDown)","%getlabel(MODACTSDESC106,Geographic DrillDown)",1,'MapView_new.gif')
,('GEOExport',2,"%getlabel(MODACTSNAME107,Geographic Export)","%getlabel(MODACTSDESC107,Export)",4,'Export_new.gif')
,('GEOMap',2,"%getlabel(MODACTSNAME108,Map View)","%getlabel(MODACTSDESC108,Map View)",8,'MapView_new.gif')
,('GEOOptions',2,"%getlabel(MODACTSNAME109,Geographic Options)","%getlabel(MODACTSDESC109,Options)",64,'ViewProperties.gif')
,('GEOPrint',2,"%getlabel(MODACTSNAME110,Print)","%getlabel(MODACTSDESC110,Print)",2,'Print.gif')
,('GEOTableView',2,"%getlabel(MODACTSNAME111,Table View)","%getlabel(MODACTSDESC111,Table View)",16,'TableView_new.gif')
,('GEOWorkspace',2,"%getlabel(MODACTSNAME112,Workspace Manager)","%getlabel(MODACTSDESC112,Workspace Manager)",32,'workspace.gif')
,('GSAdvSearch',4,"%getlabel(MODACTSNAME113,Advanced Search)","%getlabel(MODACTSDESC113,Advanced Search)",131072,'')
,('GSBarChart',4,"%getlabel(MODACTSNAME114,Bar Chart)","%getlabel(MODACTSDESC114,Bar Chart)",1,'BarChart.gif')
,('GSBarlineChart',4,"%getlabel(MODACTSNAME115,BarLine Chart)","%getlabel(MODACTSDESC115,BarLine Chart)",2,'BarLineChart.gif')
,('GSEmailAll',4,"%getlabel(MODACTSNAME116,Search Email All)","%getlabel(MODACTSDESC116,Email All)",4,'email.gif')
,('GSExportToExcel',4,"%getlabel(MODACTSNAME117,Export Excel)","%getlabel(MODACTSDESC117,Excel)",8,'Export_Org.gif')
,('GSExportToPDF',4,"%getlabel(MODACTSNAME118,PDF File)","%getlabel(MODACTSDESC118,PDF File)",16,'ExportData.gif')
,('GSGeoMap',4,"%getlabel(MODACTSNAME119,Geo Map)","%getlabel(MODACTSDESC119,Geo Map)",128,'Map.gif')
,('GSHierarchy',4,"%getlabel(MODACTSNAME120,Search Hierarchy)","%getlabel(MODACTSDESC120,Hierarchy)",32,'MapHierarchies.gif')
,('GSHistorySearch',4,"%getlabel(MODACTSNAME121,History Search)","%getlabel(MODACTSDESC121,History Search)",65536,'')
,('GSLineChart',4,"%getlabel(MODACTSNAME122,Line Chart)","%getlabel(MODACTSDESC122,Line Chart)",64,'LineChart.gif')
,('GSOptions',4,"%getlabel(MODACTSNAME123,Search Options)","%getlabel(MODACTSDESC123,Options)",512,'ViewProperties.gif')
,('GSPieChart',4,"%getlabel(MODACTSNAME124,Pie Chart)","%getlabel(MODACTSDESC124,Pie Chart)",1024,'PieChart.gif')
,('GSPrint',4,"%getlabel(MODACTSNAME125,Search Print)","%getlabel(MODACTSDESC125,Print)",2048,'Print.gif')
,('GSSaveAs',4,"%getlabel(MODACTSNAME126,Save As)","%getlabel(MODACTSDESC126,Save As)",4096,'ExportData.gif')
,('GSScatterPlot',4,"%getlabel(MODACTSNAME127,Scatter Plot)","%getlabel(MODACTSDESC127,Scatter Plot)",8192,'ScatterPlot.gif')
,('GSSearch',4,"%getlabel(MODACTSNAME128,Search)","%getlabel(MODACTSDESC128,Search)",32768,'')
,('GSTableViewer',4,"%getlabel(MODACTSNAME129,Table Viewer)","%getlabel(MODACTSDESC129,Table Viewer)",16384,'TableViewer.gif')
,('HHAdmin',6,"%getlabel(MODACTSNAME130,Administration)","%getlabel(MODACTSDESC130,Administration)",64,'')
,('HHAdvSearch',6,"%getlabel(MODACTSNAME131,Advance Search)","%getlabel(MODACTSDESC131,Advance Search)",4,'')
,('HHDashboard',6,"%getlabel(MODACTSNAME132,Manage Dashboard)","%getlabel(MODACTSDESC132,Manage Dashboard)",128,'')
,('HHEmpProfile',6,"%getlabel(MODACTSNAME133,Your Employee Profile)","%getlabel(MODACTSDESC133,Your Employee Profile)",8,'')
,('HHHistorySearch',6,"%getlabel(MODACTSNAME134,History Search)","%getlabel(MODACTSDESC134,History Search)",2,'')
,('HHNewGeo',6,"%getlabel(MODACTSNAME135,New Geographic Analysis)","%getlabel(MODACTSDESC135,New Geographic Analysis)",32,'')
,('HHNewOrg',6,"%getlabel(MODACTSNAME136,New Organization Analysis)","%getlabel(MODACTSDESC136,New Organization Analysis)",16,'')
,('HHOlap',6,"%getlabel(MODACTSNAME137,New OLAP Analysis)","%getlabel(MODACTSDESC137,New OLAP Analysis)",2048,'')
,('HHPortal',6,"%getlabel(MODACTSNAME138,My Portal)","%getlabel(MODACTSDESC138,My Portal)",8192,'')
,('HHReport',6,"%getlabel(MODACTSNAME139,My Report)","%getlabel(MODACTSDESC139,My Report)",4096,'')
,('HHScorecard',6,"%getlabel(MODACTSNAME140,New HR Scorecard)","%getlabel(MODACTSDESC140,New HR Scorecard)",1024,'')
,('HHSearch',6,"%getlabel(MODACTSNAME141,Search)","%getlabel(MODACTSDESC141,Search)",1,'')
,('HHSearchAssist',6,"%getlabel(MODACTSNAME142,Search Assist)","%getlabel(MODACTSDESC142,Search Assist)",16384,'')
,('HHWorkspace',6,"%getlabel(MODACTSNAME143,Workspace)","%getlabel(MODACTSDESC143,Workspace)",512,'')
,('ORGAnalysisView',3,"%getlabel(MODACTSNAME145,Analysis view)","%getlabel(MODACTSDESC145,Analysis view)",2,'Analysis_View_Org.gif')
,('ORGCopy',3,"%getlabel(MODACTSNAME146,Copy To)","%getlabel(MODACTSDESC146,Copy To)",4,'Copy_Org.gif')
,('ORGExport',3,"%getlabel(MODACTSNAME147,Export)","%getlabel(MODACTSDESC147,Export)",32,'Export_Org.gif')
,('ORGExportToPDF',3,"%getlabel(MODACTSNAME148,Export to PDF)","%getlabel(MODACTSDESC148,Export to PDF)",64,'ExportData.gif')
,('ORGFindPeople',3,"%getlabel(MODACTSNAME149,Find People)","%getlabel(MODACTSDESC149,Find People)",2048,'')
,('ORGMeasures',3,"%getlabel(MODACTSNAME150,Measures)","%getlabel(MODACTSDESC150,Measures)",1,'Add_Remove_Measures_Org.gif')
,('ORGModify',3,"%getlabel(MODACTSNAME151,Modify)","%getlabel(MODACTSDESC151,Modify)",128,'Modify_Struct_Org.gif')
,('ORGNew',3,"%getlabel(MODACTSNAME152,New)","%getlabel(MODACTSDESC152,New)",8,'')
,('ORGOptions',3,"%getlabel(MODACTSNAME153,Org Options)","%getlabel(MODACTSDESC153,Options)",512,'ViewProperties.gif')
,('ORGPresentation',3,"%getlabel(MODACTSNAME154,Presentation view)","%getlabel(MODACTSDESC154,Presentation view)",16,'Diagram_View_Org.gif')
,('ORGPrint',3,"%getlabel(MODACTSNAME155,Print)","%getlabel(MODACTSDESC155,Print)",16384,'')
,('ORGSaveAs',3,"%getlabel(MODACTSNAME156,Save As)","%getlabel(MODACTSDESC156,Save As)",8192,'')
,('ORGScorecard',3,"%getlabel(MODACTSNAME157,Scorecard)","%getlabel(MODACTSDESC157,Scorecard)",1024,'ScoreCard_Project.gif')
,('ADCubeMenu',10,"%getlabel(MODACTSNAME158,Cube Context Menu)","%getlabel(MODACTSDESC158,Cube Context Menu)", 2048,'CubeNew.gif' )
,('ADPlanApplyChanges', 8, "%getlabel(MODACTSNAME159,Planning - Apply Changes)", "%getlabel(MODACTSDESC159,Planning - Apply Changes)", 1024, 'SaveFile.gif')
,('ADPlanSASCode', 8, "%getlabel(MODACTSNAME160,Planning - SAS Code)", "%getlabel(MODACTSDESC160,Planning - SAS Code)", 1024, 'tablev.gif')
 ) by PASSTHRU;
/*************************/
/*      SAS_MODULES      */
/*************************/
 execute (Insert into SAS_MODULES (MODULE_ID,MODULE_NM,MODULE_DESC,SECURITY_CD,IS_CUSTOM,PARENT_MODULE) Values
(1,"%getlabel(MODULENAME1,Employee Profile)","%getlabel(MODULEDESC1,Employee Profile Module)",NULL,0,1)
,(2,"%getlabel(MODULENAME2,Geographical Analysis)","%getlabel(MODULEDESC2,Geographical Analysis Module)",NULL,0,2)
,(3,"%getlabel(MODULENAME3,Organization Analysis)","%getlabel(MODULEDESC3,Organization Analysis Module)",NULL,0,3)
,(4,"%getlabel(MODULENAME4,General Search)","%getlabel(MODULEDESC4,General Search Module)",NULL,0,4)
,(5,"%getlabel(MODULENAME5,Administrator)","%getlabel(MODULEDESC5,Administrator Module)",NULL,0,5)
,(6,"%getlabel(MODULENAME6,HCM Home)","%getlabel(MODULEDESC6,HCM Home Module)",NULL,0,6)
,(7,"%getlabel(MODULENAME7,Custom)","%getlabel(MODULEDESC7,Custom Module)",NULL,1,7)
,(8,"%getlabel(MODULENAME8,AdminDataCommon)","%getlabel(MODULEDESC8,Administrator Data Common)",NULL,0,5)
,(9,"%getlabel(MODULENAME9,AdminData)","%getlabel(MODULEDESC9,Admin Data)",NULL,0,5)
,(10,"%getlabel(MODULENAME10,AdminCubeIMAP)","%getlabel(MODULEDESC10,Admin Cube and IMAP)",NULL,0,5)
,(11,"%getlabel(MODULENAME11,AdminCustom)","%getlabel(MODULEDESC11,Administrator Customize)",NULL,0,5)
,(12,"%getlabel(MODULENAME12,Admin Security)","%getlabel(MODULEDESC12,Administrator Security Module)",NULL,0,5)
,(13,"%getlabel(MODULENAME13,AdminConfig)","%getlabel(MODULEDESC13,Administrator Configuration)",NULL,0,5)
 ) by PASSTHRU;
/*************************/
/*     SAS_TEMPLATES     */
/*************************/
 execute (Insert into SAS_TEMPLATES (TEMPLATE_ID,TEMPLATE_NM,TEMPLATE_DESC,FILE_NM,IS_DEFAULT,TEMPLATE_TYPE) Values
(1,"%getlabel(TEMPLATENAME1,Executive Profile)","%getlabel(TEMPLATEDESC1,Employee Profile for Executives)",'HCMDetailPhoto.jsp',0,'EMPPROFILE_TEMPLATE')
,(2,"%getlabel(TEMPLATENAME2,General Profile)","%getlabel(TEMPLATEDESC2,General Employee Profile)",'HCMProfileViewerContent.jsp',1,'EMPPROFILE_TEMPLATE')
,(3,"%getlabel(TEMPLATENAME3,Home Template)","%getlabel(TEMPLATEDESC3,Home Page Template)",'HcmHomeRight.jsp',1,'HOME_TEMPLATE')
,(4,"%getlabel(TEMPLATENAME4,Dashboard Template)","%getlabel(TEMPLATEDESC4,Dashboard View Template for Home)",'HCM_BID_Template.jsp',0,'HOME_TEMPLATE')
,(5,"%getlabel(TEMPLATENAME5,Employee Profile Template Section View)","%getlabel(TEMPLATEDESC5,Employee Profile Template Section View)",'HCMProfileTemplateSectionView.jsp',0,'EMPPROFILE_TEMPLATE')
,(6,"%getlabel(TEMPLATENAME6,Employee Profile Template Tab View)","%getlabel(TEMPLATEDESC6,Employee Profile Template Tab View)",'HCMProfileTemplateTabView.jsp',0,'EMPPROFILE_TEMPLATE')
 ) by PASSTHRU;
/*************************/
/*SAS_USER_MODULE_SETTINGS*/
/*************************/
 execute (Insert into SAS_USER_MODULE_SETTINGS (USERID,MODULETYPE,PROPERTYNAME,PROPERTYVALUE,PROPERTYDESCRIPTION,DOC_ID) Values
('sasdemo','Search','DISPLAYCOLUMNS','EMPLOYEE_NAME',"%getlabel(USERMOD_DISPLAYCOLUMNS,LIST OF COLUMNS TO BE DISPLAYED)",'')
,('sasdemo','Search','DISPLAYCOLUMNS','EMPLOYEE_ID',"%getlabel(USERMOD_DISPLAYCOLUMNS,LIST OF COLUMNS TO BE DISPLAYED)",'')
,('sasdemo','Search','DISPLAYCOLUMNS','ANNUAL_SALARY',"%getlabel(USERMOD_DISPLAYCOLUMNS,LIST OF COLUMNS TO BE DISPLAYED)",'')
,('sasdemo','Search','DISPLAYCOLUMNS','AGE',"%getlabel(USERMOD_DISPLAYCOLUMNS,LIST OF COLUMNS TO BE DISPLAYED)",'')
,('sasdemo','Search','DISPLAYCOLUMNS','MONTHLY_SALARY',"%getlabel(USERMOD_DISPLAYCOLUMNS,LIST OF COLUMNS TO BE DISPLAYED)",'')
,('sasdemo','Search','DISPLAYCOLUMNS','EMPLOYEE_STATUS_CD',"%getlabel(USERMOD_DISPLAYCOLUMNS,LIST OF COLUMNS TO BE DISPLAYED)",'')
,('sasdemo','Search','SELECTEDCATEGORY','SALHIST',"%getlabel(USERMOD_SELECTEDCATEGORY,CATEGORY TABLES TO BE DISPLAYED)",'')
,('sasdemo','Search','SELECTEDCATEGORY','ABSHMAST',"%getlabel(USERMOD_SELECTEDCATEGORY,CATEGORY TABLES TO BE DISPLAYED)",'')
,('sasdemo','Search','SELECTEDCATEGORY','EMPMAST',"%getlabel(USERMOD_SELECTEDCATEGORY,CATEGORY TABLES TO BE DISPLAYED)",'')
,('sasdemo','Search','DISPLAYTEMPLATE','TEMPLATE1',"%getlabel(USERMOD_DISPLAYTEMPLATE,DISPLAY TEMPLATE)",'')
,('sasdemo','Search','DISPLAYROWS','10',"%getlabel(USERMOD_DISPLAYTEMPLATE,DISPLAY TEMPLATE)",'')
 ) by PASSTHRU;
/*************************/
/*   SAS_USER_OPTIONS    */
/*************************/
 execute (Insert into SAS_USER_OPTIONS (USER_ID,MODULE_ID,OPTION_NM,OPTION_VALUE) Values
('SAS Demo User','4','enable_search_assist','1')
 ) by PASSTHRU;
/*************************/
/*   SAS_XACTIONPARAMS   */
/*************************/
 execute (Insert into SAS_XACTIONPARAMS (PARAM_ID,ACTION_ID,PARAM_NM,COLUMN_NM) Values
(21,13,'q','EMPMAST.ADDRESS_LINE_1_TXT'),
(22,13,'q','EMPMAST.ADDRESS_LINE_2_TXT'),
(23,13,'q','EMPMAST.CITY_NM'),
(24,13,'q','EMPMAST.STATE_REGION_CD'),
(25,14,'q1','EMPMAST.ADDRESS_LINE_1_TXT'),
(26,14,'q1','EMPMAST.ADDRESS_LINE_2_TXT'),
(27,14,'q1','EMPMAST.CITY_NM'),
(28,14,'q1','EMPMAST.STATE_REGION_CD')
 ) by PASSTHRU;
/*************************/
/*      SAS_XACTIONS     */
/*************************/
 execute (Insert into SAS_XACTIONS (ACTION_ID,MODULE_ID,PROFILE_KEY,ACTION_LABEL,ACTION_TYPE,ACTION_LINK,ACTION_EMP_LEVEL,ACTION_DEPT_LEVEL,ACTION_IN_USE) Values
(1,1,'EmployeeProfile',"%getlabel(XACTIONLABEL1,SAS - The Power to Know)",0,'http://www.sas.com',1,1,1)
,(2,2,'GeoAnalysis',"%getlabel(XACTIONLABEL2,SAS - The Power to Know)",0,'http://www.sas.com',0,0,1)
,(3,2,'GeoAnalysis',"%getlabel(XACTIONLABEL3,Workforce Percentage above 40)",4,'SBIP://METASERVER/Products/SAS Human Capital Management/5.1 Reports/Employee/Workforce Percentage Above 40(StoredProcess)',0,0,1)
,(4,1,'EmployeeProfile',"%getlabel(XACTIONLABEL4,Average Annual FTE Salary by Ethnicity)",2,'SBIP://METASERVER/Products/SAS Human Capital Management/5.1 Reports/Compensation/Average Annual FTE Salary by Ethnicity.srx(Report)',1,1,1)
,(5,1,'EmployeeProfile',"%getlabel(XACTIONLABEL5,Average Annual FTE Salary by Job Group)",2,'SBIP://METASERVER/Products/SAS Human Capital Management/5.1 Reports/Compensation/Average Annual FTE Salary by Job Group.srx(Report)',1,1,1)
,(6,1,'EmployeeProfile',"%getlabel(XACTIONLABEL6,Number of Employees by Service Years)",4,'SBIP://METASERVER/Products/SAS Human Capital Management/5.1 Reports/Employee/Number of Employees by Years of Service(StoredProcess)',1,1,1)
,(7,1,'EmployeeProfile',"%getlabel(XACTIONLABEL7,Time in Position)",1,'SBIP://METASERVER/Products/SAS Human Capital Management/Data Sources/Information Maps/TIP_MAP(InformationMap)',1,1,1)
,(13,1,'EmployeeProfile',"%getlabel(XACTIONLABEL13,Google Map)",0,'http://maps.google.com/maps?f=q&hl=en',1,1,1)
,(14,1,'EmployeeProfile',"%getlabel(XACTIONLABEL14,Yahoo Maps)",0,'http://maps.yahoo.com/?#mvt=m',1,1,1)
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;

/*************************/
/*  SAS_ROWLEVELSECURITY */
/*************************/
/*************************************************************/
/*Extract the 4.4 SolutionsSecurity Tree Names from metadata.*/
/*************************************************************/
Options metarepository=HR;
/*Build the XML metadata request.*/
filename request temp lrecl=1024;
  data _null_;
    file request;
    put "<GetMetadataObjects>";
    put "<ReposId>$METAREPOSITORY</ReposId>";
    put "<Type>Column</Type>";
    put "<Objects/>";
    put "<ns>SAS</ns>";
    put "<Flags>10628</Flags>";
    put "<Options>";
    put "<XMLSelect search=""%bquote(*[Table/PhysicalTable/TablePackage/DatabaseSchema/UsedByPackages/SASLibrary[@Name='HCMData']])""/>";
    put "<Templates>";
    put '<Column SASColumnName="" SASColumnType="">';
    put   '<Trees search="@TreeType=''SolutionsSecurity''"/>';
    put   "<Table/>";
    put "</Column>";
    put '<Tree Name=""/>';
    put '<PhysicalTable SASTableName=""/>';
    put "</Templates>";
    put "</Options>";
    put "</GetMetadataObjects>";
  run;

filename response temp lrecl=1024;
  proc metadata
    in=request
    out=response;
    run;
 %checkErr
  %if &_tmpRC %then %goto exit;

  /*  Build the XML Map file to parse the response. */
filename map temp;
  data _null_;
    file map encoding="utf-8";
    put '<?xml version="1.0" encoding="utf-8"?>';
    put '<SXLEMAP version="1.0">';
    put '<TABLE name="columns">';
    put "<TABLE_XPATH>//Column</TABLE_XPATH>";
    put '<COLUMN name="TABLE_NM">';
    put "<XPATH>//Column/Table/PhysicalTable@SASTableName</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>50</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="COLUMN_NM">';
    put "<XPATH>//Column@SASColumnName</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>50</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DATA_TYPE">';
    put "<XPATH>//Column@SASColumnType</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>1</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="TREENAME">';
    put "<XPATH>//Column/Trees/Tree@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>60</LENGTH>';
    put '</COLUMN>';
    put '</TABLE>';
    put '</SXLEMAP>';
  run;

  /*  Parse the response with the XML library engine and PROC SQL. */
libname response xml xmlmap=map;

%If %sysfunc(exist(response.columns)) %then %do;
Data columns;
 Set response.columns;
Run;
%End;
%Else %do;
Data columns;
 Length TABLE_NM $ 50 COLUMN_NM $ 50 DATA_TYPE $ 1 TREENAME $ 60;
 TABLE_NM='';
 COLUMN_NM='';
 DATA_TYPE='';
 TREENAME='';
Run;
%End;

/*  Cleanup. */
filename request;
filename response;
filename map;
libname  response;

Options metarepository=&metarepository.;

/*Extract the 4.4 data filters. */
Proc sql;
 Create table filters as
 Select u.*, c.table_nm, c.column_nm, c.data_type
 From _sdmlib.sas_userrolecellsecurity u left join columns c
 On  upcase(u.treename) = upcase(c.treename)
 and c.treename ne '';
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;

/*Keep only hcmdata tables and columns extracted from _sdmlib.
  Derive values for some fields for certain conditions.*/
Data filters;
 Set filters;
 Drop treename lib;
 Length FILTER_TYPE $1;
 ID=upcase(id);
 lib=kindex(upcase(treename),'HCMDATA');
 If table_nm ne '' or lib>0;
 If id='ANALYST' then id='HCM ANALYST';
 If table_nm = '' then do;
  table_nm = ksubstr(upcase(treename),lib+8); /*Extract table_nm from treename.*/
  filter_type='T';
  column_nm='*';
  data_type='*';
 End;
 Else filter_type='C';
Run;
 %checkErr
  %if &_tmpRC %then %goto exit;

/*Insert migrated data into sas_rowlevelsecurity.*/
/*Don't insert the migrated data if EDUHIST is already in the table.  If EDUHIST is already in the table, then the migrated filters had
  already been inserted in a previous run.*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='EDUHIST';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 Insert into &hcmlibref..sas_rowlevelsecurity (FILTER_TYPE, IDENTITY_TYPE, ACTOR_NM, TABLE_NM, COLUMN_NM, FILTER_OPERATOR, FILTER_VALUE, DATA_TYPE)
 Select FILTER_TYPE, FLAG, ID, TABLE_NM, COLUMN_NM, OPERATOR_CD, VALUE_TXT, DATA_TYPE
 From filters;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

/*Seed filters for new 5.1 tables.*/
%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='EDUHIST';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','EDUHIST','*','*','*','*'),
('T','R','HCM USER','EDUHIST','*','*','*','*'),
('T','R','HCM ANALYST','EDUHIST','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='EMPSCORES';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','EMPSCORES','*','*','*','*'),
('T','R','HCM USER','EMPSCORES','*','*','*','*'),
('T','R','HCM ANALYST','EMPSCORES','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='MODELSCORES';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','MODELSCORES','*','*','*','*'),
('T','R','HCM USER','MODELSCORES','*','*','*','*'),
('T','R','HCM ANALYST','MODELSCORES','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='EDUVAL';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','EDUVAL','*','*','*','*'),
('T','R','HCM USER','EDUVAL','*','*','*','*'),
('T','R','HCM ANALYST','EDUVAL','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='HCMMONTHFORECAST';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','HCMMONTHFORECAST','*','*','*','*'),
('T','R','HCM USER','HCMMONTHFORECAST','*','*','*','*'),
('T','R','HCM ANALYST','HCMMONTHFORECAST','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='HCMQTRFORECAST';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','HCMQTRFORECAST','*','*','*','*'),
('T','R','HCM USER','HCMQTRFORECAST','*','*','*','*'),
('T','R','HCM ANALYST','HCMQTRFORECAST','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;

%let nFoundARecord=;
proc sql noprint;
 select count(*) into :nFoundARecord
 from &hcmlibref..SAS_ROWLEVELSECURITY
 where upcase(TABLE_NM)='HCMYEARFORECAST';
quit;
%if &nFoundARecord EQ 0 %then %do;
Proc sql;
 connect to &sesetl_EngineType as PASSTHRU (&sesetl_BestLogin &sesetl_ConnectionOptions );
 execute (Insert into SAS_ROWLEVELSECURITY (FILTER_TYPE,IDENTITY_TYPE,ACTOR_NM,TABLE_NM,COLUMN_NM,FILTER_OPERATOR,FILTER_VALUE,DATA_TYPE) Values
('T','R','HCM ADMINISTRATOR','HCMYEARFORECAST','*','*','*','*'),
('T','R','HCM USER','HCMYEARFORECAST','*','*','*','*'),
('T','R','HCM ANALYST','HCMYEARFORECAST','*','*','*','*')
 ) by PASSTHRU;
 disconnect from PASSTHRU;
Quit;
 %checkErr
  %if &_tmpRC %then %goto exit;
%end;


/******************************************************************************************************
 * Add %hcmlibol to the precode for the 4.4 Cube jobs.                                                *
 ******************************************************************************************************/
filename request temp lrecl=1024;
Options metarepository=HR;
  data _null_;
    file request;
    put "<GetMetadataObjects>";
    put "<ReposId>$METAREPOSITORY</ReposId>";
    put "<Type>TextStore</Type>";
    put "<Objects/>";
    put "<ns>SAS</ns>";
    put "<Flags>2948</Flags>";
    put "<Options>";
    put "<XMLSelect search=""%bquote(*[AssociatedTransformation/Transformation/SourceTransformations/Job[@PublicType='Job']])""/>";
    put "<Templates>";
    put '<TextStore Id="" StoredText="">';
    put '<AssociatedTransformation/>';
    put "</TextStore>";
    put '<Transformation>';
    put '<SourceTransformations/>';
    put '</Transformation>';
    put '<Job Name=""/>';
    put "</Templates>";
    put "</Options>";
    put "</GetMetadataObjects>";
  run;

filename response temp lrecl=1024;
  proc metadata
    in=request
    out=response;
    run;

  /*  Build the XML Map file to parse the response. */
filename map temp;
  data _null_;
    file map encoding="utf-8";
    put '<?xml version="1.0" encoding="utf-8"?>';
    put '<SXLEMAP version="1.0">';
    put '<TABLE name="TABLERESP">';
    put "<TABLE_XPATH>//TextStore</TABLE_XPATH>";
    put '<COLUMN name="ID">';
    put "<XPATH>//TextStore@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="TEXT">';
    put "<XPATH>//TextStore@StoredText</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="JOB">';
    put "<XPATH>//Job@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '</TABLE>';
    put '</SXLEMAP>';
  run;

  /*  Parse the response with the XML library engine and PROC SQL. */
libname response xml xmlmap=map;

%local obs prebuildobs;
%let obs=0;
%let prebuildobs=0;
%If %sysfunc(exist(response.tableresp)) %then %do;
Data precode;
 Set response.tableresp;
 Where id ne '';
Run;

%Let dsnum=%sysfunc(open(precode));
%Let obs=%sysfunc(attrn(&dsnum,nlobs));
%Let rc=%sysfunc(close(&dsnum));
%If &obs.>0 %then %do;
Data _null_;
 Set precode end=last;
 Where kindex(upcase(text),'%PREBUILD')>0 and kindex(upcase(job),'CUBE')>0;
 If kindex(upcase(text),'%HCMLIBOL')=0 then do;
  length precode $ 256;
  precode='%hcmlibol; '||ktrim(text);
  rc=metadata_setattr(id, "StoredText", precode);
  If rc=0 then put 'NOTE: "%hcmlibol" has been added to the Precode on the HCM 4.4 Job "' job +(-1) '" in the HR repository.';
  If rc ne 0 then put 'WARNING: "%hcmlibol" could not be added to the Precode on the HCM 4.4 Job "' job +(-1) '" in the HR repository.';
 End;
 If last then call symput('prebuildobs',strip(put(_n_,4.)));
Run;
%End;
%End;

filename request;
filename response;
filename map;
libname  response;

%If &obs.<=0 or &prebuildobs.=0 %then
 %put WARNING: Unable to find HCM 4.4 Cube jobs in the HR repository on which to add %nrstr("%hcmlibol") to the Precode.;

/******************************************************************************************************
 * Migrate Org and Geo Anylyses from 4.4.                                                             *
 ******************************************************************************************************/
filename request temp lrecl=1024;
  data _null_;
    file request;
    put "<GetMetadataObjects>";
    put "<ReposId>$METAREPOSITORY</ReposId>";
    put "<Type>Property</Type>";
    put "<Objects/>";
    put "<ns>SAS</ns>";
    put "<Flags>2948</Flags>";
    put "<Options>";
    put "<XMLSelect search=""%bquote(*[AssociatedObject/Document[@TextType='Geographic']])""/>";
    put "<Templates>";
    put '<Property Id="" Name="" DefaultValue="">';
    put '<AssociatedObject/>';
    put "</Property>";
    put '<Document Id="" Name="" Desc="">';
    put '<ExternalIdentities/>';
    put "</Document>";
    put '<ExternalIdentity Id="" Name="" Identifier=""/>';
    put "</Templates>";
    put "</Options>";
    put "</GetMetadataObjects>";
  run;

filename response temp lrecl=1024;
  proc metadata
    in=request
    out=response;
    run;

  /*  Build the XML Map file to parse the response. */
filename map temp;
  data _null_;
    file map encoding="utf-8";
    put '<?xml version="1.0" encoding="utf-8"?>';
    put '<SXLEMAP version="1.0">';
    put '<TABLE name="GEORESP">';
    put "<TABLE_XPATH>//Property</TABLE_XPATH>";
    put '<COLUMN name="PROPID">';
    put "<XPATH>//Property@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="PROPNAME">';
    put "<XPATH>//Property@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="PROPVALUE">';
    put "<XPATH>//Property@DefaultValue</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DOCNAME">';
    put "<XPATH>//Document@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DOCDESC">';
    put "<XPATH>//Document@Desc</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DOCID">';
    put "<XPATH>//Document@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="EXTNAME">';
    put "<XPATH>//ExternalIdentity@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="EXTID">';
    put "<XPATH>//ExternalIdentity@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="EXTGUID">';
    put "<XPATH>//ExternalIdentity@Identifier</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '</TABLE>';
    put '</SXLEMAP>';
  run;

  /*  Parse the response with the XML library engine and PROC SQL. */
libname response xml xmlmap=map;

/*Get the list of Geo document properties from the xml map.*/
Data propgeo;
 Set response.georesp;
 where propid ne '';
Run;

/*Get a unique list of Geo Documents.*/
Proc sql;
 Create table geoid as
 Select distinct DOCID, EXTID, EXTGUID
 From propgeo;
quit;

/*Set the PublicType, TextType, and URI attributes for the GEO document.  Delete the associated GUID object.*/
Data _null_;
 Set geoid;
 rc=metadata_setattr(docid, "PublicType", "GeographicAnalysis");
 rc=metadata_setattr(docid, "TextType", "");
 rc=metadata_setattr(docid, "URI", extguid);
 rc=metadata_delobj(extid);
Run;

/*Delete the Geo document associated properties.*/
Data _null_;
 Set propgeo;
 rc=metadata_delobj(propid);
Run;

filename request;
filename response;
filename map;
libname  response;

filename request temp lrecl=1024;
  data _null_;
    file request;
    put "<GetMetadataObjects>";
    put "<ReposId>$METAREPOSITORY</ReposId>";
    put "<Type>Property</Type>";
    put "<Objects/>";
    put "<ns>SAS</ns>";
    put "<Flags>2948</Flags>";
    put "<Options>";
    put "<XMLSelect search=""%bquote(*[AssociatedObject/Document[@TextType='OrgChart']])""/>";
    put "<Templates>";
    put '<Property Id="" Name="" DefaultValue="">';
    put '<AssociatedObject/>';
    put "</Property>";
    put '<Document Id="" Name="" Desc="" MetadataUpdated="">';
    put '<ExternalIdentities/>';
    put "</Document>";
    put '<ExternalIdentity Id="" Name="" Identifier=""/>';
    put "</Templates>";
    put "</Options>";
    put "</GetMetadataObjects>";
  run;

filename response temp lrecl=1024;
  proc metadata
    in=request
    out=response;
    run;

  /*  Build the XML Map file to parse the response. */
filename map temp;
  data _null_;
    file map encoding="utf-8";
    put '<?xml version="1.0" encoding="utf-8"?>';
    put '<SXLEMAP version="1.0">';
    put '<TABLE name="ORGRESP">';
    put "<TABLE_XPATH>//Property</TABLE_XPATH>";
    put '<COLUMN name="PROPID">';
    put "<XPATH>//Property@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="PROPNAME">';
    put "<XPATH>//Property@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="PROPVALUE">';
    put "<XPATH>//Property@DefaultValue</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DOCNAME">';
    put "<XPATH>//Document@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DOCDESC">';
    put "<XPATH>//Document@Desc</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="DOCID">';
    put "<XPATH>//Document@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="METADATE">';
    put "<XPATH>//Document@MetadataUpdated</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>18</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="EXTNAME">';
    put "<XPATH>//ExternalIdentity@Name</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="EXTID">';
    put "<XPATH>//ExternalIdentity@Id</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>17</LENGTH>';
    put '</COLUMN>';
    put '<COLUMN name="EXTGUID">';
    put "<XPATH>//ExternalIdentity@Identifier</XPATH>";
    put '<TYPE>character</TYPE>';
    put '<DATATYPE>STRING</DATATYPE>';
    put '<LENGTH>256</LENGTH>';
    put '</COLUMN>';
    put '</TABLE>';
    put '</SXLEMAP>';
  run;

  /*  Parse the response with the XML library engine and PROC SQL. */
libname response xml xmlmap=map;

/*Get the list of Org Document properties from the xml map.*/
Data proporg;
 Set response.orgresp;
 where propid ne '';
Run;

/*Get a unique list of Org Documents.*/
Proc sql;
 Create table orgid as
 Select distinct DOCID, EXTID, EXTGUID
 From proporg;
quit;

/*Set the PublicType, TextType, and URI attributes for the Org document.  Delete the associated GUID object.*/
Data _null_;
 Set orgid;
 rc=metadata_setattr(docid, "PublicType", "OrganizationAnalysis");
 rc=metadata_setattr(docid, "TextType", "");
 rc=metadata_setattr(docid, "URI", extguid);
 rc=metadata_delobj(extid);
Run;

/*Delete the Org document associated properties.*/
Data _null_;
 Set proporg;
 rc=metadata_delobj(propid);
Run;

filename request;
filename response;
filename map;
libname  response;


/*Generate the Geo Properties XML, which will be loaded to sas_doc_properties.*/
Data geoxml;
 Set propgeo;
 By docid notsorted;
 Retain propxml;
 Length field $ 100 propxml $ 2000;
 If first.docid then do;
  propxml="<?xml version='1.0' encoding='UTF-8'?><Document modultype='Geo' docid='"||trim(extguid)||
"'><folder-location/><document-name>"||trim(docname)||"</document-name>"||
"<start-level>1</start-level><end-level>3</end-level><active>true</active><in-active>false</in-active><start-color>#ADBFEE</start-color><end-color>#515CC9</end-color>"||
"<searchable-columns><column>ANNUAL_SALARY</column><column>HIRE_DT</column><column>AGE</column><column>SERVICE_YEARS</column></searchable-columns>";
  If docdesc='' then propxml=trim(propxml)||"<document-description/>";
  Else propxml=trim(propxml)||"<document-description>"||trim(docdesc)||"</document-description>";
 End;
  If propname='geoTable' then do;
   If scan(propvalue,2,'.') ne '' then propvalue=scan(propvalue,2,'.');
   propxml=trim(propxml)||"<map-reference-table>"||trim(propvalue)||"</map-reference-table><response-table>"||trim(propvalue)||"</response-table>";
  End;
  Else If propname='legendLabel' then propxml=trim(propxml)||"<legend-label>"||trim(propvalue)||"</legend-label>";
  Else If propname='analysisFields' then do;
   propxml=trim(propxml)||"<selected-analysis>";
   propvalue=tranwrd(propvalue,'-','::');
   i=1;
   If propvalue ne '' then do until(field='');
    field=scan(propvalue,i,'~');
    i=i+1;
    If field ne '' and trim(scan(field,2,'::')) ne 'RANGE'
     then propxml=trim(propxml)||"<analysis-column>"||trim(field)||"</analysis-column>";
   End;
   propxml=trim(propxml)||"</selected-analysis>";
  End;
 If last.docid then do;
  propxml=trim(propxml)||"</Document>";
  output;
 End;
Run;


/*Generate the Org Properties XML, which will be loaded to sas_doc_properties.*/
Data orgxml;
 Set proporg;
 By docid notsorted;
 Retain propxml;
 Length field $ 100 propxml $ 2000;
 If first.docid then do;
  propxml="<?xml version='1.0' encoding='UTF-8'?><Document type='Organization Analysis' docid='"||trim(extguid)||
"' docName='"||trim(docname)||"' docDesc=''><groupingColumn/><groupingSelection/><isActiveEmployee>true</isActiveEmployee>"||
"<isInActiveEmployee>false</isInActiveEmployee><isEmployeeNodeChecked>true</isEmployeeNodeChecked><isDepartmentNodeChecked>true</isDepartmentNodeChecked>"||
"<rollOverField><column name='EMPLOYEE_NAME' format='0'/></rollOverField><displayFormat>oneline</displayFormat><orgChartInfo/><orgChartVersion/>"||
"<updateDate>"||trim(metadate)||"</updateDate>";
 End;
  If propname='HierarchyCode' then propxml=trim(propxml)||"<hierarchyCode>"||trim(propvalue)||"</hierarchyCode><hierarchyColumn>"||trim(propvalue)||"</hierarchyColumn>";
  Else If propname='IsStatic' then propxml=trim(propxml)||"<isSatic>"||trim(propvalue)||"</isSatic>";
  Else If propname='AnalysisColumns' then do;
   propxml=trim(propxml)||"<analysisColumn>";
   i=1;
   If propvalue ne '' then do until(field='');
    field=scan(propvalue,i,' ');
    i=i+1;
    If field ne '' then propxml=trim(propxml)||"<column>"||trim(field)||"</column>";
   End;
   propxml=trim(propxml)||"</analysisColumn>";
  End;
  Else If propname='DataXML' then propxml=trim(propxml)||"<dataXml>"||trim(propvalue)||"</dataXml>";
  Else If propname='Measures' then do;
   propxml=trim(propxml)||"<measures>";
   i=1;
   If propvalue ne '' then do until(field='');
    field=scan(propvalue,i,' ');
    i=i+1;
    If field ne '' then do;
     If field='HCNT' then propxml=trim(propxml)||"<column name='HCNT' stat='HCNT'/>";
     else propxml=trim(propxml)||"<column name='"||trim(scan(field,2,'!'))||"' stat='"||scan(field,1,'!')||"'/>";
    End;
   End;
   propxml=trim(propxml)||"</measures>";
  End;
  Else If propname='MeasureSource' then propxml=trim(propxml)||"<measureSource>"||trim(propvalue)||"</measureSource>";
  Else If propname='IsModified' then propxml=trim(propxml)||"<isModified>"||trim(propvalue)||"</isModified>";
  Else If propname='NodeAttributes' then do;
   propxml=trim(propxml)||"<nodeAttribute>";
   i=1;
   If propvalue ne '' then do until(field='');
    field=scan(propvalue,i,' ');
    i=i+1;
    If field ne '' then propxml=trim(propxml)||"<column name='"||trim(field)||"' format='0'/>";
   End;
   propxml=trim(propxml)||"</nodeAttribute>";
  End;
  Else If propname='ViewXML' then propxml=trim(propxml)||"<viewXml>"||trim(propvalue)||"</viewXml>";
  Else If propname='ScoreCardProject' then do;
   If propvalue='' then propxml=trim(propxml)||"<scoreCardProject/>";
   Else propxml=trim(propxml)||"<scoreCardProject>"||trim(propvalue)||"</scoreCardProject>";
  End;
  Else If propname='isSameHierarchy' then do;
   If propvalue='' then propxml=trim(propxml)||"<isSameHierarchy/>";
   Else propxml=trim(propxml)||"<isSameHierarchy>"||trim(propvalue)||"</isSameHierarchy>";
  End;
 If last.docid then do;
  propxml=trim(propxml)||"</Document>";
  output;
 End;
Run;


Data docprop;
 Set geoxml orgxml;
run;

/*Insert migrated Org/Geo Analyses into sas_doc_properties.*/
Proc sql;
 Insert into &hcmlibref..sas_doc_properties (doc_id, doc_nm, doc_desc, created_by, property_xml)
 Select extguid, docname, docdesc, 'sasdemo', propxml
 From docprop;
Quit;

Options metarepository=&metarepository.;
/*Register HCMData Tables in OMR in the 5.1 folder location.*/
%reghcmdb(libref=&hcmlibref., addHonorTableSec=Y, HonorTableSecVal=Y, ModifyHonorTableSec=N)
%checkErr
%if &_tmpRC %then %goto exit;

/*Register Conform Tables in OMR.*/
%pmetalib(  sesetl_omrSASLibraryObjName=%nrbquote(&Soldatatier_folder./&CONFLibObj./&CONFLibObj.)
          , sesetl_folder=%nrbquote(&Soldatatier_folder./&CONFLibObj.)
          , sesetl_noexec=
          , sesetl_SASLibraryKeyword=
          , sesetl_folderKeyword=
          , sesetl_update_rule=DELETE
         )

/******************************************************************************/

/*Set status of HCM data migration to "Successful".*/
%sesshver(solName=HCM, solVer=&sesetl_version., solStatus=Successful, solMethod=Migration)

%exit:
%mend;

/*SAS_HCM_FORMATS*/
/*This section of code is run outside of the macro because the Cards statement causes errors when run in the macro.*/
/* msa format.  This fomat is built using government supplied */
/* MSA codes, included below in the program.   */
data msa;
  input state $3. +1 msa $65. +1 start $4.;
  retain type 'C';
  fmtname='MSAMSA   ';
  label=msa;
  fmtend=start;
  output;
  fmtname='MSASTATE';
  label=state;
  output;
  cards;
TX  Abilene, TX MSA                                                   0040
GA  Albany, GA MSA                                                    0120
NY  Albany-Schenectady-Troy, NY MSA                                   0160
NM  Albuquerque, NM MSA                                               0200
LA  Alexandria, LA MSA                                                0220
PA  Allentown-Bethlehem-Easton, PA-NJ MSA                             0240
PA  Altoona, PA MSA                                                   0280
TX  Amarillo, TX MSA                                                  0320
AK  Anchorage, AK MSA                                                 0380
IN  Anderson, IN MSA                                                  0400
SC  Anderson, SC MSA                                                  0405
AL  Anniston, AL MSA                                                  0450
WI  Appleton-Oshkosh-Neenah, WI MSA                                   0460
NC  Asheville, NC MSA                                                 0480
GA  Athens, GA MSA                                                    0500
GA  Atlanta, GA MSA                                                   0520
NJ  Atlantic City, NJ MSA                                             0560
GA  Augusta, GA-SC MSA                                                0600
TX  Austin, TX MSA                                                    0640
CA  Bakersfield, CA MSA                                               0680
MD  Baltimore, MD MSA                                                 0720
ME  Bangor, ME MSA                                                    0730
LA  Baton Rouge, LA MSA                                               0760
MI  Battle Creek, MI MSA                                              0780
TX  Beaumont-Port Arthur, TX MSA                                      0840
WA  Bellingham, WA MSA                                                0860
MI  Benton Harbor, MI MSA                                             0870
MT  Billings, MT MSA                                                  0880
MS  Biloxi-Gulfport, MS MSA                                           0920
NY  Binghamton, NY MSA                                                0960
AL  Birmingham, AL MSA                                                1000
ND  Bismarck, ND MSA                                                  1010
IN  Bloomington, IN MSA                                               1020
IL  Bloomington-Normal, IL MSA                                        1040
ID  Boise City, ID MSA                                                1080
MA  Boston-Lawrence-Salem, MA-NH CMSA                                 1122
FL  Bradenton, FL MSA                                                 1140
WA  Bremerton, WA MSA                                                 1150
TX  Brownsville-Harlingen, TX MSA                                     1240
TX  Bryan-College Station, TX MSA                                     1260
NY  Buffalo-Niagara Falls, NY CMSA                                    1282
NC  Burlington, NC MSA                                                1300
VT  Burlington, VT MSA                                                1305
OH  Canton, OH MSA                                                    1320
WY  Casper, WY MSA                                                    1350
IA  Cedar Rapids, IA MSA                                              1360
IL  Champaign-Urbana-Rantoul, IL MSA                                  1400
SC  Charleston, SC MSA                                                1440
WV  Charleston, WV MSA                                                1480
NC  Charlotte-Gastonia-Rock Hill, NC-SC MSA                           1520
VA  Charlottesville, VA MSA                                           1540
TN  Chattanooga, TN-GA MSA                                            1560
WY  Cheyenne, WY MSA                                                  1580
IL  Chicago-Gary-Lake County, IL-IN-WI CMSA                           1602
CA  Chico, CA MSA                                                     1620
OH  Cincinnati-Hamilton, OH-KY-IN CMSA                                1642
TN  Clarksville-Hopkinsville, TN-KY MSA                               1660
OH  Cleveland-Akron-Lorain, OH CMSA                                   1692
CO  Colorado Springs, CO MSA                                          1720
MO  Columbia, MO MSA                                                  1740
SC  Columbia, SC MSA                                                  1760
GA  Columbus, GA-AL MSA                                               1800
OH  Columbus, OH MSA                                                  1840
TX  Corpus Christi, TX MSA                                            1880
MD  Cumberland, MD-WV MSA                                             1900
TX  Dallas-Fort Worth, TX CMSA                                        1922
VA  Danville, VA MSA                                                  1950
IA  Davenport-Rock Island-Moline, IA-IL MSA                           1960
OH  Dayton-Springfield, OH MSA                                        2000
FL  Daytona Beach, FL MSA                                             2020
AL  Decatur, AL MSA                                                   2030
IL  Decatur, IL MSA                                                   2040
CO  Denver-Boulder, CO CMSA                                           2082
IA  Des Moines, IA MSA                                                2120
MI  Detroit-Ann Arbor, MI CMSA                                        2162
AL  Dothan, AL MSA                                                    2180
IA  Dubuque, IA MSA                                                   2200
MN  Duluth, MN-WI MSA                                                 2240
WI  Eau Claire, WI MSA                                                2290
TX  El Paso, TX MSA                                                   2320
IN  Elkhart-Goshen, IN MSA                                            2330
NY  Elmira, NY MSA                                                    2335
OK  Enid, OK MSA                                                      2340
PA  Erie, PA MSA                                                      2360
OR  Eugene-Springfield, OR MSA                                        2400
IN  Evansville, IN-KY MSA                                             2440
ND  Fargo-Moorhead, ND-MN MSA                                         2520
NC  Fayetteville, NC MSA                                              2560
AR  Fayetteville-Springdale, AR MSA                                   2580
MA  Fitchburg-Leominster, MA MSA                                      2600
MI  Flint, MI MSA                                                     2640
AL  Florence, AL MSA                                                  2650
SC  Florence, SC MSA                                                  2655
CO  Fort Collins-Loveland, CO MSA                                     2670
FL  Fort Myers-Cape CoraL, Fl MSA                                     2700
FL  Fort Pierce, FL MSA                                               2710
AR  Fort Smith, AR-OK MSA                                             2720
FL  Fort Walton Beach, FL MSA                                         2750
IN  Fort Wayne, IN MSA                                                2760
CA  Fresno, CA MSA                                                    2840
AL  Gadsden, AL MSA                                                   2880
FL  Gainesville, FL MSA                                               2900
NY  Glens Falls, NY MSA                                               2975
ND  Grand Forks, ND MSA                                               2985
MI  Grand Rapids, MI MSA                                              3000
MT  Great Falls, MT MSA                                               3040
CO  Greeley, CO MSA                                                   3060
WI  Green Bay, WI MSA                                                 3080
NC  Greensboro-Winston-Salem-High Point, NC MSA                       3120
SC  Greenville-Spartanburg, SC MSA                                    3160
MD  Hagerstown, MD MSA                                                3180
PA  Harrisburg-Lebanon-Carlisle, PA MSA                               3240
CT  Hartford-New Britain-Middletown, CT CMSA                          3282
NC  Hickory-Morganton, NC MSA                                         3290
HI  Honolulu, HI MSA                                                  3320
LA  Houma-Thibodaux, LA MSA                                           3350
TX  Houston-Galveston-Brazoria, TX CMSA                               3362
WV  Huntington-Ashland, WV-KY-OH MSA                                  3400
AL  Huntsville, AL MSA                                                3440
IN  Indianapolis, IN MSA                                              3480
IA  Iowa City, IA MSA                                                 3500
MI  Jackson, MI MSA                                                   3520
MS  Jackson, MS MSA                                                   3560
TN  Jackson, TN MSA                                                   3580
FL  Jacksonville, FL MSA                                              3600
NC  Jacksonville, NC MSA                                              3605
NY  Jamestown-Dunkirk, NY                                             3610
WI  Janesville-Beloit, WI MSA                                         3620
TN  Johnson City-Kingsport-Bristol, TN-VA MSA                         3660
PA  Johnstown, PA MSA                                                 3680
MO  Joplin, MO MSA                                                    3710
MI  Kalamazoo, MI MSA                                                 3720
IL  Kankakee, IL MSA                                                  3740
MO  Kansas City, MO-KS MSA                                            3760
TX  Killeen-Temple, TX MSA                                            3810
TN  Knoxville, TN MSA                                                 3840
IN  Kokomo, IN MSA                                                    3850
WI  La Crosse, WI MSA                                                 3870
LA  Lafayette, LA MSA                                                 3880
IN  Lafayette-West Lafayette, IN MSA                                  3920
LA  Lake Charles, LA MSA                                              3960
FL  Lakeland-Winter Haven, FL MSA                                     3980
PA  Lancaster, PA MSA                                                 4000
MI  Lansing-East Lansing, MI MSA                                      4040
TX  Laredo, TX MSA                                                    4080
NM  Las Cruces, NM MSA                                                4100
NV  Las Vegas, NV MSA                                                 4120
KS  Lawrence, KS MSA                                                  4150
OK  Lawton, OK MSA                                                    4200
ME  Lewiston-Auburn, ME MSA                                           4240
KY  Lexington-Fayette, KY MSA                                         4280
OH  Lima, OH MSA                                                      4320
NE  Lincoln, NE MSA                                                   4360
AR  Little Rock-North Little Rock, AR MSA                             4400
TX  Longview-Marshall, TX MSA                                         4420
CA  Los Angeles-Anaheim-Riverside, CA CMSA                            4472
KY  Louisville, KY-IN MSA                                             4520
TX  Lubbock, TX MSA                                                   4600
VA  Lynchburg, VA MSA                                                 4640
GA  Macon-Warner Robins, GA MSA                                       4680
WI  Madison, WI MSA                                                   4720
NH  Manchester, NH MSA                                                4760
OH  Mansfield, OH MSA                                                 4800
TX  Mcallen-Edinburg-Mission, TX MSA                                  4880
OR  Medford, OR MSA                                                   4890
FL  Melbourne-Titusville-Palm Bay, FL MSA                             4900
TN  Memphis, TN-AR-MS MSA                                             4920
CA  Merced, CA MSA                                                    4940
FL  Miami-Fort Lauderdale, FL CMSA                                    4992
TX  Midland, TX MSA                                                   5040
WI  Milwaukee-Racine, WI CMSA                                         5082
MN  Minneapolis-St. Paul, MN-WI MSA                                   5120
AL  Mobile, AL MSA                                                    5160
CA  Modesto, CA MSA                                                   5170
LA  Monroe, LA MSA                                                    5200
AL  Montgomery, AL MSA                                                5240
IN  Muncie, IN MSA                                                    5280
MI  Muskegon, MI MSA                                                  5320
FL  Naples, FL MSA                                                    5345
TN  Nashville, TN MSA                                                 5360
MA  New Bedford, MA MSA                                               5400
CT  New Haven-Meriden, CT MSA                                         5480
CT  New London-Norwich, CT-RI MSA                                     5520
LA  New Orleans, LA MSA                                               5560
NY  New York-Northern New Jersey-Long Island, NY-NJ-CT CMSA           5602
VA  Norfolk-Virginia Beach-Newport News, VA MSA                       5720
FL  Ocala, FL MSA                                                     5790
TX  Odessa, TX MSA                                                    5800
OK  Oklahoma City, OK MSA                                             5880
WA  Olympia, WA MSA                                                   5910
NE  Omaha, NE-IA MSA                                                  5920
FL  Orlando, FL MSA                                                   5960
KY  Owensboro, KY MSA                                                 5990
FL  Panama City, FL MSA                                               6015
WV  Parkersburg-Marietta, WV-OH MSA                                   6020
MS  Pascagoula, MS MSA                                                6025
FL  Pensacola, FL MSA                                                 6080
IL  Peoria, IL MSA                                                    6120
PA  Philadelphia-Wilmington-Trenton, PA-NJ-DE-MD CMSA                 6162
AZ  Phoenix, AZ MSA                                                   6200
AR  Pine Bluff, AR MSA                                                6240
PA  Pittsburgh-Beaver Valley, PA CMSA                                 6282
MA  Pittsfield, MA MSA                                                6320
ME  Portland, ME MSA                                                  6400
OR  Portland-Vancouver, OR-WA CMSA                                    6442
NH  Portsmouth-Dover-Rochester, NH-ME MSA                             6450
NY  Poughkeepsie, NY MSA                                              6460
RI  Providence-Pawtucket-Fall River, RI-MA CMSA                       6482
UT  Provo-Orem, UT MSA                                                6520
CO  Pueblo, CO MSA                                                    6560
NC  Raleigh-Durham, NC MSA                                            6640
SD  Rapid City, SD MSA                                                6660
PA  Reading, PA MSA                                                   6680
CA  Redding, CA MSA                                                   6690
NV  Reno, NV MSA                                                      6720
WA  Richland-Kennewick-Pasco, WA MSA                                  6740
VA  Richmond-Petersburg, VA MSA                                       6760
VA  Roanoke, VA MSA                                                   6800
MN  Rochester, MN MSA                                                 6820
NY  Rochester, NY MSA                                                 6840
IL  Rockford, IL MSA                                                  6880
CA  Sacramento, CA MSA                                                6920
MI  Saginaw-Bay City-Midland, MI MSA                                  6960
OR  Salem, OR MSA                                                     7080
CA  Salinas-Seaside-Monterey, CA MSA                                  7120
UT  Salt Lake City-Ogden, UT MSA                                      7160
TX  San Angelo, TX MSA                                                7200
TX  San Antonio, TX MSA                                               7240
CA  San Diego, CA MSA                                                 7320
CA  San Francisco-Oakland-San Jose, CA CMSA                           7362
CA  Santa Barbara-Santa Maria-Lompoc, CA MSA                          7480
NM  Santa Fe, NM MSA                                                  7490
FL  Sarasota, FL MSA                                                  7510
GA  Savannah, GA MSA                                                  7520
PA  Scranton-Wilkes-Barre, PA MSA                                     7560
WA  Seattle-Tacoma, WA CMSA                                           7602
PA  Sharon, PA MSA                                                    7610
WI  Sheboygan, WI MSA                                                 7620
TX  Sherman-Denison, TX MSA                                           7640
LA  Shreveport, LA MSA                                                7680
IA  Sioux City, IA-NE MSA                                             7720
SD  Sioux Falls, SD MSA                                               7760
IN  South Bend-Mishawaka, IN MSA                                      7800
WA  Spokane, WA MSA                                                   7840
IL  Springfield, IL MSA                                               7880
MA  Springfield, MA MSA                                               8000
MO  Springfield, MO MSA                                               7920
MN  St. Cloud, MN MSA                                                 6980
MO  St. Joseph, MO MSA                                                7000
MO  St. Louis, MO-IL MSA                                              7040
PA  State College, PA MSA                                             8050
OH  Steubenville-Weirton, OH-WV MSA                                   8080
CA  Stockton, CA MSA                                                  8120
NY  Syracuse, NY MSA                                                  8160
FL  Tallahassee, FL MSA                                               8240
FL  Tampa-St. Petersburg-Clearwater, FL MSA                           8280
IN  Terre Haute, IN MSA                                               8320
TX  Texarkana, TX-Texarkana, AR MSA                                   8360
OH  Toledo, OH MSA                                                    8400
KS  Topeka, KS MSA                                                    8440
AZ  Tucson, AZ MSA                                                    8520
OK  Tulsa, OK MSA                                                     8560
AL  Tuscaloosa, AL MSA                                                8600
TX  Tyler, TX MSA                                                     8640
NY  Utica-Rome, NY MSA                                                8680
TX  Victoria, TX MSA                                                  8750
CA  Visalia-Tulare-Porterville, CA MSA                                8780
TX  Waco, TX MSA                                                      8800
DC  Washington, DC-MD-VA MSA                                          8840
CT  Waterbury, CT MSA                                                 8880
IA  Waterloo-Cedar Falls, IA MSA                                      8920
WI  Wausau, WI MSA                                                    8940
FL  West Palm Beach-Boca Raton-Delray Beach, FL MSA                   8960
WV  Wheeling, WV-OH MSA                                               9000
TX  Wichita Falls, TX MSA                                             9080
KS  Wichita, KS MSA                                                   9040
PA  Williamsport, PA MSA                                              9140
NC  Wilmington, NC MSA                                                9200
MA  Worcester, MA MSA                                                 9240
WA  Yakima, WA MSA                                                    9260
PA  York, PA MSA                                                      9280
OH  Youngstown-Warren, OH MSA                                         9320
CA  Yuba City, CA MSA                                                 9340
AZ  Yuma, AZ MSA                                                      9360
;

Proc sort data=msa;
 by fmtname start;
Run;
